//*****************************************************************************
//*
//*
//*		CalcGenerator.cpp
//*
//*
//*****************************************************************************
#include	<math.h>
#include	<windows.h>
#include	"CalcGenerator.h"
#include	"StringFunctions.h"
#include	"BitFunctions.h"


#include <malloc.h>

static int	CalcGen(const char *pText,double *pValue,int *pOk,int iLevel,CalcGenData *pData);


#define		ENTRIES(a)		(sizeof(a)/sizeof((a)[0]))
#define 	ACT_LOGGING		0
#define 	CALC_PI			3.1415926535897932384626433832795
#define		SET_NAN(a)		((unsigned short*)(&(a)))[3]=0x7FFF
#define		IS_NAN(a)		((((unsigned*)(&(a)))[1]&0x7FFF0000)>=0x7FF00000)

#define 	ACT_END			0x0000
#define 	ACT_ERROR		0x0001
#define 	ACT_UP			0x0002
#define 	ACT_NEG			0x0003
#define 	ACT_NOT			0x0004
#define 	ACT_LOGIC_NOT	0x0005
#define 	ACT_ACOS		0x0006
#define 	ACT_ASIN		0x0007
#define 	ACT_ATAN		0x0008
#define 	ACT_ACOT		0x0009
#define 	ACT_ACOSH		0x000A
#define 	ACT_ASINH		0x000B
#define 	ACT_ATANH		0x000C
#define 	ACT_ACOTH		0x000D
#define 	ACT_COS			0x000E
#define 	ACT_SIN			0x000F
#define 	ACT_TAN			0x0010
#define 	ACT_COT			0x0011
#define 	ACT_COSH		0x0012
#define 	ACT_SINH		0x0013
#define 	ACT_TANH		0x0014
#define 	ACT_COTH		0x0015
#define 	ACT_LOG			0x0016
#define 	ACT_LOGX		0x0017
#define 	ACT_LOG10		0x0018
#define 	ACT_ROOT		0x0019
#define 	ACT_SQRT		0x001A
#define 	ACT_EQ			0x001B
#define 	ACT_NEQ			0x001C
#define 	ACT_LOWER		0x001D
#define 	ACT_LOWER_EQ	0x001E
#define 	ACT_UPPER		0x001F
#define 	ACT_UPPER_EQ	0x0020
#define 	ACT_SHR			0x0021
#define 	ACT_SHL			0x0022
#define 	ACT_LOGIC_OR	0x0023
#define 	ACT_LOGIC_AND	0x0024
#define 	ACT_OR			0x0025
#define 	ACT_AND			0x0026
#define 	ACT_XOR			0x0027
#define 	ACT_ADD			0x0028
#define 	ACT_SUB			0x0029
#define 	ACT_MUL			0x002A
#define 	ACT_DIV			0x002B
#define 	ACT_MOD			0x002C
#define 	ACT_POW			0x002D
#define 	ACT_POW2		0x002E
#define 	ACT_POW3		0x002F
#define 	ACT_ABS			0x0030
#define 	ACT_FLOOR		0x0031
#define 	ACT_ROUND		0x0032
#define 	ACT_CEIL		0x0033
#define 	ACT_CMP			0x1000
#define 	ACT_STOP		0x2000
#define 	ACT_GOTO		0x4000
#define 	ACT_GETVAR		0x8000
#define 	ACT_MASK		0x0FFF

#define 	ActionEntry(a)	a,#a

typedef struct
	{
	CalcAction	uAction;
	const char *pName;
	int			iDelta;
	int			iLevel;
	}CalcActionEntry;

static CalcActionEntry		aActNames[] =
	{
	ActionEntry(ACT_END			), 0, 0,
	ActionEntry(ACT_UP			), 1, 0,
	ActionEntry(ACT_NEG			), 0, 1,
	ActionEntry(ACT_NOT			), 0, 1,
	ActionEntry(ACT_LOGIC_NOT	), 0, 1,
	ActionEntry(ACT_ACOS		), 0, 1,
	ActionEntry(ACT_ASIN		), 0, 1,
	ActionEntry(ACT_ATAN		), 0, 1,
	ActionEntry(ACT_ACOT		), 0, 1,
	ActionEntry(ACT_ACOSH		), 0, 1,
	ActionEntry(ACT_ASINH		), 0, 1,
	ActionEntry(ACT_ATANH		), 0, 1,
	ActionEntry(ACT_ACOTH		), 0, 1,
	ActionEntry(ACT_COS			), 0, 1,
	ActionEntry(ACT_SIN			), 0, 1,
	ActionEntry(ACT_TAN			), 0, 1,
	ActionEntry(ACT_COT			), 0, 1,
	ActionEntry(ACT_COSH		), 0, 1,
	ActionEntry(ACT_SINH		), 0, 1,
	ActionEntry(ACT_TANH		), 0, 1,
	ActionEntry(ACT_COTH		), 0, 1,
	ActionEntry(ACT_LOG			), 0, 1,
	ActionEntry(ACT_LOGX		),-1, 2,
	ActionEntry(ACT_LOG10		), 0, 1,
	ActionEntry(ACT_ROOT		),-1, 2,
	ActionEntry(ACT_SQRT		), 0, 1,
	ActionEntry(ACT_EQ			),-1, 2,
	ActionEntry(ACT_NEQ			),-1, 2,
	ActionEntry(ACT_LOWER		),-1, 2,
	ActionEntry(ACT_LOWER_EQ	),-1, 2,
	ActionEntry(ACT_UPPER		),-1, 2,
	ActionEntry(ACT_UPPER_EQ	),-1, 2,
	ActionEntry(ACT_SHR			),-1, 2,
	ActionEntry(ACT_SHL			),-1, 2,
	ActionEntry(ACT_LOGIC_OR	),-1, 2,
	ActionEntry(ACT_LOGIC_AND	),-1, 2,
	ActionEntry(ACT_OR			),-1, 2,
	ActionEntry(ACT_AND			),-1, 2,
	ActionEntry(ACT_XOR			),-1, 2,
	ActionEntry(ACT_ADD			),-1, 2,
	ActionEntry(ACT_SUB			),-1, 2,
	ActionEntry(ACT_MUL			),-1, 2,
	ActionEntry(ACT_DIV			),-1, 2,
	ActionEntry(ACT_MOD			),-1, 2,
	ActionEntry(ACT_POW			),-1, 2,
	ActionEntry(ACT_POW2		), 0, 1,
	ActionEntry(ACT_POW3		), 0, 1,
	ActionEntry(ACT_ABS			), 0, 1,
	};

static int			iCalcInit=0;
static int			aDeltaList[ENTRIES(aActNames)];
static int			aLevelList[ENTRIES(aActNames)];
static const char  *aNameList [ENTRIES(aActNames)];
static double	CalcExec(CalcAction *pList,unsigned uMax,double *pVars,int *pOk);


static inline double cot(double dValue)
{
return tan(CALC_PI*0.5-dValue);
}

static inline double coth(double dValue)
{
return 1.0+2.0/(exp(2.0*dValue)-1.0);				
}

static inline double asinh(double dValue)
{
return log(dValue+sqrt(dValue*dValue+1.0));				
}

static inline double acosh(double dValue)
{
return log(dValue+sqrt(dValue*dValue-1.0));				
}

static inline double atanh(double dValue)
{
return log((1.0+dValue)/(1.0-dValue))*0.5;				
}

static inline double acoth(double dValue)
{
return log((dValue+1.0)/(dValue-1.0))*0.5;				
}


#if	ACT_LOGGING
extern void	__stdcall ShowConsole();
#endif

//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
CalcGenData::CalcGenData()
{
int		iPos;


	if(!iCalcInit)
		{
		iCalcInit = 1;

		for(iPos=0;iPos<ENTRIES(aActNames);iPos++)
			{
			aDeltaList[aActNames[iPos].uAction] = aActNames[iPos].iDelta;
			aLevelList[aActNames[iPos].uAction] = aActNames[iPos].iLevel;
			aNameList [aActNames[iPos].uAction] = aActNames[iPos].pName;
			}
		}


	pErrorPos	= 0;									// Zeiger auf Fehlerposition
	pParam		= 0;									// Parameter fr Tokenfunktion
	pProc		= 0;									// Callback fr Token
	pValues		= 0;									// Array mit Werten
	uValueCnt	= 0;									// Anzahl der Werte
	uValueMax	= 0;									// Gre des Werte-Arrays
	pActions	= 0;									// Array mit Befehlen
	uActionCnt	= 0;									// Anzahl der Befehle
	uActionMax	= 0;									// Gre des Befehles-Arrays
	uCmpValue	= 0;									// Alles

}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
CalcGenData::~CalcGenData()
{

	CalcExit(this);


}

//*****************************************************************************
//*
//*		CalcActName
//*
//*****************************************************************************
//	Infos zu einem Befehl abfragen
//	pData	: Daten fr Calc-Generator
//	uAction		: Befehlsnummer
#if	ACT_LOGGING
static const char *CalcActName(CalcGenData *pData,CalcAction uAction)
{
static char		cText[64];
int				iPos;


	for(iPos=0;iPos<ENTRIES(aActNames);iPos++)
		{
		if(aActNames[iPos].uAction==uAction)
			{
			return aActNames[iPos].pName+4;
			}
		}

	if(uAction&ACT_GETVAR)
		{
		uAction &= ACT_MASK;
		sprintf(cText,"GETVAR_%i (%g)",uAction,pData->pValues[uAction]);
		return cText;
		}

	if(uAction&ACT_CMP)
		{
		uAction &= ACT_MASK;
		sprintf(cText,"CMP_%i",uAction);
		return cText;
		}

	if(uAction&ACT_GOTO)
		{
		uAction &= ACT_MASK;
		sprintf(cText,"GOTO_%i",uAction);
		return cText;
		}

	if(uAction&ACT_STOP)
		{
		uAction &= ACT_MASK;
		sprintf(cText,"STOP_%i",uAction);
		return cText;
		}

return "\?\?\?";
}

#endif

//*****************************************************************************
//*
//*		CalcPrint
//*
//*****************************************************************************
//	Gibt eine
//	pData		: Zeiger auf Generatordaten
void CalcPrint(CalcGenData *pData)
{
#if			ACT_LOGGING
static int	iShow=0;
unsigned	uPos;
double		dVal;
int			iOk;



	if(!iShow)
		{
		iShow=1;
		ShowConsole();
		}

	printf("\n");

	for(uPos=0;uPos<pData->uActionCnt;uPos++)
		{
		printf("ACT[%03i] %04Xh  %s\n",uPos,pData->pActions[uPos],CalcActName(pData,pData->pActions[uPos]));
		}

	dVal = CalcExec(pData->pActions,pData->uActionCnt,pData->pValues,&iOk);
	printf("[%sOK] = %g\n",(iOk)? "":"N",dVal);

#endif
}



//*****************************************************************************
//*
//*		CalcInit
//*
//*****************************************************************************
//	Initalisiert die Berechnung
//	pData		: Zeiger auf Generatordaten
//	iVars		: Anzahl der Variablen die nicht konstand sind
//	pProc		: Callback um die Variablenamen zu holen
//	pParam		: Parameter fr den Callback
void CalcInit(CalcGenData *pData,int iVars,TokenCb pProc,void *pParam)
{


	pData->pParam		= pParam;
	pData->pProc		= pProc;
	pData->uValueCnt	= iVars;
	pData->uActionCnt	= 0;

	if(iVars>(int)pData->uValueMax)
		{
		if(pData->pValues)delete pData->pValues;

		iVars		     += 0xFF;
		iVars			 &=~0xFF;
		pData->pValues    = new double[iVars];
		pData->uValueMax  = iVars;
		memset(pData->pValues,0,sizeof(double)*iVars);
		}

}

//*****************************************************************************
//*
//*		CalcExit
//*
//*****************************************************************************
//	Deinitalisiert die Berechnung
//	pData		: Zeiger auf Generatordaten
void CalcExit(CalcGenData *pData)
{


	if(pData->pValues )delete pData->pValues;
	if(pData->pActions)delete pData->pActions;

	memset(pData,0,sizeof(*pData));

}

//*****************************************************************************
//*
//*		CalcInstertAction
//*
//*****************************************************************************
//	Fgt eine neuen Befehl in das Array ein
//	pData		: Zeiger auf Generatordaten
//	uAction		: Befehlsnummer
static void CalcInstertAction(CalcGenData *pData,CalcAction uAction)
{
CalcAction	   *pNew;
unsigned		uMax;


	#if ACT_LOGGING
	printf("ACT[%03i] %04Xh  %s\n",pData->uActionCnt,uAction,CalcActName(pData,uAction));
	#endif
	if(pData->uActionCnt>=pData->uActionMax)
		{
		uMax	= pData->uActionMax+pData->uActionMax/2+32;
		pNew	= new CalcAction[uMax];

		memcpy(pNew,pData->pActions,sizeof(CalcAction)*pData->uActionMax);

		pData->uActionMax = uMax;

		if(pData->pActions)delete pData->pActions;

		pData->pActions = pNew;
		}

	pData->pActions[pData->uActionCnt]	= uAction;
	pData->uActionCnt++;

}

//*****************************************************************************
//*
//*		CalcInstertValue
//*
//*****************************************************************************
//	Fgt eine neue Konstante in das Wert-Array ein
//	pData		: Zeiger auf Generatordaten
//	dValue		: Wert der Konstanten
//	Ergibt die Einfgeposition von der Konstante.
static unsigned CalcInstertValue(CalcGenData *pData,double dValue)
{
double	   *pNew;
unsigned	uMax;
unsigned	uPos;


	#if ACT_LOGGING>1
	printf("VAL      %g\n",dValue);
	#endif


	uPos = pData->uValueCnt;

	if(uPos>=pData->uValueMax)
		{
		uMax	= pData->uValueMax+pData->uValueMax/2+32;
		pNew	= new double[uMax];

		memcpy(pNew,pData->pValues,sizeof(double)*uPos);
		pData->uValueMax = uMax;

		if(pData->pValues)delete pData->pValues;
		pData->pValues = pNew;
		}


	pData->pValues[uPos] = dValue;
	pData->uValueCnt++;


return uPos;
}


//*****************************************************************************
//*
//*		CalcToken
//*
//*****************************************************************************
//	Holt das nchste Zahlen-Token aus einem Text.
//	Vorzeichen wie + - ~ ! '...' werden dabei auch bercksichtigt.
//	pText	: Zeiger auf die Formel
//	pValue	: Zeiger auf das Ergebnis
//	pOk		: Zeiger auf die Variabel die angibt ob Fehler vorhanden waren
//	pData	: Daten fr Calc-Generator
//	Ergibt die Anzahl der Zeichen die in Text verarbeitet wurden.
static int CalcToken(const char *pText,double *pValue,int *pOk,CalcGenData *pData)
{
unsigned 	uVar;
int			iPos;
int			iLen;
int			iTyp;
int			iOk;



	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	*pValue=0;
	iPos=strwlen(pText,0xA0);
	pData->pErrorPos = pText+iPos;

	switch(pText[iPos])
		{
	case '(':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=CalcGen(pText+iPos,pValue,pOk,-1,pData);
				iPos+=strwlen(pText+iPos,0xA0);
				if(pText[iPos]!=')'){*pOk=0;break;}
				iPos++;
				break;
	case '+':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=CalcGen(pText+iPos,pValue,pOk,5,pData);
				break;
	case '-':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=CalcGen(pText+iPos,pValue,pOk,5,pData);
				CalcInstertAction(pData,ACT_NEG);
				*pValue=-*pValue;
				break;
	case '~':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=CalcGen(pText+iPos,pValue,pOk,3,pData);
				CalcInstertAction(pData,ACT_NOT);
				*pValue=(double)(~(__int64)*pValue);
				break;
	case '!':	iPos++;
				iPos+=strwlen(pText+iPos,0xA0);
				iPos+=CalcGen(pText+iPos,pValue,pOk,1,pData);
				CalcInstertAction(pData,ACT_LOGIC_NOT);
				*pValue=(*pValue<0 || *pValue>0)? 1:0;
				break;
	case '\'':	iPos++;
				*pValue=strtoc(pText+iPos,&iLen)&0xFF;
				iPos+=iLen;
				if(iLen<=0)*pOk=0;
				if(pText[iPos]=='\'')
					{
					uVar = CalcInstertValue(pData,*pValue);
					CalcInstertAction(pData,ACT_GETVAR|uVar);
					iPos++;
					}
				else{
					*pOk=0;
					}
				break;
	case 'F':
	case 'f':
				if(pText[iPos+1]=='L' || pText[iPos+1]=='l')
				if(pText[iPos+2]=='O' || pText[iPos+2]=='o')
				if(pText[iPos+3]=='O' || pText[iPos+3]=='o')
				if(pText[iPos+4]=='R' || pText[iPos+4]=='r')
				if(strwchara(pText[iPos+5],0x3000001F)==0)
					{
					iPos+=5;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
					CalcInstertAction(pData,ACT_FLOOR);
					*pValue=floor(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'R':
	case 'r':
				if(pText[iPos+1]=='O' || pText[iPos+1]=='o')
				if(pText[iPos+2]=='O' || pText[iPos+2]=='o')
				if(pText[iPos+3]=='T' || pText[iPos+3]=='t')
				if(strwchara(pText[iPos+4],0x00000001))
					{
					double	dBase;
					iPos+=4;
					dBase=strtof(pText+iPos,&iLen,0);
					uVar = CalcInstertValue(pData,dBase);
					CalcInstertAction(pData,ACT_GETVAR|uVar);
					CalcInstertAction(pData,ACT_UP);
					if(dBase==0.0){*pOk=0;break;}
					iPos+=iLen;
					iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
					if(*pValue<=0.0)*pValue=1.0;
					*pValue=pow(*pValue,1.0/dBase);
					CalcInstertAction(pData,ACT_ROOT);
					break;
					}
				if(pText[iPos+1]=='O' || pText[iPos+1]=='o')
				if(pText[iPos+2]=='U' || pText[iPos+2]=='u')
				if(pText[iPos+3]=='N' || pText[iPos+3]=='n')
				if(pText[iPos+4]=='D' || pText[iPos+4]=='d')
					{
					iPos+=5;
					iPos+=strwlen  (pText+iPos,0xA0);
					iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
					CalcInstertAction(pData,ACT_ROUND);
					*pValue=floor(*pValue+0.5);
					break;
					}
				*pOk=0;
				break;
	case 'L':
	case 'l':	if(pText[iPos+1]=='O' || pText[iPos+1]=='o')
				if(pText[iPos+2]=='G' || pText[iPos+2]=='g')
					{
					if(strwchara(pText[iPos+3],0x00000001))
						{
						double	dBase;
						iPos+=3;
						dBase=strtof(pText+iPos,&iLen,0);
						if(dBase<=0.0){*pOk=0;break;}
						uVar = CalcInstertValue(pData,dBase);
						CalcInstertAction(pData,ACT_GETVAR|uVar);
						CalcInstertAction(pData,ACT_UP);
						iPos+=iLen;
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_LOGX);
						if(*pValue>0.0)
								*pValue=log(*pValue)/log(dBase);
						else	*pValue=0;
						break;
						}
					if(strwchara(pText[iPos+3],0x3000001F)==0)
						{
						iPos+=3;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_LOG10);
						if(*pValue<=0.0)
								*pValue=log10(*pValue);
						else	*pValue=0;
						break;
						}
					}
				if(pText[iPos+1]=='N' || pText[iPos+1]=='n')
				if(strwchara(pText[iPos+2],0x3000001F)==0)
					{
					iPos+=2;
					iPos+=strwlen  (pText+iPos,0xA0);
					iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
					CalcInstertAction(pData,ACT_LOG);
					if(*pValue<=0.0)
							*pValue=log(*pValue);
					else	*pValue=0;
					break;
					}
				*pOk=0;
				break;
	case 'A':
	case 'a':	if(pText[iPos+1]=='T' || pText[iPos+1]=='t')
				if(pText[iPos+2]=='A' || pText[iPos+2]=='a')
				if(pText[iPos+3]=='N' || pText[iPos+3]=='n')
					{
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						*pValue=atan(*pValue);
						CalcInstertAction(pData,ACT_ATAN);
						break;
						}
					if(pText[iPos+4]=='H' || pText[iPos+4]=='h')
					if(strwchara(pText[iPos+5],0x3000001F)==0)
						{
						iPos+=5;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						if(fabs(*pValue)>=1.0)
								*pValue=0;
						else	*pValue=atanh(*pValue);
						CalcInstertAction(pData,ACT_ATANH);
						break;
						}
					}
				if(pText[iPos+1]=='C' || pText[iPos+1]=='c')
				if(pText[iPos+2]=='O' || pText[iPos+2]=='o')
				if(pText[iPos+3]=='T' || pText[iPos+3]=='t')
					{
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						*pValue=CALC_PI*0.5-atan(*pValue);
						CalcInstertAction(pData,ACT_ACOT);
						break;
						}
					if(pText[iPos+4]=='H' || pText[iPos+4]=='h')
					if(strwchara(pText[iPos+5],0x3000001F)==0)
						{
						iPos+=5;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						if(fabs(*pValue)<=1.0)
								*pValue=0;
						else	*pValue=acoth(*pValue);
						CalcInstertAction(pData,ACT_ACOTH);
						break;
						}
					}
				if(pText[iPos+1]=='C' || pText[iPos+1]=='c')
				if(pText[iPos+2]=='O' || pText[iPos+2]=='o')
				if(pText[iPos+3]=='S' || pText[iPos+3]=='s')
					{
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_ACOS);
						if(*pValue<=1.0 && *pValue>=-1.0)
								*pValue=acos(*pValue);
						else	*pValue=0;
						break;
						}
					if(pText[iPos+4]=='H' || pText[iPos+4]=='h')
					if(strwchara(pText[iPos+5],0x3000001F)==0)
						{
						iPos+=5;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						if(*pValue<=1.0)
								*pValue=0.0;
						else	*pValue=acosh(*pValue);
						CalcInstertAction(pData,ACT_ACOSH);
						break;
						}
					}
				if(pText[iPos+1]=='S' || pText[iPos+1]=='s')
				if(pText[iPos+2]=='I' || pText[iPos+2]=='i')
				if(pText[iPos+3]=='N' || pText[iPos+3]=='n')
					{
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_ASIN);
						if(*pValue<=1.0 && *pValue>=-1.0)
								*pValue=asin(*pValue);
						else	*pValue=0;
						break;
						}
					if(pText[iPos+4]=='H' || pText[iPos+4]=='h')
					if(strwchara(pText[iPos+5],0x3000001F)==0)
						{
						iPos+=5;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						*pValue=asinh(*pValue);
						CalcInstertAction(pData,ACT_ASINH);
						break;
						}
					}
				if(pText[iPos+1]=='B' || pText[iPos+1]=='b')
				if(pText[iPos+2]=='S' || pText[iPos+2]=='s')
				if(strwchara(pText[iPos+3],0x3000001F)==0)
					{
					iPos+=3;
					iPos+=strwlen  (pText+iPos,0xA0);
					iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
					CalcInstertAction(pData,ACT_ABS);
					*pValue=fabs(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'T':
	case 't':	if(pText[iPos+1]=='A' || pText[iPos+1]=='a')
				if(pText[iPos+2]=='N' || pText[iPos+2]=='n')
					{
					if(strwchara(pText[iPos+3],0x3000001F)==0)
						{
						iPos+=3;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_TAN);
						*pValue=tan(*pValue);
						break;
						}
					if(pText[iPos+3]=='H' || pText[iPos+3]=='h')
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_TANH);
						*pValue=tanh(*pValue);
						break;
						}
					}
				*pOk=0;
				break;
	case 'C':
	case 'c':	if(pText[iPos+1]=='O' || pText[iPos+1]=='o')
				if(pText[iPos+2]=='S' || pText[iPos+2]=='s')
					{
					if(strwchara(pText[iPos+3],0x3000001F)==0)
						{
						iPos+=3;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_COS);
						*pValue=cos(*pValue);
						break;
						}
					if(pText[iPos+3]=='H' || pText[iPos+3]=='h')
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_COSH);
						*pValue=cosh(*pValue);
						break;
						}
					}
				if(pText[iPos+1]=='O' || pText[iPos+1]=='o')
				if(pText[iPos+2]=='T' || pText[iPos+2]=='t')
					{
					if(strwchara(pText[iPos+3],0x3000001F)==0)
						{
						iPos+=3;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_COT);
						*pValue=cot(*pValue);
						break;
						}
					if(pText[iPos+3]=='H' || pText[iPos+3]=='h')
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_COTH);
						if(fabs(*pValue)==0)
								*pValue=0.0;
						else	*pValue=coth(*pValue);
						break;
						}
					}
				if(pText[iPos+1]=='E' || pText[iPos+1]=='e')
				if(pText[iPos+2]=='I' || pText[iPos+2]=='i')
				if(pText[iPos+3]=='L' || pText[iPos+3]=='l')
				if(strwchara(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=4;
					iPos+=strwlen (pText+iPos,0xA0);
					iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
					CalcInstertAction(pData,ACT_CEIL);
					*pValue=ceil(*pValue);
					break;
					}
				*pOk=0;
				break;
				*pOk=0;
				break;
	case 'S':
	case 's':	if(pText[iPos+1]=='I' || pText[iPos+1]=='i')
				if(pText[iPos+2]=='N' || pText[iPos+2]=='n')
					{
					if(strwchara(pText[iPos+3],0x3000001F)==0)
						{
						iPos+=3;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_SIN);
						*pValue=sin(*pValue);
						break;
						}
					if(pText[iPos+3]=='H' || pText[iPos+3]=='h')
					if(strwchara(pText[iPos+4],0x3000001F)==0)
						{
						iPos+=4;
						iPos+=strwlen  (pText+iPos,0xA0);
						iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
						CalcInstertAction(pData,ACT_SINH);
						*pValue=sinh(*pValue);
						break;
						}
					}
				if(pText[iPos+1]=='Q' || pText[iPos+1]=='q')
				if(pText[iPos+2]=='R' || pText[iPos+2]=='r')
				if(pText[iPos+3]=='T' || pText[iPos+3]=='t')
				if(strwchara(pText[iPos+4],0x3000001F)==0)
					{
					iPos+=4;
					iPos+=strwlen  (pText+iPos,0xA0);
					iPos+=CalcToken(pText+iPos,pValue,pOk,pData);
					CalcInstertAction(pData,ACT_SQRT);
					if(*pValue<=0.0)*pValue=sqrt(*pValue);
					break;
					}
				*pOk=0;
				break;
	case 'P':
	case 'p':	if(pText[iPos+1]=='I' || pText[iPos+1]=='i')
				if(strwchara(pText[iPos+2],0x3000001F)==0)
					{
					*pValue = CALC_PI;
					uVar = CalcInstertValue(pData,*pValue);
					CalcInstertAction(pData,ACT_GETVAR|uVar);
					iPos+=2;
					iPos+=strwlen(pText+iPos,0xA0);
					break;
					}
				*pOk=0;
				break;
	case 'E':
	case 'e':	if(strwchara(pText[iPos+1],0x3000001F)==0)
					{
					*pValue = 2.7182818284590452353602874713527;
					uVar = CalcInstertValue(pData,*pValue);
					CalcInstertAction(pData,ACT_GETVAR|uVar);
					iPos+=1;
					iPos+=strwlen(pText+iPos,0xA0);
					break;
					}
				*pOk=0;
				break;

	default:	if(pText[iPos]=='0')
					{
					iTyp=strchar("XxOoDdBb",pText[iPos+1]);
					if(iTyp&4)
						{
						iLen=strwlen(pText,0x07)-1;
						if(pText[iLen]=='h' || pText[iLen]=='H')iTyp=8;
						if(iTyp&2 && iLen==1)iTyp=8;
						}
					if(iTyp<8)
						{
						*pValue=(double)strtoq(pText+iPos,0,&iLen);
						iPos+=iLen;
						if(iLen<=2)*pOk=0;
						break;
						}
					}

				*pValue=strtof(pText+iPos,&iLen,0x07);

				iPos+=iLen;
				if(iLen<=0)
					{
					*pOk=0;
					}
				else{
					uVar = CalcInstertValue(pData,*pValue);
					CalcInstertAction(pData,ACT_GETVAR|uVar);
					}
		}

	if(!*pOk)											// User-Token suchen
		{
		   iLen = pData->pProc(pData->pParam,pText+iPos,pValue,&uVar);
		if(iLen>0 && uVar<pData->uValueCnt)
			{
			CalcInstertAction(pData,ACT_GETVAR|uVar);
			iPos+=iLen;
			*pOk=1;
			}
		}


	iPos+=strwlen(pText+iPos,0xA0);



return iPos;
}

//*****************************************************************************
//*
//*		CalcGen
//*
//*****************************************************************************
//	Berechnet eine Formel die in einem Text enthalten ist
//	pText	: Zeiger auf die Formel
//	pValue	: Zeiger auf das Ergebnis
//	pOk		: Zeiger auf die Variabel die angibt ob Fehler vorhanden waren
//	iLevel	: Hhe des Berechnugs Levels ( -3=Auswahl -2=Start  -1 Klammer 0=Boolsch
//											1=Vergleich 2=Logic 3=Schiebeoperation
//											4=Strichrechnung 5=Punktrechnung )
//	pData	: Daten fr Calc-Generator
//	Ergibt die Anzahl der Zeichen die in Text verarbeitet wurden
static int CalcGen(const char *pText,double *pValue,int *pOk,int iLevel,CalcGenData *pData)
{
double	dValue2;
double	dValue3;
char	cChar;
int		iPos;
int		iVal;
int		iOk;



	if(! pOk) pOk=&iOk;
	if(!*pOk)*pOk=1;

	iPos=CalcToken(pText,pValue,pOk,pData);
	if(!*pOk)return iPos;


	while(pText[iPos])
		{
		pData->pErrorPos = pText+iPos;

		switch(pText[iPos])
			{
		case '\t':
		case ' ':	iPos++;
					break;
		case ')':	if(iLevel<-1)*pOk=0;
					return iPos;
		case ':':	if(iLevel!=-3)*pOk=0;
					return iPos;
		case '!':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					CalcInstertAction(pData,ACT_UP);
					if(iLevel>1)return iPos;		  iPos+=2;
					if(iLevel<1)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,2,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_NEQ);
					*pValue=*pValue != dValue2;
					if(!*pOk)return iPos;
					break;
		case '=':	cChar=pText[iPos+1];
					if(cChar!='='){*pOk=0;return iPos;}
					CalcInstertAction(pData,ACT_UP);
					if(iLevel>1)return iPos;	   	  iPos+=2;
					if(iLevel<1)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,2,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_EQ);
					*pValue=*pValue == dValue2;
					if(!*pOk)return iPos;
					break;
		case '<':	cChar=pText[iPos+1];
					if(cChar=='<')
						{
						if(iLevel>3)return iPos;		  iPos+=2;
						CalcInstertAction(pData,ACT_UP);
						if(iLevel<3)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,4,pData);
						else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
						*pValue = (double)((__int64)*pValue<<(int)dValue2);
						CalcInstertAction(pData,ACT_SHL);
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		  iPos+=(cChar=='=')? 2:1;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<1)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,2,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					if(cChar=='=')
						{
						CalcInstertAction(pData,ACT_LOWER_EQ);
						*pValue=*pValue <= dValue2;
						}
					else{
						CalcInstertAction(pData,ACT_LOWER);
						*pValue=*pValue <  dValue2;
						}
					if(!*pOk)return iPos;
					break;
		case '>':	cChar=pText[iPos+1];
					if(cChar=='>')
						{
						if(iLevel>3)return iPos;		  iPos+=2;
						CalcInstertAction(pData,ACT_UP);
						if(iLevel<3)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,4,pData);
						else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
						CalcInstertAction(pData,ACT_SHR);
						*pValue = (double)((__int64)*pValue>>(int)dValue2);
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>1)return iPos;		  iPos+=(cChar=='=')? 2:1;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<1)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,2,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					if(cChar=='=')
						{
						CalcInstertAction(pData,ACT_UPPER_EQ);
						*pValue=*pValue >= dValue2;
						}
					else{
						CalcInstertAction(pData,ACT_UPPER);
						*pValue=*pValue >  dValue2;
						}
					if(!*pOk)return iPos;
					break;
		case '|':	if(pText[iPos+1]=='|')
						{
						if(iLevel>0)return iPos;		  iPos+=2;
						CalcInstertAction(pData,ACT_UP);
						if(iLevel<0)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,1,pData);
						else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
						CalcInstertAction(pData,ACT_LOGIC_OR);
						*pValue=*pValue || dValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<2)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,3,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_OR);
					*pValue = (double)((__int64)*pValue|(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '&':	if(pText[iPos+1]=='&')
						{
						if(iLevel>0)return iPos;	      iPos+=2;
						CalcInstertAction(pData,ACT_UP);
						if(iLevel<0)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,1,pData);
						else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
						CalcInstertAction(pData,ACT_LOGIC_AND);
						*pValue=*pValue && dValue2;
						if(!*pOk)return iPos;
						break;
						}
					if(iLevel>2)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<2)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,3,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_AND);
					*pValue = (double)((__int64)*pValue&(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '^':	if(iLevel>2)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<2)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,3,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_XOR);
					*pValue = (double)((__int64)*pValue^(__int64)dValue2);
					if(!*pOk)return iPos;
					break;
		case '+':	if(iLevel>4)return iPos;	      iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<4)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,5,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_ADD);
					*pValue+=dValue2;
					if(!*pOk)return iPos;
					break;
		case '-':	if(iLevel>4)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<4)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,5,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_SUB);
					*pValue-=dValue2;
					if(!*pOk)return iPos;
					break;
		case '*':	if(iLevel>5)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<5)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,6,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_MUL);
					*pValue*=dValue2;
					if(!*pOk)return iPos;
					break;
		case '/':	if(iLevel>5)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<5)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,6,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_DIV);
					if(dValue2!=0.0)*pValue/=dValue2;
					if(!*pOk)return iPos;
					break;
		case '%':	if(iLevel>5)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<5)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,6,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_MOD);
					if(dValue2==0.0)*pValue = fmod(*pValue,dValue2);
					if(!*pOk)return iPos;
					break;
		case '\'':	if(iLevel>6)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_UP);
					if(iLevel<6)iPos+=CalcGen  (pText+iPos,&dValue2,pOk,7,pData);
					else	    iPos+=CalcToken(pText+iPos,&dValue2,pOk  ,pData);
					CalcInstertAction(pData,ACT_POW);
					*pValue = pow(*pValue,dValue2);
					if(!*pOk)return iPos;
					break;
		case '':	if(iLevel>6)return iPos;		  iPos++;
					CalcInstertAction(pData,ACT_POW2);
					dValue2 = *pValue;
					*pValue = dValue2*dValue2;
					break;
		case '':	if(iLevel>6)return iPos;		  iPos++;
					dValue2 = *pValue;
					CalcInstertAction(pData,ACT_POW3);
					*pValue = dValue2*dValue2*dValue2;
					break;
		case '?':	if(iLevel>7)return iPos;		  iPos++;
					iVal = pData->uCmpValue;
					pData->uCmpValue++;
					CalcInstertAction(pData,ACT_CMP|iVal);
					iPos+=CalcGen  (pText+iPos,&dValue2,pOk,-3,pData);
					if(pText[iPos]!=':'){*pOk=0;break;}
					iPos++;
					CalcInstertAction(pData,ACT_GOTO|iVal);
					iPos+=CalcGen  (pText+iPos,&dValue3,pOk,-2,pData);
					CalcInstertAction(pData,ACT_STOP|iVal);
					*pValue = (*pValue)? dValue2:dValue3;
					if(!*pOk)return iPos;
					break;
		default:	*pOk=0;
					return iPos;
			}
		}


return iPos;
}

//*****************************************************************************
//*
//*		CalcSort
//*
//*****************************************************************************
//	Sortiert eine Befehlsliste so um, dass Konstanetn beisammen stehen
//	pData		: Daten fr Calc-Generator
//	pConstList	: Welche Werte sind Konstanden
//	uSize		: Anzahl der Eintrge in der Const-Liste
//	Ergibt die Anzahl der Sortiertenteile
static int CalcSort(CalcGenData *pData,const unsigned char *pConstList,unsigned uSize)
{
CalcAction *pList;
CalcAction	uTemp;
unsigned	uPos;
unsigned	uOff;
unsigned	uMax;
unsigned	uEnd;
unsigned	uVar;
unsigned	uCnt;


	   uMax = pData->uActionCnt;
	if(uMax<6)return 0;

	uMax  -= 5;
	uCnt   = 0;
	uEnd   = uMax+3;
	pList  = pData->pActions;

	for(uPos=0;uPos<uMax;uPos++)
		{
		if( (pList[uPos  ]!=ACT_UP    ))continue;				// Start Muster
		if(!(pList[uPos+1]& ACT_GETVAR))continue;
		if( (pList[uPos+3]!=ACT_UP    ))continue;
		if(!(pList[uPos+4]& ACT_GETVAR))continue;

		for(uOff=uPos+3;uOff<uEnd;uOff+=3)						// Sortierschleife
			{
			if( (pList[uOff  ]!=ACT_UP    ))break;
			if(!(pList[uOff+1]& ACT_GETVAR))break;

			   uVar =pList[uOff+1]&ACT_MASK;
			if(uVar< uSize && !pConstList[uVar])continue;
			   uVar =pList[uOff-2]&ACT_MASK;
			if(uVar>=uSize ||  pConstList[uVar])continue;

			if(pList[uOff-1]==ACT_ADD || pList[uOff-1]==ACT_SUB)
				{
				if(pList[uOff+2]!=ACT_ADD)
				if(pList[uOff+2]!=ACT_SUB)
					{
					continue;
					}
				}
			else if(pList[uOff-1]==ACT_MUL || pList[uOff-1]==ACT_DIV)
				{
				if(pList[uOff+2]!=ACT_MUL)
				if(pList[uOff+2]!=ACT_DIV)
					{
					continue;
					}
				}
			else{
				break;
				}

			uTemp		  = pList[uOff-2];						// Tauschen
			pList[uOff-2] = pList[uOff+1];
			pList[uOff+1] = uTemp;

			uTemp		  = pList[uOff-1];
			pList[uOff-1] = pList[uOff+2];
			pList[uOff+2] = uTemp;

			uOff  = uPos;
			uCnt += 1;
			}

		uPos = uOff-1;
		}


return uCnt;
}

//*****************************************************************************
//*
//*		CalcComp
//*
//*****************************************************************************
//	Komprimiert eine Befehlsliste so dass Konstaneten Ausdrcke zusammen gefasst werden.
//	pData		: Daten fr Calc-Generator
//	pConstList	: Welche Werte sind Konstanden
//	Ergibt die Anzahl der Komprimiertenteile
static int CalcComp(CalcGenData *pData,const unsigned char *pConstList,unsigned uSize)
{
CalcAction *pList;
CalcAction	uLast;
CalcAction	uAction;
double		dVal;
unsigned	uSet;
unsigned	uPos;
unsigned	uOff;
unsigned	uMax;
unsigned	uVar;
unsigned	uOut;
unsigned	uLen;
unsigned	uCnt;
int			iOld;
int			iDif;
int			iOk;


	   uMax = pData->uActionCnt;
	if(uMax<2)return 0;

	uCnt   = 0;
	uOut   = 1;
	pList  = pData->pActions;
	uLast  = pList[0];

	for(uPos=1;uPos<uMax;uPos++)
		{
		uAction = pList[uPos];

		if(uAction==ACT_NEG)									// Doppelte Negierung
		if(uPos+1<uMax && pList[uPos+1]==ACT_NEG)
			{
			uPos++;
			continue;
			}

		if(uAction==ACT_NOT && uLast==ACT_NOT)					// Doppelte Negierung
		if(uPos+1<uMax && pList[uPos+1]==ACT_NOT)
		if(uPos+2<uMax && pList[uPos+2]==ACT_NOT)
			{
			uPos++;
			continue;
			}

		if(uAction==ACT_NEG && (uLast&ACT_GETVAR))				// Negierung bei Konstanten entfernen
			{
			   uVar = uLast&ACT_MASK;
			if(uVar>=uSize || pConstList[uVar])
				{
				uVar		  = CalcInstertValue(pData,-pData->pValues[uVar]);
				pList[uOut-1] = ACT_GETVAR|uVar;
				uLast		  = ACT_GETVAR|uVar;
				uCnt         += 1;
				continue;
				}
			}

		if(uAction==ACT_NEG && (uLast&ACT_GETVAR))				// Negierung bei Plus in Minus umwandeln
		if(uPos+1<uMax && pList[uPos+1]==ACT_ADD)
			{
			uPos++;
			pList[uPos]=ACT_SUB;
			continue;
			}

		if(uAction==ACT_NEG && (uLast&ACT_GETVAR))				// Negierung bei Minus in Plus umwandeln
		if(uPos+1<uMax && pList[uPos+1]==ACT_SUB)
			{
			uPos++;
			pList[uPos]=ACT_ADD;
			continue;
			}

		if(uAction==ACT_NOT && (uLast&ACT_GETVAR))				// Invertierung bei Konstanten entfernen
			{
			   uVar = uLast&ACT_MASK;
			if(uVar>=uSize || pConstList[uVar])
				{
				uVar		  = CalcInstertValue(pData,(double)~(__int64)pData->pValues[uVar]);
				pList[uOut-1] = ACT_GETVAR|uVar;
				uLast		  = ACT_GETVAR|uVar;
				uCnt         += 1;
				continue;
				}
			}

		if(uAction==ACT_DIV && (uLast&ACT_GETVAR))				// DIV in MUL umwandeln
			{
			    uVar = uLast&ACT_MASK;
			if((uVar>=uSize || pConstList[uVar]) && pData->pValues[uVar]!=0.0)
				{
				uVar		  = CalcInstertValue(pData,1.0/pData->pValues[uVar]);
				pList[uOut-1] = ACT_GETVAR|uVar;
				uLast		  = ACT_GETVAR|uVar;
				pList[uPos]   = ACT_MUL;
				uPos--;
				continue;
				}
			}

																// Konstanten +- optimieren
		if((uAction==ACT_ADD || uAction==ACT_SUB) && (uLast&ACT_GETVAR))
		if(uPos+1<uMax && pList[uPos+1]==ACT_UP)
		if(uPos+2<uMax &&(pList[uPos+2] &ACT_GETVAR))
		if(uPos+3<uMax &&(pList[uPos+3]==ACT_ADD||pList[uPos+3]==ACT_SUB))
			{
			   uVar =pList[uPos+2]&ACT_MASK;
			if(uVar>=uSize || pConstList[uVar])
				{
				dVal = pData->pValues[uVar];
				uVar = uLast&ACT_MASK;

				if(uVar>=uSize || pConstList[uVar])
					{
					if(pList[uPos+3]==ACT_SUB)dVal=-dVal;

					if(uAction==ACT_ADD)
							uVar  = CalcInstertValue(pData,pData->pValues[uVar]+dVal);
					else	uVar  = CalcInstertValue(pData,pData->pValues[uVar]-dVal);

					pList[uPos+3] = uAction;
					pList[uOut-1] = ACT_GETVAR|uVar;
					uLast		  = ACT_GETVAR|uVar;
					uCnt         += 1;
					uPos		 += 2;
					continue;
					}
				}
			}
																// Konstanten */ optimieren
		if((uAction==ACT_MUL || uAction==ACT_DIV) && (uLast&ACT_GETVAR))
		if(uPos+1<uMax && pList[uPos+1]==ACT_UP)
		if(uPos+2<uMax &&(pList[uPos+2] &ACT_GETVAR))
		if(uPos+3<uMax &&(pList[uPos+3]==ACT_MUL||pList[uPos+3]==ACT_DIV))
			{
			   uVar =pList[uPos+2]&ACT_MASK;
			if(uVar>=uSize || pConstList[uVar])
				{
				dVal = pData->pValues[uVar];
				uVar = uLast&ACT_MASK;

				if((uVar>=uSize || pConstList[uVar]) && (dVal!=0.0 || pList[uPos+3]==ACT_DIV))
					{
					if(pList[uPos+3]==ACT_DIV)dVal=1.0/dVal;

					if(uAction==ACT_MUL)
							uVar  = CalcInstertValue(pData,pData->pValues[uVar]*dVal);
					else	uVar  = CalcInstertValue(pData,pData->pValues[uVar]/dVal);

					pList[uPos+3] = uAction;
					pList[uOut-1] = ACT_GETVAR|uVar;
					uLast		  = ACT_GETVAR|uVar;
					uCnt         += 1;
					uPos		 += 2;
					continue;
					}
				}
			}


		pList[uOut] = uAction;									// Keine nderung
		uLast       = uAction;
		uOut++;
		}

	pData->uActionCnt = uOut;
	uMax			  = uOut;

	if(uMax<2)return uCnt;

	uOut   = 0;
	uLast  = ACT_UP;

	for(uPos=0;uPos<uMax;uPos++)
		{
		uAction = pList[uPos];

		if(uLast!=ACT_UP || !(uAction&ACT_GETVAR))				// Startmuster gefunden
			{
			pList[uOut] = uAction;
			uLast       = uAction;
			uOut++;
			continue;
			}

		uSet = uPos;
		iDif = 1;

		for(uOff=uPos;uOff<uMax;uOff++)							// Lnge des Konstantenbereiches ermitteln
			{
			uAction = pList[uOff];

			if(uAction&ACT_GETVAR)
				{
				   uVar = uAction&ACT_MASK;
				if(uVar<uSize && !pConstList[uVar])break;
				continue;
				}

			if(uAction&~ACT_MASK)
				{
				continue;
				}

			   iOld  = iDif;
			   iDif += aDeltaList[uAction];
			if(iDif<0)break;

			if(aLevelList[uAction]==0)continue;
			if(aLevelList[uAction]>iOld)
				{
				break;
				}

			if(aLevelList[uAction]==iOld)
				{
				uSet = uOff;
				}
			}

		   uLen = uSet-uPos+1;
		if(uLen==1)												// Kein Bereich gefunden
			{
			pList[uOut] = pList[uPos];
			uLast       = pList[uPos];
			uOut++;
			continue;
			}

		iOk	 = 1;
		dVal = CalcExec(pList+uPos,uLen,pData->pValues,&iOk);

		if(iOk)
			{
			uVar = CalcInstertValue(pData,dVal);
			pList[uOut] = ACT_GETVAR|uVar;
			}
		else{
			pList[uOut] = ACT_ERROR;
			}

		uLast = pList[uOut];
		uPos += uLen-1;
		uOut += 1;
		uCnt += 1;
		}

	pData->uActionCnt = uOut;


return uCnt;
}

//*****************************************************************************
//*
//*		CalcCompress
//*
//*****************************************************************************
//	Komprimiert eine Befehlsliste so dass Konstaneten Ausdrcke zusammen gefast werden.
//	pData		: Daten fr Calc-Generator
//	pConstList	: Welche Werte sind Konstanden
//	uSize		: Anzahl der Eintrge in der Const-Liste
//	Ergibt die Anzahl der Komprimiertenteile
void CalcCompress(CalcGenData *pData,const unsigned char *pConstList,unsigned uSize)
{
int				iChg=1;


	while(iChg)
		{
		iChg  = CalcComp(pData,pConstList,uSize);
		iChg |= CalcSort(pData,pConstList,uSize);
		}


}

//*****************************************************************************
//*
//*		CalcChangeVars
//*
//*****************************************************************************
//	Tascht die Indizes zweier Variablen aus.
//	pData		: Daten fr Calc-Generator
//	uVar1/2		: Indizes der Variablen
//	Ergibt 0 wenn ok oder eine Fehlernummer
//					1 = Keine Daten
//					2 Falscher Index fr uVar1
//					2 Falscher Index fr uVar2
int CalcChangeVars(CalcGenData *pData,unsigned uVar1,unsigned uVar2)
{
CalcAction	uAction;
unsigned	uPos;
unsigned	uVar;

	if(!pData)return 1;

	if(uVar1>=pData->uValueCnt)return 2;
	if(uVar2>=pData->uValueCnt)return 3;

	if(uVar1==uVar2)return 0;

	for(uPos=0;uPos<pData->uActionCnt;uPos++)
		{
		uAction = pData->pActions[uPos];

		if(!(uAction&ACT_GETVAR))continue;

		uVar = uAction&ACT_MASK;

		if(uVar==uVar1)
			{
			pData->pActions[uPos] = ACT_GETVAR|uVar2;
			continue;
			}

		if(uVar==uVar2)
			{
			pData->pActions[uPos] = ACT_GETVAR|uVar1;
			continue;
			}
		}


return 0;
}


//*****************************************************************************
//*
//*		CalcCopy
//*
//*****************************************************************************
//	Kopiert einen Calc-Generator.
//	pData		: Daten fr Calc-Generator
//	pNew		: Zeiger auf die neuen Daten
void CalcCopy(CalcGenData *pData,CalcGenData *pNew)
{

	pNew->pErrorPos	= pData->pErrorPos;
	pNew->pParam	= pData->pParam;
	pNew->pProc		= pData->pProc;
	pNew->uValueCnt	= pData->uValueCnt;
	pNew->uActionCnt= pData->uActionCnt;
	pNew->uCmpValue	= pData->uCmpValue;


	if(pNew->uValueMax<pData->uValueMax)
		{
		if(pNew->pValues)delete pNew->pValues;

		pNew->uValueMax = pData->uValueMax;
		pNew->pValues	= new double[pNew->uValueMax];
		}

	if(pNew->uActionMax<pData->uActionMax)
		{
		if(pNew->pActions)delete pNew->pActions;

		pNew->uActionMax = pData->uActionMax;
		pNew->pActions	 = new CalcAction[pNew->uActionMax];
		}

	memcpy(pNew->pActions,pData->pActions,sizeof(CalcAction)*pData->uActionCnt);
	memcpy(pNew->pValues ,pData->pValues ,sizeof(double    )*pData->uValueCnt );

}

//*****************************************************************************
//*
//*		CalcDependency
//*
//*****************************************************************************
//	Berechnet die Abhngigkeiten einer Formel
//	pData		: Daten fr Calc-Generator
//	pBits		: Bitfeld in dem fr jede benutzte Variable und Konstande
//				  ein Bit gesetzt wird
//	uMax		: Gre des Bitfeldes in Bits
//	iClear		: Alle Bits lschen
//	Ergibt die Anzahl der gesetzen Bits
int CalcDependency(CalcGenData *pData,void *pBits,unsigned uMax,int iClear)
{
CalcAction	uAction;
unsigned	uPos;
unsigned	uVar;
unsigned	uCnt;


	if(iClear)memset(pBits,0,(uMax+7)/8);
	if(pData==0)return 0;

	uCnt = 0;

	for(uPos=0;uPos<pData->uActionCnt;uPos++)
		{
		uAction = pData->pActions[uPos];

		if(!(uAction&ACT_GETVAR))continue;

		uVar = uAction&ACT_MASK;
		if(uVar>=uMax)continue;

		bit_set(pBits,uVar);
		uCnt++;
		}


return uCnt;
}

//*****************************************************************************
//*
//*		CalcExec
//*
//*****************************************************************************
//	Berechnet den Wert einer Actions-Liste
//	pList		: Zeiger auf die Actions-Liste
//	uMax		: Anzahl der Eintrge in der Actions-Liste
//	pVars		: Zeiger auf die Variabelwerte
//	pOk			: Hier wird 1 gespeichert wenn die Berechung ok war
//	Ergibt den berechneten Wert
static double CalcExec(CalcAction *pList,unsigned uMax,double *pVars,int *pOk)
{
double		aValues[512];
CalcAction	uAction;
CalcAction	uStop;
unsigned	uPos;
unsigned 	uOff;
int			iOk;


	uOff = 0;
	if(!pOk){iOk=1;pOk=&iOk;}

	for(uPos=0;uPos<uMax;uPos++)
		{
		#if _DEBUG
		if(uOff>=0xF0000000)
			{
			MessageBox(0,"Variabelstack Unterlauf","Fehler",MB_OK|MB_ICONERROR);
			}
		else if(uOff>=ENTRIES(aValues))
			{
			MessageBox(0,"Variabelstack berlauf","Fehler",MB_OK|MB_ICONERROR);
			}
		#endif

		uAction = pList[uPos];

		switch(uAction)
			{
		case ACT_END:		uPos=uMax;													break;
		case ACT_ERROR:		uPos=uMax;													break;
		case ACT_UP:		uOff++;														break;
		case ACT_NEG:		aValues[uOff  ] = -aValues[uOff];							break;
		case ACT_NOT:		aValues[uOff  ] = (double)(~(__int64)aValues[uOff]);		break;
		case ACT_LOGIC_NOT:	aValues[uOff  ] = !aValues[uOff];							break;
		case ACT_ABS:		aValues[uOff  ] = fabs(aValues[uOff]);						break;
		case ACT_CEIL:		aValues[uOff  ] = ceil(aValues[uOff]);						break;
		case ACT_FLOOR:		aValues[uOff  ] = floor(aValues[uOff]);						break;
		case ACT_ROUND:		aValues[uOff  ] = floor(aValues[uOff]+0.5);					break;
		case ACT_ACOS:		if(aValues[uOff]>1.0 || aValues[uOff]<-1.0){uPos=uMax;break;}
							aValues[uOff  ] = acos(aValues [uOff]);						break;
		case ACT_ASIN:		if(aValues[uOff]>1.0 || aValues[uOff]<-1.0){uPos=uMax;break;}
							aValues[uOff  ] = asin(aValues [uOff]);						break;
		case ACT_ATAN:		aValues[uOff  ] = atan(aValues [uOff]);						break;
		case ACT_ACOT:		aValues[uOff  ] = CALC_PI*0.5-atan(aValues[uOff]);			break;
		case ACT_ACOSH:		if(aValues[uOff]<=1.0){uPos=uMax;break;}
							aValues[uOff  ] = acosh(aValues [uOff]);					break;
		case ACT_ASINH:		aValues[uOff  ] = asinh(aValues [uOff]);					break;
		case ACT_ATANH:		if(fabs(aValues[uOff])>=1.0){uPos=uMax;break;}
							aValues[uOff  ] = atanh(aValues[uOff]);						break;
		case ACT_ACOTH:		if(fabs(aValues[uOff])<=1.0){uPos=uMax;break;}
							aValues[uOff  ] = acoth(aValues[uOff]);						break;
		case ACT_COS:		aValues[uOff  ] = cos(aValues[uOff]);						break;
		case ACT_SIN:		aValues[uOff  ] = sin(aValues[uOff]);						break;
		case ACT_TAN:		aValues[uOff  ] = tan(aValues[uOff]);						break;
		case ACT_COT:		aValues[uOff  ] = cot(aValues[uOff]);						break;
		case ACT_COSH:		aValues[uOff  ] = cosh(aValues[uOff]);						break;
		case ACT_SINH:		aValues[uOff  ] = sinh(aValues[uOff]);						break;
		case ACT_TANH:		aValues[uOff  ] = tanh(aValues[uOff]);						break;
		case ACT_COTH:		if(fabs(aValues[uOff])==0.0){uPos=uMax;break;}
							aValues[uOff  ] = coth(aValues[uOff]);						break;
		case ACT_LOG:		aValues[uOff  ] = log(aValues[uOff]);						break;
		case ACT_LOGX:		aValues[uOff-1] = log(aValues[uOff])/log(aValues[uOff-1]);uOff--;break;
		case ACT_LOG10:		aValues[uOff  ] = log10(aValues[uOff]);						break;
		case ACT_ROOT:		aValues[uOff-1] = pow(aValues[uOff],1.0/aValues[uOff-1]);uOff--;break;
		case ACT_SQRT:		if(aValues[uOff]<0.0){uPos=uMax;break;}
							aValues[uOff  ] = sqrt(aValues[uOff]);						break;
		case ACT_EQ:		aValues[uOff-1] = aValues[uOff-1]==aValues[uOff];uOff--;	break;
		case ACT_NEQ:		aValues[uOff-1] = aValues[uOff-1]!=aValues[uOff];uOff--;	break;
		case ACT_LOWER:		aValues[uOff-1] = aValues[uOff-1]< aValues[uOff];uOff--;	break;
		case ACT_LOWER_EQ:	aValues[uOff-1] = aValues[uOff-1]<=aValues[uOff];uOff--;	break;
		case ACT_UPPER:		aValues[uOff-1] = aValues[uOff-1]> aValues[uOff];uOff--;	break;
		case ACT_UPPER_EQ:	aValues[uOff-1] = aValues[uOff-1]>=aValues[uOff];uOff--;	break;
		case ACT_LOGIC_OR:	aValues[uOff-1] = aValues[uOff-1]||aValues[uOff];uOff--;	break;
		case ACT_LOGIC_AND:	aValues[uOff-1] = aValues[uOff-1]&&aValues[uOff];uOff--;	break;
		case ACT_SHR:		aValues[uOff-1] = (double)((__int64)aValues[uOff-1]>>(int)aValues[uOff]);uOff--;break;
		case ACT_SHL:		aValues[uOff-1] = aValues[uOff-1]||aValues[uOff];uOff--;	break;
		case ACT_OR:		aValues[uOff-1] = (double)((__int64)aValues[uOff-1]|(__int64)aValues[uOff]);uOff--;break;
		case ACT_AND:		aValues[uOff-1] = (double)((__int64)aValues[uOff-1]&(__int64)aValues[uOff]);uOff--;break;
		case ACT_XOR:		aValues[uOff-1] = (double)((__int64)aValues[uOff-1]^(__int64)aValues[uOff]);uOff--;break;
		case ACT_ADD:		aValues[uOff-1] = aValues[uOff-1] +aValues[uOff];uOff--;	break;
		case ACT_SUB:		aValues[uOff-1] = aValues[uOff-1] -aValues[uOff];uOff--;	break;
		case ACT_MUL:		aValues[uOff-1] = aValues[uOff-1] *aValues[uOff];uOff--;	break;
		case ACT_DIV:		if(aValues[uOff]==0.0){uPos=uMax;break;}
							aValues[uOff-1] = aValues[uOff-1] /aValues[uOff];uOff--;	break;
		case ACT_MOD:		if(aValues[uOff]==0.0){uPos=uMax;break;}
							aValues[uOff-1] = fmod(aValues[uOff-1],aValues[uOff]);uOff--;break;
		case ACT_POW:		aValues[uOff-1] = pow (aValues[uOff-1],aValues[uOff]);uOff--;break;
		case ACT_POW2:		aValues[uOff  ] = aValues[uOff]*aValues[uOff];				break;
		case ACT_POW3:		aValues[uOff  ] = aValues[uOff]*aValues[uOff]*aValues[uOff];break;

		default:

			if(uAction&ACT_GETVAR)
				{
				uAction      &= ACT_MASK;
				aValues[uOff] = pVars[uAction];
				break;
				}

			if(uAction&ACT_CMP)
				{
				uOff--;
				if(aValues[uOff+1])continue;

				uStop  = uAction&ACT_MASK;
				uStop |= ACT_GOTO;

				for(uPos++;;uPos++)
					{
					if(uPos>=uMax){*pOk=0;break;}
					if(uStop==pList[uPos])break;
					}
				}

			if(uAction&ACT_GOTO)
				{
				uStop  = uAction&ACT_MASK;
				uStop |= ACT_STOP;

				for(uPos++;;uPos++)
					{
					if(uPos>=uMax){*pOk=0;break;}
					if(uStop==pList[uPos])break;
					}
				}

			if(uAction&ACT_STOP)
				{
				continue;
				}

			#if _DEBUG
			MessageBox(0,"Unbekannter Befehl","Fehler",MB_OK|MB_ICONERROR);
			#endif

			uPos=uMax;
			break;
			}
		}

	if(uPos>uMax)
		{
		if(!IS_NAN(aValues[0]))SET_NAN(aValues[0]);
		*pOk=0;
		}


return aValues[0];
}

//*****************************************************************************
//*
//*		CalcGenerate
//*
//*****************************************************************************
//	Dekotiert eine Formel
//	pData		: Daten fr Calc-Generator
//	pEquation	: Text der Formel
//	pErrorPos	: Hier wird ein Zeiger
//	Ergibt 1 wenn ok oder 0 bei einem Fehler
int CalcGenerate(CalcGenData *pData,const char *pEquation,const char **pErrorPos)
{
double			dValue;
int				iOk;


	iOk = 1;
	pData->pErrorPos = "";
	CalcGen(pEquation,&dValue,&iOk,-2,pData);

	if(pErrorPos)
		{
		*pErrorPos = (iOk)? "":pData->pErrorPos;
		}


return iOk;
}

//*****************************************************************************
//*
//*		CalcSetValues
//*
//*****************************************************************************
//	bergibt Werte an die Formel
//	pData		: Daten fr Calc-Generator
//	pValues		: Zeiger auf die Werte
//	uCount		: Anzahl der Werte
void CalcSetValues(CalcGenData *pData,const double *pValues,unsigned uCount)
{

	if(uCount>pData->uValueCnt)uCount=pData->uValueCnt;

	memcpy(pData->pValues,pValues,uCount*sizeof(double));

}

//*****************************************************************************
//*
//*		CalcExec
//*
//*****************************************************************************
//	Berechnet den Wert einer Actions-Liste
//	pData		: Daten fr Calc-Generator
//	pOk			: Hier wird 1 gespeichert wenn die Berechung ok war
//	Ergibt den berechneten Wert
double CalcExecute(CalcGenData *pData,int *pOk)
{

	if(pOk)*pOk=1;

return CalcExec(pData->pActions,pData->uActionCnt,pData->pValues,pOk);
}

