//*****************************************************************************
//*
//*
//*		CurveDrawer.cpp
//*
//*
//*****************************************************************************
//
//										start	= Erster  Pixel in der Scala
//										stop	= Letzter Pixel in der Scala
//										max		= Maximal Wert
//										min		= Minimal Wert
//	
//	Umrechnung lineare Skala:			pos		= (val-offset)*fact
//										val		= pos*fact+offset;
//										fact	= (start-stop)/(max-min)	
//										offset	= min-start/fact;
//
//	Umrechnung logarithmische Skala:	pos		= log(val)*fact-offset
//										val		= exp((pos+offset)/fact)
//										fact	= (stop-start)/log(max/min)	
//										offset	= log(min)*fact+start
//	
#include	<math.h>
#include	<stdio.h>
#include	"StringFunctions.h"
#include	"CurveDrawer.h"

#define 	FRAME_X				5							// X-Rand rund um Bild
#define 	FRAME_Y				5							// Y-Rand rund um Bild
#define 	FONT_HEIGHT_1		8							// Kleine Schrifthhe
#define 	FONT_HEIGHT_2		16							// Groe Schrifthhe
#define 	FONT_OFFSET_1		7							// Kleiner Schrift-Offset zur Mitte
#define 	FONT_OFFSET_2		13							// Groer Schrift-Offset zur Mitte
#define 	FONT_WIDTH_1		6							// Breite eines kleinen Buchstabens
#define 	FONT_WIDTH_2		14							// Breite eines groen  Buchstabens
#define 	STEP_SIZE_NULL		8							// Bereite der Nullmarkierung
#define 	STEP_SIZE_MAIN		5							// Bereite der Textmarkierung
#define 	STEP_SIZE_SUB		3							// Bereite der Zwischenmarkierung
#define 	STEP_SIZE_X			100							// Minimale Breite einer X-Beschriftung
#define 	STEP_SIZE_Y			50							// Minimale Breite einer Y-Beschriftung
#define 	STEP_OFFSET_X		8							// X-Abstand Skala Kurve
#define 	STEP_OFFSET_Y		8							// Y-Abstand Skala Kurve
#define 	STEP_OFFSET_FX		1							// X-Abstand Skala Beschriftung
#define 	STEP_OFFSET_FY		5							// Y-Abstand Skala Beschriftung
#define 	SHADOW_OFFSET_X		2							// X-Abstand zwischen Schatten und Schrift
#define 	SHADOW_OFFSET_Y		3							// Y-Abstand zwischen Schatten und Schrift
#define 	SHADOW_ALPHA		10							// Alfablending-Wert fr Schatten

#define 	COLOR_BACKPLANE		BGR(255,255,255)			// Farbe fr den Hintergrund 
#define 	COLOR_CURVEPLANE	BGR(247,247,247)			// Farbe fr den Kurvengrund 
#define 	COLOR_CURVEGRID		BGR(240,240,240)			// Farbe fr den Kurvengitter
#define 	COLOR_SCALA			BGR(0,0,0)					// Farbe fr die Skalen-Striche
#define 	COLOR_SCALA_NULL	BGR(200,200,200)			// Farbe fr die Skalen-Striche-Null
#define 	COLOR_SCALATEXT_X	BGR(0,0,0)					// Farbe fr die Skalen-Texte fr X
#define 	COLOR_SCALATEXT_Y1	BGR(0,200,0)				// Farbe fr die Skalen-Texte fr Y1
#define 	COLOR_SCALATEXT_Y2	BGR(200,0,0)				// Farbe fr die Skalen-Texte fr Y2
#define 	COLOR_SCALATEXT_Y3	BGR(0,0,200)				// Farbe fr die Skalen-Texte fr Y3
#define 	COLOR_CURVE_Y1		BGR(0,255,0)				// Farbe fr die Kurve Y1
#define 	COLOR_CURVE_Y2		BGR(255,0,0)				// Farbe fr die Kurve Y2
#define 	COLOR_CURVE_Y3		BGR(0,0,255)				// Farbe fr die Kurve Y3
#define 	COLOR_CURVE_SHADOW	BGR(220,220,220)			// Farbe fr die Kurvenschatten
#define 	COLOR_MARKER_1		BGR(160,160,0)				// Farbe fr die Marker
#define 	COLOR_MARKER_2		BGR(0,160,0)				// Farbe fr die Marker
#define 	COLOR_MARKER_3		BGR(160,0,0)				// Farbe fr die Marker
#define 	COLOR_MARKER_4		BGR(0,0,222)				// Farbe fr die Marker

#define		IS_NAN(a)		((((unsigned*)(&(a)))[1]&0x7FFF0000)>=0x7FF00000)

static unsigned char aFont1[]=
	{
	#include "Image/Fonts/Arial8.h"
	};

static unsigned char aFont2[]=
	{
	#include "Image/Fonts/Arial16.h"
	};

static unsigned aMarkerColors[]=
	{
	COLOR_MARKER_1,			
	COLOR_MARKER_2,
	COLOR_MARKER_3,
	COLOR_MARKER_4,
	};

static unsigned	uDrawTurn = 0;


//*****************************************************************************
//*
//*		CurveColor
//*
//*****************************************************************************
//	Die Farbe einer Kurve abfragen
//	iCurve		: Ist die Kurvennummer (1=X 2=Y1 3=Y2...)
//				  Marker 0...-n	
int CurveColor(int iCurve)
{

	if(iCurve<=0)
		{
		iCurve = -iCurve-1;
		return BitmapImage::SwapRGB(aMarkerColors[iCurve]);
		}

	if(iCurve==2)return BitmapImage::SwapRGB(COLOR_CURVE_Y1);
	if(iCurve==3)return BitmapImage::SwapRGB(COLOR_CURVE_Y2);
	if(iCurve==4)return BitmapImage::SwapRGB(COLOR_CURVE_Y3);


return RGB(0,0,0);
}

//*****************************************************************************
//*
//*		CurveCalcLinear
//*
//*****************************************************************************
//	Berechnet die linearen Schritte fr eine Skalierung
//	pScala			: Zeiger auf die Scala Daten
//	iSteps			: Unterteilung der gesammten Skala
//	Ergibt die zustzlichen Kommastellen
static int CurveCalcLinear(CurveScala *pScala,int iSteps)
{
double		dComma;
double		dSteps;
double		dValue;
double		dDelta;
int			iAdd;


	   dDelta = pScala->dMax-pScala->dMin;
	if(dDelta<=0.0)											// Prfe den Anzeigebereich
		{
		dDelta=1.0;
		if(pScala->dMin>1.0e06)pScala->dMin=0;
		pScala->dMax = pScala->dMin+dDelta;	
		}	
	

	dComma	= log10(dDelta/(double)iSteps);
	dValue	= floor(dComma);
	dComma -= dValue+0.02;
	dValue  = pow(10.0,dValue);

	if(dComma<=0.000000000)									// 1.0 Schritte
		{
		dSteps = dValue;
		iAdd   = 0;
		}
	else if(dComma<0.301029995)								// 2.0 Schritte
		{
		dSteps = dValue*2.0;
		iAdd   = 0;
		}
	else if(dComma<0.397940008)								// 2.5 Schritte
		{
		dSteps = dValue*2.5;
		iAdd   = 1;
		}
	else{													// 5.0 Schritte
		dSteps = dValue*5.0;
		iAdd   = 0;
		}


	dValue			  = ceil(pScala->dMin/dSteps-0.00001);
	pScala->iIsLog	  = 0;
	pScala->iStepAt	  = 0;	
	pScala->iStepCnt  = 1;
	pScala->aSteps[0] = dSteps;
	pScala->dStart	  = dValue*dSteps;
	pScala->dScale	  = pScala->dMax;
	pScala->dScale   -= pScala->dMin;
	pScala->dScale	 /= (double)(pScala->iSize)-1.25;
	pScala->dFact	  = 1.0/pScala->dScale;


return iAdd;
}

//*****************************************************************************
//*
//*		CurveCalcLog
//*
//*****************************************************************************
//	Berechnet die logarithmischen Schritte fr eine Skalierung
//	pScala			: Zeiger auf die Scala Daten
//	iSteps			: Breite der Skala in Ounkten
//	Ergibt die zustzlichen Kommastellen bzw. -3 bei linear Umschaltung sonst 
//						-1 = keine Kommastelle
//						-2 = eine Kommastelle
//						-3 = linear Umschaltung 
static int CurveCalcLog(CurveScala *pScala,int iSteps)
{
double		dComma;
double		dSteps;
double		dValue;
double		dDelta;
int			iAdd;
int			iPos;


	   dValue = pScala->dMin;
	if(dValue<=1.0e-15)return -3;
	   dComma = pScala->dMax;
	if(dComma<=1.0e-15)return -3;
	   dDelta = log10(dComma/dValue);
	   dDelta/= (double)iSteps;
	if(dDelta<=0.06)return -3;

	dComma	= dDelta;
	dValue	= floor(dComma);
	dComma -= dValue+0.02;
	dSteps  = pow(10.0,dValue);

	if(dComma<0.17 && dValue>=1.0)							// 10,100 Schritte
		{													
		iAdd				= -1;
		pScala->iStepCnt	=  1;
		pScala->aSteps[0]	=  1.0;
		pScala->aSteps[1]	= 10.0;
		}
	else if(dComma<0.22)									// 10,16,25,40,60,100 Schritte
		{
		iAdd				= -2;
		pScala->iStepCnt	=  5;
		pScala->aSteps[0]	=  1.0;
		pScala->aSteps[1]	=  1.6;
		pScala->aSteps[2]	=  2.5;
		pScala->aSteps[3]	=  4.0;
		pScala->aSteps[4]	=  6.0;
		pScala->aSteps[5]	= 10.0;
		}
	else if(dComma<0.25)									// 10,18,30,56,100 Schritte
		{
		iAdd				= -2;
		pScala->iStepCnt	=  4;
		pScala->aSteps[0]	=  1.0;
		pScala->aSteps[1]	=  1.8;
		pScala->aSteps[2]	=  3.0;
		pScala->aSteps[3]	=  5.6;
		pScala->aSteps[4]	= 10.0;
		}
	else if(dComma<0.33)									// 10,20,50,100 Schritte
		{
		iAdd				= -1;
		pScala->iStepCnt	=  3;
		pScala->aSteps[0]	=  1.0;
		pScala->aSteps[1]	=  2.0;
		pScala->aSteps[2]	=  5.0;
		pScala->aSteps[3]	= 10.0;
		}
	else if(dComma<0.49)									// 10,30,100 Schritte
		{
		iAdd				= -1;
		pScala->iStepCnt	=  2;
		pScala->aSteps[0]	=  1.0;
		pScala->aSteps[1]	=  3.0;
		pScala->aSteps[2]	= 10.0;
		}
	else{													// 10,100 Schritte
		iAdd				= -1;
		pScala->iStepCnt	=  1;
		pScala->aSteps[0]	=  1.0;
		pScala->aSteps[1]	= 10.0;
		}
	
	for(iPos=0;iPos<=pScala->iStepCnt;iPos++)				// Faktoren anfgen
		{
		pScala->aSteps[iPos] *= pow(dSteps,iPos);
		}
	
	dValue	= pow(10.0,floor(log10(pScala->dMin)));

	for(iPos=0;iPos<pScala->iStepCnt;iPos++)				// Suche Startposition
		{
		if(dValue*pScala->aSteps[iPos]*1.00001>=pScala->dMin)break;
		}

	if(iPos>=pScala->iStepCnt)
		{
		dValue *= pScala->aSteps[iPos];	
		iPos    = 0;
		}
	
	pScala->dStart	= dValue*pScala->aSteps[iPos];
	pScala->iStepAt = iPos;
	pScala->iIsLog  = 1;
	pScala->dFact   = log(pScala->dMax /pScala->dMin);
	pScala->dScale  = log(pScala->dMin)*pScala->dFact;


return iAdd;
}

//*****************************************************************************
//*
//*		ScalaText
//*
//*****************************************************************************
//	Erzeugt einen Text fr einen Skalenwert
static const char *ScalaText(CurveScala *pData,char *pBuffer,double dValue)
{
double		dBase;
double		dCalc;
int			iBase;
int			iLen;
int			iMax;
int			iNum;


	   iBase = pData->iBase;
	if(iBase>=0)
		{
		if(pData->iComma)
			{
			iBase -= (pData->iComma+2)/3;
			}

		iMax  = pData->iPlaces;
		iMax -= pData->iComma;
		if(pData->iComma)iMax--;
		if(iMax<2)iMax=2;
		
		dBase = pow(1000.0,(double)(iBase-8));
		dCalc = dValue/dBase;

		if(dCalc<0.01 && dCalc>-0.01)						// Bei Null eine Verkrzung verhintern
			{
			   iNum = (int)log10(pData->aSteps[0]/dBase);
			if(iNum>=iMax)iBase++;
			}

		dCalc+= (dCalc<0.0)? -0.1:0.1;
		iLen  = vtostr((__int64)dCalc,pBuffer,VTS_SIGNED|VTS_ADDSUFFIX|VTS_CONSTSIZE|(iMax<<8)|(pData->iComma<<4)|(iBase<<12));

		if(iLen>0 && pBuffer[iLen-1]==' ')pBuffer[iLen-1]=0;
		
		return pBuffer+strwlen(pBuffer,0xE0);
		}


	iBase = 8;

	while(fabs(dValue)<9999.999)
		{
		dBase  = floor(dValue);
		dBase -= dValue;
		
		if(dBase ==0)break;
		if(dValue==0)break;
		if(iBase ==0)break;

		dValue *= 1000.0;
		iBase  -= 1;
		}

	while(fabs(dValue)>9999999.0)
		{
		if(dValue== 0)break;
		if(iBase ==15)break;

		dValue /= 1000.0;
		iBase  += 1;
		}

	if(pData->iComma && dValue<9999.0 && dValue>=1000.0 && fmod(dValue,100.0)==0)
		{
		pData->iComma=1; 
		iMax = 2;
		}
	else{
		iMax = 5;
		}
		
	
	dCalc = (dValue<0.0)? dValue-0.1:dValue+0.1;
	iLen  = vtostr((__int64)dCalc,pBuffer,VTS_SIGNED|VTS_ADDSUFFIX|VTS_SHRINKCOMMA|(iMax<<8)|(pData->iComma<<4)|(iBase<<12));

	if(iLen>0 && pBuffer[iLen-1]==' ')pBuffer[iLen-1]=0;



return pBuffer;
}

//*****************************************************************************
//*
//*		CurveDrawScala
//*
//*****************************************************************************
//	Zeichnet eine Skala auf ein Bild
void CurveDrawScala(CurveScala *pData,BitmapImage *pImage)
{
char		cText[64];
double		dVal;
double		dSub;
double		dBit;
double		dMax;
double		dMin;
double		dLog;
int			iPos;
int			iLen;
int			iCol;
int			iMul;
int			iIdx;
int			iNum;
int			iSub;
int			iAdd;
int			iOk;

	
	iMul	=  pData->iMul;
	iAdd	= (pData->iMul==2)? 1:0;
	iOk		= (pData->iPosX>=0 && pData->iPosY>=0)? 1:0;


	switch(pData->iMode)
		{
	case -2:												// Rechte kompinierte Y-Skala nur mit Text
	case -3:
			if(pData->iPosF>=0)								
				{
				iLen = strlcpy(cText,pData->cVarName,sizeof(cText));
				
				if(pData->cUnitName[0])
					{
					iLen += sprintf(cText+iLen," [%s]",pData->cUnitName);
					}

				iCol = (pData->iMode==-3)? COLOR_SCALATEXT_Y3:COLOR_SCALATEXT_Y2;

				pImage->SetAlign(0); 
				pImage->SetPixelFunction((iMul==2)? TEXT_SMOTH:TEXT_SMOTH,iCol,0);
				pImage->Print(pData->iPosF,FRAME_Y*iMul,cText);	
				}

			break;
			   
	case 0:													// X-Scala

			if(iOk)											// Hauplinie
				{
				pImage->Box(pData->iPosX,pData->iPosY,pData->iSize,iMul,COLOR_SCALA);
				}
															// Nullline
			if(pData->iPosNull>=0 && pData->iPosNull!=pData->iPosY)							
				{
				pImage->Box(pData->iPosX,pData->iPosNull,pData->iSize,iMul,COLOR_SCALA_NULL);
				}

			pImage->SetAlign(2); 
			pImage->SetFont((iMul==2)? aFont2:aFont1); 
			pImage->SetPixelFunction(TEXT_NONE,0,1); 
			pImage->SetPixelFunction((iMul==2)? TEXT_SMOTH:TEXT_SMOTH,COLOR_SCALATEXT_X,0);

			iLen = strlcpy(cText,pData->cVarName,sizeof(cText));
			if(pData->cUnitName[0])
				{
				iLen += sprintf(cText+iLen," [%s]",pData->cUnitName);
				}
			
			if(pData->iPosF>=0)			
				{
				iPos  = pData->iPosX-STEP_OFFSET_X*iMul;
				iPos -= (iMul==2)? FONT_WIDTH_2*2:FONT_WIDTH_1*2; 	
				pImage->Print(iPos,pData->iPosF,cText);	
				}

			pImage->SetAlign(1); 

			if(pData->iIsLog)								// Logarithmische Teilung
				{					
				dLog	= pData->iSize-iMul;
				dLog   /= pData->dFact;
				dVal	= pData->dStart;
				dMax	= pData->dMax*1.00001;
				dMin	= pData->dMin*0.99999;
				iIdx	= pData->iStepAt;
				
				for(iNum=0;dVal<=dMax && iNum<64;iNum++)
					{
					   iPos = (int)(log(dVal/dMin)*dLog);
					if(iMul==2)iPos&=~1;

					if(iPos>=0 && iPos<pData->iSize)		// Teilungen	
						{
						iLen  =  STEP_SIZE_MAIN*iMul;
						
						if(pData->iPosNull>=0)				// Teillung auf Nullline
							{
							pImage->Box(pData->iPosX+iPos,pData->iPosNull     ,iMul, iLen,COLOR_SCALA_NULL);
							pImage->Box(pData->iPosX+iPos,pData->iPosNull+iAdd,iMul,-iLen,COLOR_SCALA_NULL);
							}

					   if(iOk)								// Haupunterteilungen	
							{
							pImage->Box(pData->iPosX+iPos,pData->iPosY,iMul,iLen,COLOR_SCALA);

							if(pData->iDoubleMark)
								{
								pImage->Box(pData->iPosX+iPos,pData->iPosY+iAdd,iMul,-iLen,COLOR_SCALA);
								}
							}
						}

					if(iPos>=0 && iPos<pData->iSize)		// Beschriftung
					if(pData->iPosF>=0)
						{
						pImage->Print(pData->iPosX+iPos,pData->iPosF,ScalaText(pData,cText,dVal));
						}

					dVal /= pData->aSteps[iIdx];iIdx++;
					dVal *=	pData->aSteps[iIdx];

					if(iIdx>=pData->iStepCnt)iIdx=0;
					}
				}
			else{											// Lineare Teilung
				dVal	= pData->dStart;
				dBit	= pData->dScale*0.25;
				dSub	= pData->dStart-pData->aSteps[0]*0.5;
				dMax	= pData->dMax  +pData->aSteps[0]*0.6;
				dMin	= pData->dMin  -dBit;

				for(iNum=0;dVal<=dMax && iNum<64;iNum++)
					{
					   iPos = (int)((dVal-dMin)*pData->dFact);
					if(iMul==2)iPos&=~1;

					if(iPos>=0 && iPos<pData->iSize)		// Teilungen	
						{
						iLen  = (dVal<dBit && dVal>=-dBit)? STEP_SIZE_NULL:STEP_SIZE_MAIN;						
						iLen *=  iMul;
						
						if(pData->iPosNull>=0)				// Teillung auf Nullline
							{
							pImage->Box(pData->iPosX+iPos,pData->iPosNull     ,iMul, iLen,COLOR_SCALA_NULL);
							pImage->Box(pData->iPosX+iPos,pData->iPosNull+iAdd,iMul,-iLen,COLOR_SCALA_NULL);
							}

						if(iOk)								// Haupunterteilungen	
							{
							pImage->Box(pData->iPosX+iPos,pData->iPosY,iMul,iLen,COLOR_SCALA);

							if(pData->iDoubleMark)
								{
								pImage->Box(pData->iPosX+iPos,pData->iPosY+iAdd,iMul,-iLen,COLOR_SCALA);
								}
							}
						}

					if(iPos>=0 && iPos<pData->iSize)		// Beschriftung
					if(pData->iPosF>=0)
						{
						pImage->Print(pData->iPosX+iPos,pData->iPosF,ScalaText(pData,cText,dVal));
						}


					   iPos = (int)((dSub-dMin)/pData->dScale);
					if(iMul==2)iPos&=~1;

					if(iOk && iPos>=0 && iPos<pData->iSize)	// Nebenunterteilungen
						{
						iLen = STEP_SIZE_SUB*iMul;						

						pImage->Box(pData->iPosX+iPos,pData->iPosY,iMul,iLen,COLOR_SCALA);

						if(pData->iDoubleMark)
							{
							pImage->Box(pData->iPosX+iPos,pData->iPosY+iAdd,iMul,-iLen,COLOR_SCALA);
							}
						}

					dVal += pData->aSteps[0];
					dSub += pData->aSteps[0];
					}
				}

			break;

		case 1:												// Linke Y-Skala										
		case 2:

			if(iOk)											// Hauplinie
				{
				pImage->Box(pData->iPosX,pData->iPosY+iAdd,iMul,-pData->iSize,COLOR_SCALA);
				}
															// Nullline
			if(pData->iPosNull>=0 && pData->iPosNull!=pData->iPosX)
				{
				pImage->Box(pData->iPosNull,pData->iPosY+iAdd,iMul,-pData->iSize,COLOR_SCALA_NULL);
				}

			pImage->SetAlign(2); 
			pImage->SetFont((iMul==2)? aFont2:aFont1); 
			pImage->SetPixelFunction(TEXT_NONE,0,1); 
			pImage->SetPixelFunction((iMul==2)? TEXT_SMOTH:TEXT_SMOTH,COLOR_SCALATEXT_Y1,0);
			iSub = (iMul==2)? FONT_OFFSET_2:FONT_OFFSET_1;

			
			if(pData->iPosF>=0)			
				{
				iLen = strlcpy(cText,pData->cVarName,sizeof(cText));
				if(pData->cUnitName[0])
					{
					iLen += sprintf(cText+iLen," [%s]",pData->cUnitName);
					}

				pImage->Print(pData->iPosF,FRAME_Y*iMul,cText);	
				}

			if(pData->iMode==2)
				{
				pImage->SetPixelFunction((iMul==2)? TEXT_SMOTH:TEXT_SMOTH,COLOR_SCALATEXT_X,0);
				}

			if(pData->iIsLog)								// Logarithmische Teilung
				{
				dLog	= pData->iSize-iMul;
				dLog   /= pData->dFact;
				dVal	= pData->dStart;
				dMax	= pData->dMax*1.00001;
				dMin	= pData->dMin*0.99999;
				iIdx	= pData->iStepAt;
				
				for(iNum=0;dVal<=dMax && iNum<64;iNum++)
					{
					   iPos = (int)(log(dVal/dMin)*dLog);
					if(iMul==2)iPos&=~1;

					if(iPos>=0 && iPos<pData->iSize)		// Teilungen		
						{
						iLen  =  STEP_SIZE_MAIN*iMul;
						
						if(pData->iPosNull>=0)				// Teillung auf Nullline
							{
							pImage->Box(pData->iPosNull     ,pData->iPosY-iPos, iLen,iMul,COLOR_SCALA_NULL);
							pImage->Box(pData->iPosNull+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA_NULL);
							}

						if(iOk)								// Haupunterteilungen	
							{
							pImage->Box(pData->iPosX+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA);

							if(pData->iDoubleMark)
								{
								pImage->Box(pData->iPosX,pData->iPosY-iPos,iLen,iMul,COLOR_SCALA);
								}
							}
						}

					if(iPos>=0 && iPos<pData->iSize)		// Beschriftung
					if(pData->iPosF>=0)
						{
						pImage->Print(pData->iPosF,pData->iPosY-iPos-iSub,ScalaText(pData,cText,dVal));
						}

					dVal /= pData->aSteps[iIdx];iIdx++;
					dVal *=	pData->aSteps[iIdx];

					if(iIdx>=pData->iStepCnt)iIdx=0;
					}
				}
			else{											// Lineare Teilung
				dVal	= pData->dStart;
				dBit	= pData->dScale*0.25;
				dSub	= pData->dStart-pData->aSteps[0]*0.5;
				dMax	= pData->dMax  +pData->aSteps[0]*0.6;
				dMin	= pData->dMin  -dBit;

				for(iNum=0;dVal<=dMax && iNum<64;iNum++)
					{
					   iPos = (int)((dVal-dMin)*pData->dFact);
					if(iMul==2)iPos&=~1;

					if(iPos>=0 && iPos<pData->iSize)		// Teilungen	
						{
						iLen  = (dVal<dBit && dVal>=-dBit)? STEP_SIZE_NULL:STEP_SIZE_MAIN;						
						iLen *=  iMul;

						if(pData->iPosNull>=0)				// Teillung auf Nullline
							{
							pImage->Box(pData->iPosNull     ,pData->iPosY-iPos, iLen,iMul,COLOR_SCALA_NULL);
							pImage->Box(pData->iPosNull+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA_NULL);
							}

						if(iOk)								// Haupunterteilungen	
							{
							pImage->Box(pData->iPosX+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA);

							if(pData->iDoubleMark)
								{
								pImage->Box(pData->iPosX,pData->iPosY-iPos,iLen,iMul,COLOR_SCALA);
								}
							}
						}

					if(iPos>=0 && iPos<pData->iSize)		// Beschriftung
					if(pData->iPosF>=0)
						{
						pImage->Print(pData->iPosF,pData->iPosY-iPos-iSub,ScalaText(pData,cText,dVal));
						}


					   iPos = (int)((dSub-dMin)/pData->dScale);
					if(iMul==2)iPos&=~1;

					if(iOk && iPos>=0 && iPos<pData->iSize)	// Nebenunterteilungen
						{
						iLen = STEP_SIZE_SUB*iMul;						

						pImage->Box(pData->iPosX+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA);

						if(pData->iDoubleMark)
							{
							pImage->Box(pData->iPosX,pData->iPosY-iPos,iLen,iMul,COLOR_SCALA);
							}
						}

					dVal += pData->aSteps[0];
					dSub += pData->aSteps[0];
					}
				}

			break;

		case 3:												// Rechte Y-Skala
		case 4:

			if(iOk)											// Hauplinie
				{
				pImage->Box(pData->iPosX,pData->iPosY+iAdd,iMul,-pData->iSize,COLOR_SCALA);
				}

			if(pData->iPosNull>=0)							// Nullline
				{
				pImage->Box(pData->iPosNull,pData->iPosY+iAdd,iMul,-pData->iSize,COLOR_SCALA_NULL);
				}

			iCol =(pData->iMode==4)? COLOR_SCALATEXT_Y3:COLOR_SCALATEXT_Y2;
			pImage->SetAlign(0); 
			pImage->SetFont((iMul==2)? aFont2:aFont1); 
			pImage->SetPixelFunction(TEXT_NONE,0,1); 
			pImage->SetPixelFunction((iMul==2)? TEXT_SMOTH:TEXT_SMOTH,iCol,0);
			iSub = (iMul==2)? FONT_OFFSET_2:FONT_OFFSET_1;

			iLen = strlcpy(cText,pData->cVarName,sizeof(cText));
			if(pData->cUnitName[0])
				{
				iLen += sprintf(cText+iLen," [%s]",pData->cUnitName);
				}
			
			if(pData->iPosF>=0)			
				{
				pImage->Print(pData->iPosF,FRAME_Y*iMul,cText);	
				}

			if(pData->iIsLog)								// Logarithmische Teilung
				{
				dLog	= pData->iSize-iMul;
				dLog   /= pData->dFact;
				dVal	= pData->dStart;
				dMax	= pData->dMax*1.00001;
				dMin	= pData->dMin*0.99999;
				iIdx	= pData->iStepAt;
				
				for(iNum=0;dVal<=dMax && iNum<64;iNum++)
					{
					   iPos = (int)(log(dVal/dMin)*dLog);
					if(iMul==2)iPos&=~1;

					if(iPos>=0 && iPos<pData->iSize)		// Teilungen		
						{
						iLen  =  STEP_SIZE_MAIN*iMul;
						
						if(pData->iPosNull>=0)				// Teillung auf Nullline
							{
							pImage->Box(pData->iPosNull     ,pData->iPosY-iPos, iLen,iMul,COLOR_SCALA_NULL);
							pImage->Box(pData->iPosNull+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA_NULL);
							}

						if(iOk)								// Haupunterteilungen	
							{
							pImage->Box(pData->iPosX,pData->iPosY-iPos,iLen,iMul,COLOR_SCALA);

							if(pData->iDoubleMark)
								{
								pImage->Box(pData->iPosX+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA);
								}
							}
						}

					if(iPos>=0 && iPos<pData->iSize)		// Beschriftung
					if(pData->iPosF>=0)
						{
						pImage->Print(pData->iPosF,pData->iPosY-iPos-iSub,ScalaText(pData,cText,dVal));
						}

					dVal /= pData->aSteps[iIdx];iIdx++;
					dVal *=	pData->aSteps[iIdx];

					if(iIdx>=pData->iStepCnt)iIdx=0;
					}
				}
			else{											// Lineare Teilung
				dVal	= pData->dStart;
				dBit	= pData->dScale*0.25;
				dSub	= pData->dStart-pData->aSteps[0]*0.5;
				dMax	= pData->dMax  +pData->aSteps[0]*0.6;
				dMin	= pData->dMin  -dBit;

				for(iNum=0;dVal<=dMax && iNum<64;iNum++)
					{
					   iPos = (int)((dVal-dMin)*pData->dFact);
					if(iMul==2)iPos&=~1;

					if(iPos>=0 && iPos<pData->iSize)		// Teilungen	
						{
						iLen  = (dVal<dBit && dVal>=-dBit)? STEP_SIZE_NULL:STEP_SIZE_MAIN;						
						iLen *=  iMul;

						if(pData->iPosNull>=0)				// Teillung auf Nullline
							{
							pImage->Box(pData->iPosNull     ,pData->iPosY-iPos, iLen,iMul,COLOR_SCALA_NULL);
							pImage->Box(pData->iPosNull+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA_NULL);
							}

						if(iOk)								// Haupunterteilungen	
							{
							pImage->Box(pData->iPosX,pData->iPosY-iPos,iLen,iMul,COLOR_SCALA);

							if(pData->iDoubleMark)
								{
								pImage->Box(pData->iPosX+iAdd,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA);
								}
							}
						}

					if(iPos>=0 && iPos<pData->iSize)		// Beschriftung
					if(pData->iPosF>=0)
						{
						pImage->Print(pData->iPosF,pData->iPosY-iPos-iSub,ScalaText(pData,cText,dVal));
						}


					   iPos = (int)((dSub-dMin)/pData->dScale);
					if(iMul==2)iPos&=~1;

					if(iOk && iPos>=0 && iPos<pData->iSize)	// Nebenunterteilungen
						{
						iLen = STEP_SIZE_SUB*iMul;						

						pImage->Box(pData->iPosX,pData->iPosY-iPos,iLen,iMul,COLOR_SCALA);

						if(pData->iDoubleMark)
							{
							pImage->Box(pData->iPosX,pData->iPosY-iPos,-iLen,iMul,COLOR_SCALA);
							}
						}

					dVal += pData->aSteps[0];
					dSub += pData->aSteps[0];
					}
				}
		
			break;

		}
	


}

//*****************************************************************************
//*
//*		CurveDrawGrid
//*
//*****************************************************************************
void CurveDrawGrid(CurveData *pData,BitmapImage *pImage)
{
CurveScala *pScala;
double		dVal;
double		dSub;
double		dBit;
double		dMax;
double		dMin;
double		dLog;
int			iPos;
int			iLen;
int			iMul;
int			iIdx;
int			iNum;
int			iAdd;


	
	iMul	=  pData->aScala[0].iMul;
	iAdd	= (pData->aScala[0].iMul==2)? 1:0;
	pScala	=  pData->aScala+0;								// X-Scala

	if(pScala->iHasGrid)
		{
		if(pScala->iIsLog)									// Logarithmische Teiling
			{
			dLog	= pScala->iSize-iMul;
			dLog   /= pScala->dFact;
			dVal	= pScala->dStart;
			dMax	= pScala->dMax*1.00001;
			dMin	= pScala->dMin*0.99999;
			iIdx	= pScala->iStepAt;
			
			for(iNum=0;dVal<=dMax && iNum<64;iNum++)
				{
				   iPos = (int)(log(dVal/dMin)*dLog);
				if(iMul==2)iPos&=~1;

				if(iPos>=0 && iPos<pScala->iSize)			// Teilungen	
					{
					iLen  =  STEP_SIZE_MAIN*iMul;
															// Gitterlinien
					pImage->Box(pScala->iPosX+iPos,pScala->iPosGrid+iAdd,iMul,-pScala->iHasGrid,COLOR_CURVEGRID);
					}

				dVal /= pScala->aSteps[iIdx];iIdx++;
				dVal *=	pScala->aSteps[iIdx];

				if(iIdx>=pScala->iStepCnt)iIdx=0;
				}
			}
		else{												// Lineare Teilung
			dVal	= pScala->dStart;
			dBit	= pScala->dScale*0.25;
			dSub	= pScala->dStart-pScala->aSteps[0]*0.5;
			dMax	= pScala->dMax  +pScala->aSteps[0]*0.6;
			dMin	= pScala->dMin  -dBit;

			for(iNum=0;dVal<=dMax && iNum<64;iNum++)
				{
				   iPos = (int)((dVal-dMin)/pScala->dScale);
				if(iMul==2)iPos&=~1;

				if(iPos>=0 && iPos<pScala->iSize)			// Teilungen	
					{
					iLen  = (dVal<dBit && dVal>=-dBit)? STEP_SIZE_NULL:STEP_SIZE_MAIN;						
					iLen *=  iMul;
															// Gitterlinien
					pImage->Box(pScala->iPosX+iPos,pScala->iPosGrid+iAdd,iMul,-pScala->iHasGrid,COLOR_CURVEGRID);
					}

				dVal += pScala->aSteps[0];
				dSub += pScala->aSteps[0];
				}
			}
		}

	pScala	=  pData->aScala+1;								// Y1-Skala										

	if(pScala->iHasGrid)						
		{
		if(pScala->iIsLog)									// Logarithmische Teilung
			{
			dLog	= pScala->iSize-iMul;
			dLog   /= pScala->dFact;
			dVal	= pScala->dStart;
			dMax	= pScala->dMax*1.00001;
			dMin	= pScala->dMin*0.99999;
			iIdx	= pScala->iStepAt;
			
			for(iNum=0;dVal<=dMax && iNum<64;iNum++)
				{
				   iPos = (int)(log(dVal/dMin)*dLog);
				if(iMul==2)iPos&=~1;

				if(iPos>=0 && iPos<pScala->iSize)			// Teilungen		
					{
					iLen  =  STEP_SIZE_MAIN*iMul;			// Gitterlinien
					
					pImage->Box(pScala->iPosGrid,pScala->iPosY-iPos,pScala->iHasGrid,iMul,COLOR_CURVEGRID);
					}

				dVal /= pScala->aSteps[iIdx];iIdx++;
				dVal *=	pScala->aSteps[iIdx];

				if(iIdx>=pScala->iStepCnt)iIdx=0;
				}
			}
		else{												// Lineare Teilung
			dVal	= pScala->dStart;
			dBit	= pScala->dScale*0.25;
			dSub	= pScala->dStart-pScala->aSteps[0]*0.5;
			dMax	= pScala->dMax  +pScala->aSteps[0]*0.6;
			dMin	= pScala->dMin  -dBit;

			for(iNum=0;dVal<=dMax && iNum<64;iNum++)
				{
				   iPos = (int)((dVal-dMin)/pScala->dScale);
				if(iMul==2)iPos&=~1;

				if(iPos>=0 && iPos<pScala->iSize)			// Teilungen	
					{
					iLen  = (dVal<dBit && dVal>=-dBit)? STEP_SIZE_NULL:STEP_SIZE_MAIN;						
					iLen *=  iMul;
															// Gitterlinien
					pImage->Box(pScala->iPosGrid,pScala->iPosY-iPos,pScala->iHasGrid,iMul,COLOR_CURVEGRID);
					}

				dVal += pScala->aSteps[0];
				dSub += pScala->aSteps[0];
				}
			}
		}	


}

//*****************************************************************************
//*
//*		SetPixelSmall
//*
//*****************************************************************************
//	Pixel-Funktion fr kleine Bilder
void FC SetPixelSmall(BitmapImage *pImage,unsigned uPosX,unsigned uPosY,unsigned uColor)
{
CurveData  *pData = (CurveData*)pImage->GetUserPtr();
unsigned	uPos;		

	   uPos	= uPosY-pData->uDrawSub;
	if(uPos>=(unsigned)pData->iSizeY)return;

	BitmapImage::SetTextPixelSmoth(pImage,uPosX+pData->iOffX,uPosY+pData->iOffY,uColor);

}

//*****************************************************************************
//*
//*		SetMarkerSmall
//*
//*****************************************************************************
//	Marker-Funktion fr kleine Bilder
void FC SetMarkerSmall(BitmapImage *pImage,unsigned uPosX,unsigned uPosY,unsigned uColor)
{
CurveData  *pData = (CurveData*)pImage->GetUserPtr();
unsigned	uPos;		


	uDrawTurn++;
	if(uDrawTurn&4)return;

	   uPos	= uPosY-pData->uDrawSub;
	if(uPos>=(unsigned)pData->iSizeY)return;


	BitmapImage::SetTextPixelSmoth(pImage,uPosX+pData->iOffX,uPosY+pData->iOffY,uColor);

}


//*****************************************************************************
//*
//*		SetMarkerLarge
//*
//*****************************************************************************
//	Marker-Funktion fr groe Bilder
void FC SetMarkerLarge(BitmapImage *pImage,unsigned uPosX,unsigned uPosY,unsigned uColor)
{
CurveData  *pData = (CurveData*)pImage->GetUserPtr();
unsigned	uPos;		
unsigned	uCol;


	uDrawTurn++;
	if(uDrawTurn&8)return;

	   uPos	= uPosY-pData->uDrawSub;
	if(uPos>=(unsigned)pData->iSizeY)return;

	uPosX += pData->iOffX;
	uPosY += pData->iOffY;

	pImage->SetPixel(uPosX  ,uPosY  ,uColor);
	pImage->SetPixel(uPosX+1,uPosY  ,uColor);
	pImage->SetPixel(uPosX  ,uPosY+1,uColor);
	pImage->SetPixel(uPosX+1,uPosY+1,uColor);

	uColor	= (uColor>>1)&0x007F7F7F;
	uCol	= (pImage->GetPixel(uPosX-1,uPosY  )>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX-1,uPosY  ,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX-1,uPosY+1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX-1,uPosY+1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY  )>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+2,uPosY  ,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY+1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+2,uPosY+1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY-1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX  ,uPosY-1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY-1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+1,uPosY-1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+2)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX  ,uPosY+2,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+2)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+1,uPosY+2,uColor+uCol);

}



//*****************************************************************************
//*
//*		SetPixelLarge
//*
//*****************************************************************************
//	Pixel-Funktion fr groe Bilder
void FC SetPixelLarge(BitmapImage *pImage,unsigned uPosX,unsigned uPosY,unsigned uColor)
{
CurveData  *pData = (CurveData*)pImage->GetUserPtr();
unsigned	uPos;		
unsigned	uCol;

	   uPos	= uPosY-pData->uDrawSub;
	if(uPos>=(unsigned)pData->iSizeY)return;

	uPosX += pData->iOffX;
	uPosY += pData->iOffY;

	pImage->SetPixel(uPosX  ,uPosY  ,uColor);
	pImage->SetPixel(uPosX+1,uPosY  ,uColor);
	pImage->SetPixel(uPosX  ,uPosY+1,uColor);
	pImage->SetPixel(uPosX+1,uPosY+1,uColor);

	uColor	= (uColor>>1)&0x007F7F7F;
	uCol	= (pImage->GetPixel(uPosX-1,uPosY  )>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX-1,uPosY  ,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX-1,uPosY+1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX-1,uPosY+1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY  )>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+2,uPosY  ,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY+1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+2,uPosY+1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY-1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX  ,uPosY-1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY-1)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+1,uPosY-1,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+2)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX  ,uPosY+2,uColor+uCol);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+2)>>1)&0x007F7F7F;
			   pImage->SetPixel(uPosX+1,uPosY+2,uColor+uCol);

}


//*****************************************************************************
//*
//*		SetShadowLarge
//*
//*****************************************************************************
//	Pixel-Funktion fr groe Bilder
void FC SetShadowLarge(BitmapImage *pImage,unsigned uPosX,unsigned uPosY,unsigned uColor)
{
CurveData  *pData = (CurveData*)pImage->GetUserPtr();
unsigned	uPos;		
unsigned	uCol;

	   uPos	= uPosY-pData->uDrawSub;
	if(uPos>=(unsigned)pData->iSizeY)return;

	uPosX += pData->iOffX;
	uPosY += pData->iOffY;

	uColor	= (uColor>>3)&0x001F1F1F;

	uCol	= (pImage->GetPixel(uPosX  ,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY+1,uColor+uCol*7);


	uCol	= (pImage->GetPixel(uPosX-1,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-1,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX-1,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-1,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+2,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+2,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY-1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY-1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY-1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY-1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+2)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY+2,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+2)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY+2,uColor+uCol*7);


	uCol	= (pImage->GetPixel(uPosX-2,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-2,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX-2,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-2,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+3,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+3,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+3,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+3,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY-2)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY-2,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY-2)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY-2,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+3)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY+3,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+3)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY+3,uColor+uCol*7);


	uCol	= (pImage->GetPixel(uPosX-3,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-3,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX-3,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-3,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+4,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+4,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+4,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+4,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY-3)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY-3,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY-3)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY-3,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+4)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY+4,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+4)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY+4,uColor+uCol*7);


	uCol	= (pImage->GetPixel(uPosX-4,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-4,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX-4,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-4,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+5,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+5,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+5,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+5,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY-4)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY-4,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY-4)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY-4,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+5)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY+5,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+5)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY+5,uColor+uCol*7);

}

//*****************************************************************************
//*
//*		SetShadowSmall
//*
//*****************************************************************************
//	Pixel-Funktion fr kleine Bilder
void FC SetShadowSmall(BitmapImage *pImage,unsigned uPosX,unsigned uPosY,unsigned uColor)
{
CurveData  *pData = (CurveData*)pImage->GetUserPtr();
unsigned	uPos;		
unsigned	uCol;


	   uPos	= uPosY-pData->uDrawSub;
	if(uPos>=(unsigned)pData->iSizeY)return;

	uPosX += pData->iOffX;
	uPosY += pData->iOffY;

	uColor	= (uColor>>3)&0x001F1F1F;

	uCol	= (pImage->GetPixel(uPosX  ,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY+1,uColor+uCol*7);


	uCol	= (pImage->GetPixel(uPosX-1,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-1,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX-1,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX-1,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY  )>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+2,uPosY  ,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+2,uPosY+1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+2,uPosY+1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY-1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY-1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY-1)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY-1,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX  ,uPosY+2)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX  ,uPosY+2,uColor+uCol*7);
	uCol	= (pImage->GetPixel(uPosX+1,uPosY+2)>>3)&0x001F1F1F;
			   pImage->SetPixel(uPosX+1,uPosY+2,uColor+uCol*7);


}

//*****************************************************************************
//*
//*		CurvePosToVal
//*
//*****************************************************************************
//	Rechnet eine Koordinate im Fenster in ein Skalenwert um
//	pData		: Zeiger auf die Kurvendaten
//	dVal		: Skalenwert 
//	iNum		: Skalennummer (Koord=0=X 1=Y1 2=Y2 3=Y3, Index 4=X0 5=Y10 6=Y20 7=Y30) 
//	pOk			: Wird auf 1 gesetzt wenn imm erlaubten Bereich
//	Ergibt den Skalenwert
double CurvePosToVal(CurveData *pData,int iPos,int iNum,int *pOk)
{
CurveScala *pScale;
double		dVal;


	

	switch(iNum)
		{
	case 0:	pScale= pData->aScala+0;
			iPos *= pScale->iMul;
			iPos  = iPos-pScale->iPosX;
			break;	

	case 1:	pScale= pData->aScala+1;
			iPos *= pScale->iMul;
			iPos  = pScale->iPosY-iPos;
			break;	

	case 2:	pScale= pData->aScala+2;
			iPos *= pScale->iMul;
			iPos  = pScale->iPosY-iPos;
			break;	

	case 3:	pScale= pData->aScala+3;
			iPos *= pScale->iMul;
			iPos  = pScale->iPosY-iPos;
			break;	

	case 4:	pScale= pData->aScala+0;
			iPos *= pScale->iMul;
			break;	

	case 5:	pScale= pData->aScala+1;
			iPos *= pScale->iMul;
			break;	

	case 6:	pScale= pData->aScala+2;
			iPos *= pScale->iMul;
			break;	

	case 7:	pScale= pData->aScala+3;
			iPos *= pScale->iMul;
			break;	

	default:return 0.0;
		}

	
	if(pOk)*pOk=(iPos>=0 && iPos<pScale->iSize)? 1:0;
	
	
	if(pScale->iIsLog)
		{
		dVal = exp((double)iPos+pScale->dScale)*pScale->dFact;
		}
	else{
		dVal = (double)iPos*pScale->dScale+pScale->dMin;
		}




return dVal;
}

//*****************************************************************************
//*
//*		CurveValToPos
//*
//*****************************************************************************
//	Rechnet einen Skalenwert in eine Koordinate im Fenster um
//	pData		: Zeiger auf die Kurvendaten
//	dVal		: Koordinate 
//	iNum		: Skalennummer (0=X 1=Y1 2=Y2 3=Y3, 4=X0 5=Y10 6=Y20 7=Y30) 
//	pOk			: Wird auf 1 gesetzt wenn imm erlaubten Bereich
//	Ergibt die Koordinate
double CurveValToPos(CurveData *pData,double dVal,int iNum,int *pOk)
{
CurveScala *pScale=pData->aScala+iNum%3;
double		dPos;
int			iPos;


	if(pScale->iIsLog)
		{
		dPos = log(dVal)*pScale->dFact-pScale->dScale;
		}
	else{
		dPos = (dVal-pScale->dMin)*pScale->dScale;
		}

	
	iPos  = (int)(dPos-0.01);

	if(pOk)*pOk=(iPos>=0 && iPos<pScale->iSize)? 1:0;


	switch(iNum)
		{
	case 0:	iPos = pScale->iPosX+iPos;
			break;	

	case 1:	iPos = pScale->iPosY-iPos;
			break;	

	case 2:	iPos = pScale->iPosY-iPos;
			break;	
	
	case 3:	iPos = pScale->iPosY-iPos;
			break;	
	
	case 4:	
	case 5:	
	case 6:	
	case 7:	iPos /=pScale->iMul;
			break;	
		}


return iPos;
}

//*****************************************************************************
//*
//*		CurveInit
//*
//*****************************************************************************
//	Initialisiert die Kurvendaten
//	pData		: Zeiger auf die Kurvendaten
//	pImage		: Zeiger auf das Image
//	uOptions	: Optionen (diehe CDR_???)
//	pMinMax		: Zeiger auf die Min/Max-Werte der Skalen
//	pNames		: Name der Variablen und Einheiten (z.B. "A|A|B|B|C|C|D|D")
//	Ergibt die Anzahl der zu berechnenten Punkte
int CurveInit(CurveData *pData,BitmapImage *pImage,unsigned uOptions,double *pMinMax,const char *pNames)
{
char		cText[64];
double		dComma;
double		dSteps;
double		dValue;
int			iCount;
int			iFirst;
int			iSteps;
int			iSizeX;
int			iSizeY;
int			iPosX;
int			iPosY;
int			iBase;
int			iChar;
int			iPosF;
int			iPos;
int			iIdx;
int			iNum;
int			iMul;
int			iVal;
int			iAdd;
int			iSub;
int			iOff;
int			iLen;

														
	iSizeX	= pImage->Width ();
	iSizeY	= pImage->Height();

	if(uOptions&CDR_DOUBLE)									// Image fr Doppelt so grozeichen	
		{
		if(pImage->IsRGB())
			{
			iSizeX *= 2;
			iSizeY *= 2;
			}
		else{
			uOptions &= ~CDR_DOUBLE;
			}
		}

	pData->uCompine = 0;
	pData->iBoxX	= iSizeX;
	pData->iBoxY	= iSizeY;
	iPosX			= FRAME_X;
	iPosY			= FRAME_Y;
	iMul			= (uOptions&CDR_DOUBLE)? 2:1;
	iFirst			= 0;
	iCount			= 0;
	
	if(!pNames)pNames="";



	for(iNum=0,iIdx=0;iNum<4;iNum++)						// Werte zuweisen
		{
		pData->aScala[iNum].iIsLog		= uOptions&(CDR_IS_LOG_X<<iNum);
		pData->aScala[iNum].iMul		= iMul;
		pData->aScala[iNum].iSize		= 200;
		pData->aScala[iNum].iNameSize	=  0;
		pData->aScala[iNum].iMode		= -1;
		pData->aScala[iNum].iPosX		= -1; 
		pData->aScala[iNum].iPosY		= -1; 
		pData->aScala[iNum].iPosF		= -1; 
		pData->aScala[iNum].dMin		= pMinMax[iNum*2+0];
		pData->aScala[iNum].dMax		= pMinMax[iNum*2+1];

		if(uOptions&(CDR_SCALA_X<<iNum))					// Ist die Skalar sichtbar
			{
			if(iNum==0)
				{
				pData->aScala[iNum].iMode = 0;				// X-Skala	
				}
			else if(uOptions&(CDR_HAS_Y1<<(iNum-1)))		
				{
				pData->aScala[iNum].iMode = 1;				// Y-Skala	
				if(!iFirst)iFirst=iNum;
				iCount++;
				}
			}

		iLen = strchar(pNames,'|');
		strlcpy(pData->aScala[iNum].cVarName,pNames,sizeof(pData->aScala[iNum].cVarName),iLen);
		pNames+=iLen;
		if(*pNames)pNames++;
		pData->aScala[iNum].iNameSize+=iLen+((iLen && *pNames!='|' && *pNames!=0)? 1:0);

		iLen = strchar(pNames,'|');
		strlcpy(pData->aScala[iNum].cUnitName,pNames,sizeof(pData->aScala[iNum].cUnitName),iLen);
		pNames+=iLen;
		if(*pNames)pNames++;
		pData->aScala[iNum].iNameSize+=iLen+(iLen)? 2:0;
		}


	if(iCount>=2)											// Sind alle Y-Skalen gleich		
		{
		pData->uCompine = 1;

		for(iPos=1,iIdx=0;iPos<4;iPos++)				
			{
			if(pData->aScala[iPos].iMode<1)
				{
				continue;
				}

			if(!iIdx)										// Erste Y-Skala
				{
				iIdx = iPos;
				continue;
				}

			pData->aScala[iPos].iMode = iPos+1;				// Ist eine rechte Skala

			if(pData->aScala[iPos].iIsLog!=pData->aScala[iIdx].iIsLog ||
			   pData->aScala[iPos].dMin  !=pData->aScala[iIdx].dMin   ||
			   pData->aScala[iPos].dMax  !=pData->aScala[iIdx].dMax)
				{
				pData->uCompine = 0;
				}
			}

		if(pData->uCompine)									// Alle Y-Skalen sind gleich
			{
			pData->uCompine				= 1;
			pData->aScala[iFirst].iMode = 2;

			for(iPos=3;iPos>=2;iPos--)
				{
				if(pData->aScala[iPos].iMode<0)continue;
				pData->aScala[iPos].iMode=-iPos;			// Skala wegblenden
				iCount--;
				if(iCount<=1)break;
				}
			}
		}


	pImage->SetFont((iMul==2)? aFont2:aFont1); 

		
	for(iNum=0;iNum<4;iNum++)								// Nherungsweise Berechnung der Texte
		{
		if(iNum)											// Y Schritte abschtzen
			{
			iSteps	= iSizeY;
			iSteps -= (FRAME_Y*2+STEP_OFFSET_FY*2)*iMul;
			iSteps -= (uOptions&CDR_DOUBLE)? FONT_HEIGHT_2:FONT_HEIGHT_1;
			iSteps -= (uOptions&CDR_DOUBLE)? FONT_OFFSET_2:FONT_OFFSET_1; 
			
			if(uOptions&CDR_SCALA_X)
				{
				iSteps  -= STEP_SIZE_NULL*iMul;
				if(pData->aScala[0].dMin!=0.0)iSteps-=STEP_OFFSET_Y*iMul;
				}

			   iSteps /= STEP_SIZE_Y*iMul;
			if(iSteps<1)iSteps=1;
			}
		else{												// X Schritte abschtzen
			iSteps	= iSizeX;
			iSteps -= (FRAME_X*2+STEP_OFFSET_FX)*iMul;
			iSteps -= ((uOptions&CDR_DOUBLE)? FONT_WIDTH_2:FONT_WIDTH_1)*6;


			if(iCount>0)									// Gibt es die linke Y1-Skala	
				{
				iSteps  -= STEP_SIZE_NULL*iMul;
				if(pData->aScala[0].dMin!=0.0)
					{
					iSteps -= STEP_OFFSET_X*iMul;			// Abstand Y1 zu Kurven vorhanden
					}
				}
			else{
				iSteps  -= STEP_OFFSET_FX*iMul;
				}

			for(iPos=1,iIdx=0;iPos<4;iPos++)				// Y-Skalen abziehen
				{	
				if(pData->aScala[iPos].iMode<1)
					{
					continue;								
					}
														
				iSteps -= STEP_OFFSET_FX*iMul;
				iSteps -= ((uOptions&CDR_DOUBLE)? FONT_WIDTH_2:FONT_WIDTH_1)*6;

				if(pData->aScala[iPos].iMode>2)				// Rechte Skala
					{
					iSteps -= STEP_SIZE_NULL*iMul;
					
					if(iIdx || pData->aScala[0].dMax!=0.0)	
						{
						iSteps -= STEP_OFFSET_X*iMul;		// Abstand links neben Skala
						iIdx	= iPos;
						}
					}										// Linke Skala
				else{										
					iSteps  -= STEP_OFFSET_FX*iMul;			// Abstand rechts neben Skala
					}
				}
			
			   iSteps /= STEP_SIZE_X*iMul;
			if(iSteps<1)iSteps=1;
			}


		iAdd = -3;
				
		if(uOptions&(CDR_IS_LOG_X<<iNum))					// Logarithmisch berechnen	
			{
			iAdd = CurveCalcLog(pData->aScala+iNum,iSteps);
			}

		if(iAdd<=-3)										// Liear berechnen
			{
			iAdd = CurveCalcLinear(pData->aScala+iNum,iSteps);
			}

		
		if(iAdd>=0)											// Lineare Anzeige	
			{
			dSteps	= pData->aScala[iNum].aSteps[0];
			dValue	= fabs(pMinMax[iNum*2+0]);
			dComma	= fabs(pMinMax[iNum*2+1]);
			if(dComma>dValue)dValue=dComma;


			iVal	= (int)floor(log10(dValue));			// Stellen vom maximal Wert			
			iSub	= (int)floor(log10(dSteps))-iAdd;		// Stellen von der Schrittbreite
			iBase	= 8;

			while(iVal<-2 && iSub<-3 && iBase>0)			// Nachkommstellen reduzieren
				{
				iVal  += 3;
				iSub  += 3;
				iBase -= 1;
				}

			while(iVal>2 && iSub>3 && iBase<15)				// Vorkommstellen reduzieren
				{
				iVal  -= 3;
				iSub  -= 3;
				iBase += 1;
				}

			if(iVal<0)										// Wert hat nur Nachkommastellen
				{
				if(iSub>0)iSub=0;
				iVal  = 1-iSub;
				}
			else if(iSub<0)									// Wert hat Vor- und Nachkommastellen
				{	
				iVal -= iSub-1;
				}
			else{
				iVal += 1;
				}

			if(iBase!=8)iVal++;
			if(iSub<0  )iVal++;
			else		iSub=0;


			if(pMinMax[iNum*2+0]<0.0)						// Gibt es ein Minuszeichen
				{
				iVal++;
				}

			pData->aScala[iNum].iPlaces = iVal;
			pData->aScala[iNum].iComma	=-iSub;
			pData->aScala[iNum].iBase	= iBase;
			}
		else{												// Logarithmische Anzeige
			pData->aScala[iNum].iPlaces = (iAdd==-2)? 6:5;
			pData->aScala[iNum].iComma	= (iAdd==-2)? 1:0;
			pData->aScala[iNum].iBase	= -1;
			}
		}

	
	pData->uOptions	= uOptions;



//******************** Y-Berechnungen *****************************************


	iPosY	= iSizeY-FRAME_Y*iMul;
	if(uOptions&CDR_DOUBLE)iPosY&=~1;

	iPosY  -= (uOptions&CDR_DOUBLE)? FONT_HEIGHT_2  :FONT_HEIGHT_1  ; 
	iPosY  -= (uOptions&CDR_DOUBLE)? FONT_OFFSET_2/2:FONT_OFFSET_1/2; 
	

	pData->aScala[0].iPosF	= iPosY;

	iPosY  -= STEP_OFFSET_FY*iMul;

	if(uOptions&CDR_SCALA_X)								// Gibt es eine X-Skala
		{
		iPosY -= STEP_SIZE_NULL*iMul;
		pData->aScala[0].iPosY = iPosY;
		if(pData->aScala[1].dMin!=0.0)iPosY-=STEP_OFFSET_Y*iMul;
		}

	pData->aScala[1].iPosY	= iPosY;
	pData->aScala[2].iPosY	= iPosY;
	pData->aScala[3].iPosY	= iPosY;
	pData->iPosY			= iPosY;

	iPosY		   -= (STEP_OFFSET_FY+FRAME_X)*iMul;
	iPosY		   -= (uOptions&CDR_DOUBLE)? FONT_HEIGHT_2  :FONT_HEIGHT_1; 	
	iPosY		   -= (uOptions&CDR_DOUBLE)? FONT_OFFSET_2/2:FONT_OFFSET_1/2; 
	
	if(iPosY<(STEP_SIZE_Y+1)*iMul)
		{
		iPosY = (STEP_SIZE_Y+1)*iMul;
		}
	
	if(uOptions&CDR_DOUBLE)iPosY&=~1;
						  
	pData->aScala[1].iSize	= iPosY;
	pData->aScala[2].iSize	= iPosY;
	pData->aScala[3].iSize	= iPosY;
	pData->iSizeY			= iPosY;


//******************** X-Berechnungen *****************************************	

	
	iChar	= (uOptions&CDR_DOUBLE)? FONT_WIDTH_2:FONT_WIDTH_1; 	
	iLen	= pData->aScala[1].iPlaces*iChar;

	iNum	= strlcpy(cText,pData->aScala[iFirst].cVarName,sizeof(cText));
	if(pData->aScala[iFirst].cUnitName[0])sprintf(cText+iNum," [%s]",pData->aScala[iFirst].cUnitName);
	iNum	= pImage->Print(0,BI_TETXSIZE,cText);			

	if(iLen<iNum)iLen=iNum;

	iNum	= strlcpy(cText," 000",sizeof(cText));
	iNum   += strlcpy(cText+iNum,pData->aScala[0].cVarName,sizeof(cText)-iNum)+2;
	if(pData->aScala[0].cUnitName[0])sprintf(cText+iNum," [%s]",pData->aScala[0].cUnitName);
	iNum	= pImage->Print(0,BI_TETXSIZE,cText)			;

	if(iLen<iNum)iLen=iNum;

	iPosX	= FRAME_X*iMul;
	iPosX  += iLen; 	

	if(!iFirst)iFirst=1;
		{
		pData->aScala[iFirst].iPosF	= iPosX;
		}

	iPosF	= iPosX+iChar*2;
	iPosX  += STEP_OFFSET_FX*iMul;

	if(iCount)												// Gibt es die linke Y1-Skala
		{
		iPosX += STEP_SIZE_NULL*iMul;
		pData->aScala[iFirst].iPosX = iPosX;
		
		if(pData->aScala[0].dMin!=0.0)
			{
			iPosX += STEP_OFFSET_X*iMul;					// Abstand rechts zwischen Skala und Kurven
			}
		}
	else{
		iPosX += STEP_OFFSET_FX*iMul;
		}

	pData->aScala[0].iPosX	= iPosX;
	pData->iPosX			= iPosX;

	iSteps					= iSizeX-iPosX;
	iOff					= (FRAME_X)*iMul;

	for(iNum=2,iIdx=0;iNum<4;iNum++)						// Rechte Y-Skalen
		{
		if(pData->aScala[iNum].iMode<3)
			{
			if(pData->aScala[iNum].iMode>-2)
				{
				continue;
				}
															// Textposition von kombinierten Y-Skalen
			iAdd	= strlcpy(cText,pData->aScala[iNum].cVarName,sizeof(cText));
			if(pData->aScala[iNum].cUnitName[0])sprintf(cText+iAdd," [%s]",pData->aScala[iNum].cUnitName);
			iAdd	= pImage->Print(0,BI_TETXSIZE,cText);			

			pData->aScala[iNum].iPosF  = iPosF;
			iPosF  += iAdd+iChar+STEP_OFFSET_X;
			continue;
			}

		   iLen=pData->aScala[iNum].iPlaces;
		if(iLen<pData->aScala[iNum].iNameSize)
			{
			iLen = pData->aScala[iNum].iNameSize;
			}

		iOff += ((uOptions&CDR_DOUBLE)? FONT_WIDTH_2:FONT_WIDTH_1)*iLen; 
		pData->aScala[iNum].iPosF	= -iOff;
		iOff += STEP_OFFSET_FX*iMul;

		iOff += STEP_SIZE_NULL*iMul;
		pData->aScala[iNum].iPosX	= -iOff;
			
		if(iIdx || pData->aScala[0].dMax!=0.0)				// Abstand links von der Skala
			{
			iOff += STEP_OFFSET_X*iMul;
			iIdx  = iNum;
			}
		}

	iSteps	= iSizeX-iPosX-iOff;
	if(uOptions&CDR_DOUBLE)iSteps&=~1;
	if(iSteps<(STEP_SIZE_X+1)*iMul)iSteps=(STEP_SIZE_X+1)*iMul;
	

	for(iNum=2;iNum<4;iNum++)								// Rechte Y-Skalen richtig schieben
		{
		if(pData->aScala[iNum].iMode<3)
			{
			continue;
			}

		pData->aScala[iNum].iPosF += iPosX+iSteps+iOff;
		pData->aScala[iNum].iPosX += iPosX+iSteps+iOff;
		}


	pData->aScala[0].iSize	= iSteps;
	pData->iSizeX			= iSteps;

	
	

	for(iNum=0;iNum<4;iNum++)								// Berechnung der Skalenunterteilung
		{
		iAdd	= -3;
		iSteps	= (iNum)? STEP_SIZE_Y*iMul:STEP_SIZE_X*iMul;				
		
		pData->aScala[iNum].iPosNull	= -1;
		pData->aScala[iNum].iPosGrid	=  0;
		pData->aScala[iNum].iHasGrid    =  0;
		pData->aScala[iNum].iDoubleMark	=  0;

		if(pData->aScala[iNum].iIsLog)						// Logarithmisch berechnen	
			{
			iAdd = CurveCalcLog(pData->aScala+iNum,(pData->aScala[iNum].iSize+1)/iSteps);
			pData->aScala[iNum].iComma  = (iAdd==-2)? 1:0;
			}

		if(iAdd<=-3)										// Linear berechnen
			{
			iAdd = CurveCalcLinear(pData->aScala+iNum,(pData->aScala[iNum].iSize+1)/iSteps);
			}
		
		if(pData->aScala[iNum].dMin==0.0)
			{
			pData->aScala[iNum].iDoubleMark = 1;
			}
		}


//******************** Diverses ***********************************************


	pData->iOffX	= 0;									// X-Offset fr die Linien
	pData->iOffY	= 0;									// Y-Offset fr die Linien
	pData->uDrawSub	= 0;
	pData->iEndX	= pData->iPosX+pData->iSizeX-1;			// X-Koordinate der Kurven (Ende)
	pData->iEndY	= pData->iPosY-pData->iSizeY+1;			// Y-Koordinate der Kurven (Ende)


	for(iNum=1;iNum<4;iNum++)								// Berechnung der Nulllinie fr Fllung
		{
		if(!(uOptions&(CDR_FILL_1<<(iNum-1))))				// Keine Fllung
			{
			pData->iFill[iNum-1] = -1;
			continue;
			}
																
		if(pData->aScala[iNum].dMin<=0.0 && pData->aScala[iNum].dMax>=0.0 && !pData->aScala[iNum].iIsLog)
			{		
			pData->iFill[iNum-1] = pData->iPosY+(int)(pData->aScala[iNum].dMin*pData->aScala[iNum].dFact);
			}
		else if(pData->aScala[3].dMin<0.0)
			{
			pData->iFill[iNum-1] = pData->iEndY;	
			}
		else{
			pData->iFill[iNum-1] = pData->iPosY;	
			}

		pData->iFillMax[iNum-1]	= pData->iPosY-pData->iFill[iNum-1];	
		pData->iFillMin[iNum-1]	= pData->iEndY-pData->iFill[iNum-1];
		}


															// Berechnung der Null-Striche fr X
	if(pData->aScala[iFirst].dMin<=0.0 && pData->aScala[iFirst].dMax>=0.0 && !pData->aScala[iFirst].iIsLog)
		{		
		pData->aScala[0].iPosNull = (int)((-pData->aScala[iFirst].dMin+pData->aScala[iFirst].dScale*0.25)*pData->aScala[iFirst].dFact);
		if(pData->uOptions&CDR_DOUBLE)pData->aScala[0].iPosNull&=~1;
		pData->aScala[0].iPosNull = pData->iPosY-pData->aScala[0].iPosNull;
		}

	
															// Berechnung der Null-Striche
	if(pData->aScala[0].dMin<=0.0 && pData->aScala[0].dMax>=0.0 && !pData->aScala[0].iIsLog)
		{		
		pData->aScala[iFirst].iPosNull = (int)((pData->aScala[0].dScale*0.25-pData->aScala[0].dMin)*pData->aScala[0].dFact);
		if(pData->uOptions&CDR_DOUBLE)pData->aScala[iFirst].iPosNull&=~1;
		pData->aScala[iFirst].iPosNull = pData->iPosX+pData->aScala[iFirst].iPosNull;
		}

	if(pData->uOptions&CDR_GRID_X)							// Gitterlinien X
		{
		pData->aScala[iFirst].iPosGrid = pData->iPosX;
		pData->aScala[iFirst].iHasGrid = pData->iSizeX;
		}
	else{
		pData->aScala[iFirst].iHasGrid = 0;
		}

	if(pData->uOptions&CDR_GRID_Y)							// Gitterlinien Y
		{
		pData->aScala[0].iPosGrid = pData->iPosY;
		pData->aScala[0].iHasGrid = pData->iSizeY;
		}
	else{
		pData->aScala[0].iHasGrid = 0;
		}


	
return pData->iSizeX/iMul;
}		


//*****************************************************************************
//*
//*		CurveDraw
//*
//*****************************************************************************
//	Zeichnet eine zweidimensionale Kurve
//	pData		: Zeiger auf die Kurvendaten
//	pImage		: Zeiger auf das Image in dem gezeichnet wird
//	pValues1	: Werte fr die Y1-Kurfe
//	pValues2	: Werte fr die Y2-Kurfe
//	pValues3	: Werte fr die Y2-Kurfe
//	iMarkerCnt	: Anzahl der Marke
//	pMarkerVal	: Werte fr Marke  
//	pMarkerMode	: Markermodi (0=X 1=Y1 2=Y2 3=Y3)
void CurveDraw(CurveData *pData,BitmapImage *pImage,double *pValues1,double *pValues2,double *pValues3,int iMarkerCnt,double *pMarkerVal,char *pMarkerMode)
{
unsigned char  *pRow0;	
unsigned char  *pRow1;
PixelFunction   pDraw;
BitmapImage		sTemp;
BitmapImage    *pMain;
unsigned		uCol;
double			dAdd;
double			dLog;
double			dVal;
double			dBit;
double			dMin;
double			dOff;
int				iNum;
int				iMul;
int				iPos;
int				iOld;
int				iNow;
int				iIdx;
int				iDif;
int				iAdd;
int				iErr;



	if(pData->uOptions&CDR_DOUBLE)							// Doppelt so gro zeichnen
		{
		pMain  =  pImage;
		pImage = &sTemp;
		sTemp.SetSize(32,pData->iBoxX,pData->iBoxY,COLOR_BACKPLANE);
		}
	else{
		pImage->Box(0,0,pImage->Width(),pImage->Height(),COLOR_BACKPLANE);
		}

	pData->uDrawSub = pData->iEndY;
	iMul			= pData->aScala[0].iMul;
	iAdd			= (pData->uOptions&CDR_DOUBLE)? 1:0;

	pImage->SetFont((iMul==2)? aFont2:aFont1); 
	pImage->SetUserPtr(pData);
	pImage->Box(pData->iPosX,pData->iPosY+iAdd,pData->iSizeX,-pData->iSizeY,COLOR_CURVEPLANE);

	if(pData->uOptions&(CDR_GRID_X|CDR_GRID_Y))
		{
		CurveDrawGrid(pData,pImage);
		}

	
	for(iNum=0;iNum<4;iNum++)								// Alle Skallen zeichnen
		{
		CurveDrawScala(pData->aScala+iNum,pImage);
		}



//******************** Kurvenschatten *****************************************


	if(pData->uOptions&CDR_SHADOW)
		{
		pData->iOffX	= 0;
		pData->iOffY	= 0;

		if(pValues1)										// Schatten zeichnen
			{
			iPos  = pData->iSizeX-iMul;
			pDraw = (iMul==2)? SetShadowLarge:SetShadowSmall;	
			
			if(pData->aScala[1].iIsLog)						// Logarithmische Teilung
				{
				iIdx	= iPos/iMul;
				dLog	= pData->iSizeY-iMul;
				dLog   /= pData->aScala[1].dFact;
				dMin	= pData->aScala[1].dMin*0.99999;
				iErr	= 1;
				iOld	= 0;				
			
				for(;iIdx>=0;iIdx--,iPos-=iMul)				// Linienschleife
					{
					if(IS_NAN(pValues1[iIdx]))
						{
						iErr = 1;
						iOld = 0;
						continue;
						}

					   dOff = log(pValues1[iIdx]/dMin)*dLog;
					if(dOff>10000.0)
						{
						iNow = 10000;
						}
					else if(dOff<-10000.0)
						{
						iNow = -10000;
						}
					else{
						iNow = (int)dOff;
						}

					if(iErr==0)
						{
						pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
									 pData->iPosX+iPos     ,pData->iPosY-iNow,
									 COLOR_CURVE_SHADOW,pDraw,!iPos);
						}

					iOld  = iNow;
					iErr  = 0;	
					}
				}
			else{											// Lineare Teilung
				iIdx	= iPos/iMul;
				dAdd	= pData->aScala[1].dFact;
				dVal	= pData->aScala[1].dMax;
				dBit	= pData->aScala[1].dScale*0.25;
				dMin	= pData->aScala[1].dMin-dBit;
				iErr	= 1;
				iOld	= 0;				

				for(;iIdx>=0;iIdx--,iPos-=iMul)				// Linienschleife	
					{
					if(IS_NAN(pValues1[iIdx]))
						{
						iErr = 1;
						iOld = 0;
						continue;
						}

					   dOff = (pValues1[iIdx]-dMin)*dAdd;
					if(dOff>10000.0)
						{
						iNow = 10000;
						}
					else if(dOff<-10000.0)
						{
						iNow = -10000;
						}
					else{
						iNow = (int)dOff;
						}

					if(iErr==0)
						{
						pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
									 pData->iPosX+iPos     ,pData->iPosY-iNow,
									 COLOR_CURVE_SHADOW,pDraw,!iPos);
						}

					iOld  = iNow;
					iErr  = 0;	
					}
				}
			}
			
		if(pValues2 && (pData->uOptions&CDR_HAS_Y2))		// Schatten zeichnen
			{
			iPos  = pData->iSizeX-iMul;
			pDraw = (iMul==2)? SetShadowLarge:SetShadowSmall;	
			
			if(pData->aScala[2].iIsLog)						// Logarithmische Teilung
				{
				iIdx	= iPos/iMul;
				dLog	= pData->iSizeX-iMul;
				dLog   /= pData->aScala[2].dFact;
				dMin	= pData->aScala[2].dMin*0.99999;
				iErr	= 1;
				iOld	= 0;				
			
				for(;iIdx>=0;iIdx--,iPos-=iMul)				// Linienschleife
					{
					if(IS_NAN(pValues2[iIdx]))
						{
						iErr = 1;
						iOld = 0;
						continue;
						}

					   dOff = log(pValues2[iIdx]/dMin)*dLog;
					if(dOff>10000.0)
						{
						iNow = 10000;
						}
					else if(dOff<-10000.0)
						{
						iNow = -10000;
						}
					else{
						iNow = (int)dOff;
						}

					if(iErr==0)
						{
						pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
									 pData->iPosX+iPos     ,pData->iPosY-iNow,
									 COLOR_CURVE_SHADOW,pDraw,!iPos);
						}

					iOld  = iNow;
					iErr  = 0;	
					}
				}
			else{											// Lineare Teilung
				iIdx	= iPos/iMul;
				dAdd	= pData->aScala[2].dFact;
				dVal	= pData->aScala[2].dMax;
				dBit	= pData->aScala[2].dScale*0.25;
				dMin	= pData->aScala[2].dMin-dBit;
				iErr	= 1;
				iOld	= 0;				
			
				for(;iIdx>=0;iIdx--,iPos-=iMul)				// Linienschleife	
					{
					if(IS_NAN(pValues2[iIdx]))
						{
						iErr = 1;
						iOld = 0;
						continue;
						}

					   dOff = (pValues2[iIdx]-dMin)*dAdd;
					if(dOff>10000.0)
						{
						iNow = 10000;
						}
					else if(dOff<-10000.0)
						{
						iNow = -10000;
						}
					else{
						iNow = (int)dOff;
						}

					if(iErr==0)
						{
						pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
									 pData->iPosX+iPos     ,pData->iPosY-iNow,
									 COLOR_CURVE_SHADOW,pDraw,!iPos);
						}

					iOld  = iNow;
					iErr  = 0;	
					}
				}
			}

		if(pValues3 && (pData->uOptions&CDR_HAS_Y2))		// Schatten zeichnen
			{
			iPos  = pData->iSizeX-iMul;
			pDraw = (iMul==2)? SetShadowLarge:SetShadowSmall;	
			
			if(pData->aScala[3].iIsLog)						// Logarithmische Teilung
				{
				iIdx	= iPos/iMul;
				dLog	= pData->iSizeX-iMul;
				dLog   /= pData->aScala[3].dFact;
				dMin	= pData->aScala[3].dMin*0.99999;
				iErr	= 1;
				iOld	= 0;				
			
				for(;iIdx>=0;iIdx--,iPos-=iMul)				// Linienschleife
					{
					if(IS_NAN(pValues3[iIdx]))
						{
						iErr = 1;
						iOld = 0;
						continue;
						}

					   dOff = log(pValues3[iIdx]/dMin)*dLog;
					if(dOff>10000.0)
						{
						iNow = 10000;
						}
					else if(dOff<-10000.0)
						{
						iNow = -10000;
						}
					else{
						iNow = (int)dOff;
						}

					if(iErr==0)
						{
						pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
									 pData->iPosX+iPos     ,pData->iPosY-iNow,
									 COLOR_CURVE_SHADOW,pDraw,!iPos);
						}

					iOld  = iNow;
					iErr  = 0;	
					}
				}
			else{											// Lineare Teilung
				iIdx	= iPos/iMul;
				dAdd	= pData->aScala[3].dFact;
				dVal	= pData->aScala[3].dMax;
				dBit	= pData->aScala[3].dScale*0.25;
				dMin	= pData->aScala[3].dMin-dBit;
				iErr	= 1;
				iOld	= 0;				
			
				for(;iIdx>=0;iIdx--,iPos-=iMul)				// Linienschleife	
					{
					if(IS_NAN(pValues3[iIdx]))
						{
						iErr = 1;
						iOld = 0;
						continue;
						}

					   dOff = (pValues3[iIdx]-dMin)*dAdd;
					if(dOff>10000.0)
						{
						iNow = 10000;
						}
					else if(dOff<-10000.0)
						{
						iNow = -10000;
						}
					else{
						iNow = (int)dOff;
						}

					if(iErr==0)
						{
						pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
									 pData->iPosX+iPos     ,pData->iPosY-iNow,
									 COLOR_CURVE_SHADOW,pDraw,!iPos);
						}

					iOld  = iNow;
					iErr  = 0;	
					}
				}
			}


		for(iPos=0;iPos<iMarkerCnt;iPos++)					// Marker Zeichen
			{
			pDraw = (iMul==2)? SetShadowLarge:SetShadowSmall;	


			if(!pMarkerMode || !pMarkerMode[iPos])
				{
				if(pData->aScala[0].iIsLog)					// Logarithmische Teilung
					{
					dLog	= pData->iSizeY-iMul;
					dLog   /= pData->aScala[0].dFact;
					dMin	= pData->aScala[0].dMin*0.99999;
					iErr	= 1;
					iOld	= 0;				

					if(IS_NAN(pMarkerVal[iPos]))
						{
						continue;
						}

					   dOff = log(pMarkerVal[iPos]/dMin)*dLog;
					if(dOff>=(double)pData->iSizeX)
						{
						continue;
						}
					else if(dOff<0.0)
						{
						continue;
						}
					else{
						iNow = (int)dOff;
						}
					}
				else{
					dAdd	= pData->aScala[0].dFact;
					dVal	= pData->aScala[0].dMax;
					dBit	= pData->aScala[0].dScale*0.25;
					dMin	= pData->aScala[0].dMin-dBit;
					iErr	= 1;
					iOld	= 0;				

					if(IS_NAN(pMarkerVal[iPos]))
						{
						continue;
						}

					   dOff = (pMarkerVal[iPos]-dMin)*dAdd;
					if(dOff>=(double)pData->iSizeX)
						{
						continue;
						}
					else if(dOff<0.0)
						{
						continue;
						}
					else{
						iNow = (int)dOff;
						}
					}

				pImage->Line(pData->iPosX+iNow,pData->iPosY,
							 pData->iPosX+iNow,pData->iEndY,
							 COLOR_CURVE_SHADOW,pDraw);
				}
			else{
				iIdx = pMarkerMode[iPos];

				if(pData->aScala[iIdx].iIsLog)				// Logarithmische Teilung
					{
					dLog	= pData->iSizeX-iMul;
					dLog   /= pData->aScala[iIdx].dFact;
					dMin	= pData->aScala[iIdx].dMin*0.99999;
					iErr	= 1;
					iOld	= 0;				

					if(IS_NAN(pMarkerVal[iPos]))
						{
						continue;
						}

					   dOff = log(pMarkerVal[iPos]/dMin)*dLog;
					if(dOff>=(double)pData->iSizeY)
						{
						continue;
						}
					else if(dOff<0.0)
						{
						continue;
						}
					else{
						iNow = (int)dOff;
						}
					}
				else{
					dAdd	= pData->aScala[iIdx].dFact;
					dVal	= pData->aScala[iIdx].dMax;
					dBit	= pData->aScala[iIdx].dScale*0.25;
					dMin	= pData->aScala[iIdx].dMin-dBit;
					iErr	= 1;
					iOld	= 0;				

					if(IS_NAN(pMarkerVal[iPos]))
						{
						continue;
						}

					   dOff = (pMarkerVal[iPos]-dMin)*dAdd;
					if(dOff>=(double)pData->iSizeY)
						{
						continue;
						}
					else if(dOff<0.0)
						{
						continue;
						}
					else{
						iNow = (int)dOff;
						}

					}
				
				pImage->Line(pData->iPosX,pData->iPosY-iNow,
							 pData->iEndX,pData->iPosY-iNow,
							 COLOR_CURVE_SHADOW,pDraw);
				}
			}
		}


//******************** Marker *************************************************


	for(iPos=0;iPos<iMarkerCnt;iPos++)						// Marker Zeichen
		{
		pDraw = (iMul==2)? SetMarkerLarge:SetMarkerSmall;		
		uDrawTurn = 0;

		if(!pMarkerMode || !pMarkerMode[iPos])
			{
			if(pData->aScala[0].iIsLog)						// Logarithmische Teilung
				{
				dLog	= pData->iSizeY-iMul;
				dLog   /= pData->aScala[0].dFact;
				dMin	= pData->aScala[0].dMin*0.99999;
				iErr	= 1;
				iOld	= 0;				

				if(IS_NAN(pMarkerVal[iPos]))
					{
					continue;
					}

				   dOff = log(pMarkerVal[iPos]/dMin)*dLog;
				if(dOff>pData->aScala[0].dMax)
					{
					continue;
					}
				else if(dOff<0.0)
					{
					continue;
					}
				else{
					iNow = (int)dOff;
					}
				}
			else{
				dAdd	= pData->aScala[0].dFact;
				dVal	= pData->aScala[0].dMax;
				dBit	= pData->aScala[0].dScale*0.25;
				dMin	= pData->aScala[0].dMin-dBit;
				iErr	= 1;
				iOld	= 0;				

				if(IS_NAN(pMarkerVal[iPos]))
					{
					continue;
					}

				   dOff = (pMarkerVal[iPos]-dMin)*dAdd;
				if(dOff>=(double)pData->iSizeX)
					{
					continue;
					}
				else if(dOff<0.0)
					{
					continue;
					}
				else{
					iNow = (int)dOff;
					}
				}

			pImage->Line(pData->iPosX+iNow,pData->iPosY,
						 pData->iPosX+iNow,pData->iEndY,
						 aMarkerColors[iPos&3],pDraw);
			}
		else{
			iIdx = pMarkerMode[iPos];

			if(pData->aScala[iIdx].iIsLog)					// Logarithmische Teilung
				{
				dLog	= pData->iSizeX-iMul;
				dLog   /= pData->aScala[iIdx].dFact;
				dMin	= pData->aScala[iIdx].dMin*0.99999;
				iErr	= 1;
				iOld	= 0;				

				if(IS_NAN(pMarkerVal[iPos]))
					{
					continue;
					}

				   dOff = log(pMarkerVal[iPos]/dMin)*dLog;
				if(dOff>pData->aScala[iIdx].dMax)
					{
					continue;
					}
				else if(dOff<0.0)
					{
					continue;
					}
				else{
					iNow = (int)dOff;
					}
				}
			else{
				dAdd	= pData->aScala[iIdx].dFact;
				dVal	= pData->aScala[iIdx].dMax;
				dBit	= pData->aScala[iIdx].dScale*0.25;
				dMin	= pData->aScala[iIdx].dMin-dBit;
				iErr	= 1;
				iOld	= 0;				

				if(IS_NAN(pMarkerVal[iPos]))
					{
					continue;
					}

				   dOff = (pMarkerVal[iPos]-dMin)*dAdd;
				if(dOff>=(double)pData->iSizeY)
					{
					continue;
					}
				else if(dOff<0.0)
					{
					continue;
					}
				else{
					iNow = (int)dOff;
					}

				}
			
			pImage->Line(pData->iPosX,pData->iPosY-iNow,
						 pData->iEndX,pData->iPosY-iNow,
						 aMarkerColors[iPos&3],pDraw);
			}
		}


//******************** Kurve1 *************************************************
	

	if(pValues1)											// Kurve Zeichen
		{
		iPos  = pData->iSizeX-iMul;
		pDraw = (iMul==2)? SetPixelLarge:SetPixelSmall;		
		
		if(pData->aScala[1].iIsLog)							// Logarithmische Teilung
			{
			iIdx	= iPos/iMul;
			dLog	= pData->aScala[2].iSize-iMul;
			dLog   /= pData->aScala[1].dFact;
			dMin	= pData->aScala[1].dMin*0.99999;
			dOff	= log(pValues1[iIdx]/dMin)*dLog;		// Startposition
			iErr	= 1;
			iOld	= 0;				

			for(;iIdx>=0;iIdx--,iPos-=iMul)					// Linienschleife
				{
				if(IS_NAN(pValues1[iIdx]))
					{
					iErr = 1;
					iOld = 0;
					continue;
					}

				   dOff = log(pValues1[iIdx]/dMin)*dLog;
				if(dOff>10000.0)
					{
					iNow = 10000;
					}
				else if(dOff<-10000.0)
					{
					iNow = -10000;
					}
				else{
					iNow = (int)dOff;
					}

				if(pData->iFill[0]>=0)						// Kurve gefllt Zeichen
					{
					   iDif = pData->iPosY-iNow-pData->iFill[0];
					if(iDif<pData->iFillMin[0])iDif=pData->iFillMin[0];
					if(iDif>pData->iFillMax[0])iDif=pData->iFillMax[0];

					pImage->Box(pData->iPosX+iPos,pData->iFill[0],iMul,iDif,COLOR_CURVE_Y1,SHADOW_ALPHA);
					}

				if(iErr==0)
					{
					pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
								 pData->iPosX+iPos     ,pData->iPosY-iNow,
								 COLOR_CURVE_Y1,pDraw,!iPos);
					}


				iOld  = iNow;
				iErr  = 0;	
				}
			}
		else{												// Lineare Teilung
			iIdx	= iPos/iMul;
			dAdd	= pData->aScala[1].dFact;
			dVal	= pData->aScala[1].dMax;
			dBit	= pData->aScala[1].dScale*0.25;
			dMin	= pData->aScala[1].dMin-dBit;
			dOff	= (pValues1[iIdx]-dMin)*dAdd;			// Startposition
			iErr	= 1;
			iOld	= 0;				

			for(;iIdx>=0;iIdx--,iPos-=iMul)					// Linienschleife
				{
				if(IS_NAN(pValues1[iIdx]))
					{
					iErr = 1;
					iOld = 0;
					continue;
					}

				   dOff = (pValues1[iIdx]-dMin)*dAdd;
				if(dOff>10000.0)
					{
					iNow = 10000;
					}
				else if(dOff<-10000.0)
					{
					iNow = -10000;
					}
				else{
					iNow = (int)dOff;
					}

				if(pData->iFill[0]>=0)						// Kurve gefllt Zeichen
					{
					   iDif = pData->iPosY-iNow-pData->iFill[0];
					if(iDif<pData->iFillMin[0])iDif=pData->iFillMin[0];
					if(iDif>pData->iFillMax[0])iDif=pData->iFillMax[0];

					pImage->Box(pData->iPosX+iPos,pData->iFill[0],iMul,iDif,COLOR_CURVE_Y1,SHADOW_ALPHA);
					}

				if(iErr==0)
					{
					pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
								 pData->iPosX+iPos     ,pData->iPosY-iNow,
								 COLOR_CURVE_Y1,pDraw,!iPos);
					}

				iOld  = iNow;
				iErr  = 0;	
				}
			}
		}	


//******************** Kurve2 *************************************************
	

	if(pValues2 && (pData->uOptions&CDR_HAS_Y2))			// Kurve Zeichen
		{
		iPos  = pData->iSizeX-iMul;
		pDraw = (iMul==2)? SetPixelLarge:SetPixelSmall;		
		
		if(pData->aScala[2].iIsLog)							// Logarithmische Teilung
			{
			iIdx	= iPos/iMul;
			dLog	= pData->aScala[2].iSize-iMul;
			dLog   /= pData->aScala[2].dFact;
			dMin	= pData->aScala[2].dMin*0.99999;
			dOff	= log(pValues2[iIdx]/dMin)*dLog;		// Startposition
			iErr	= 1;
			iOld	= 0;				
			
			for(;iIdx>=0;iIdx--,iPos-=iMul)					// Linienschleife
				{
				if(IS_NAN(pValues2[iIdx]))
					{
					iErr = 1;
					iOld = 0;
					continue;
					}

				   dOff = log(pValues2[iIdx]/dMin)*dLog;
				if(dOff>10000.0)
					{
					iNow = 10000;
					}
				else if(dOff<-10000.0)
					{
					iNow = -10000;
					}
				else{
					iNow = (int)dOff;
					}

				if(pData->iFill[1]>=0)						// Kurve gefllt Zeichen
					{
					   iDif = pData->iPosY-iNow-pData->iFill[1];
					if(iDif<pData->iFillMin[1])iDif=pData->iFillMin[1];
					if(iDif>pData->iFillMax[1])iDif=pData->iFillMax[1];

					pImage->Box(pData->iPosX+iPos,pData->iFill[1],iMul,iDif,COLOR_CURVE_Y2,SHADOW_ALPHA);
					}

				if(iErr==0)
					{
					pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
								 pData->iPosX+iPos     ,pData->iPosY-iNow,
								 COLOR_CURVE_Y2,pDraw,!iPos);
					}

				iOld  = iNow;
				iErr  = 0;	
				}
			}
		else{												// Lineare Teilung
			iIdx	= iPos/iMul;
			dAdd	= pData->aScala[2].dFact;
			dVal	= pData->aScala[2].dMax;
			dBit	= pData->aScala[2].dScale*0.25;
			dMin	= pData->aScala[2].dMin-dBit;
			dOff	= (pValues2[iIdx]-dMin)*dAdd;			// Startposition
			iErr	= 1;
			iOld	= 0;				
			
			for(;iIdx>=0;iIdx--,iPos-=iMul)					// Linienschleife
				{
				if(IS_NAN(pValues2[iIdx]))
					{
					iErr = 1;
					iOld = 0;
					continue;
					}

				   dOff = (pValues2[iIdx]-dMin)*dAdd;
				if(dOff>10000.0)
					{
					iNow = 10000;
					}
				else if(dOff<-10000.0)
					{
					iNow = -10000;
					}
				else{
					iNow = (int)dOff;
					}

				if(pData->iFill[1]>=0)						// Kurve gefllt Zeichen
					{
					   iDif = pData->iPosY-iNow-pData->iFill[1];
					if(iDif<pData->iFillMin[1])iDif=pData->iFillMin[1];
					if(iDif>pData->iFillMax[1])iDif=pData->iFillMax[1];

					pImage->Box(pData->iPosX+iPos,pData->iFill[1],iMul,iDif,COLOR_CURVE_Y2,SHADOW_ALPHA);
					}

				if(iErr==0)
					{
					pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
								 pData->iPosX+iPos     ,pData->iPosY-iNow,
								 COLOR_CURVE_Y2,pDraw,!iPos);
					}

				iOld  = iNow;
				iErr  = 0;	
				}
			}
		}	

//******************** Kurve3 *************************************************
	

	if(pValues3 && (pData->uOptions&CDR_HAS_Y3))			// Kurve Zeichen
		{
		iPos  = pData->iSizeX-iMul;
		pDraw = (iMul==2)? SetPixelLarge:SetPixelSmall;		
		
		if(pData->aScala[3].iIsLog)							// Logarithmische Teilung
			{
			iIdx	= iPos/iMul;
			dLog	= pData->aScala[3].iSize-iMul;
			dLog   /= pData->aScala[3].dFact;
			dMin	= pData->aScala[3].dMin*0.99999;
			dOff	= log(pValues3[iIdx]/dMin)*dLog;		// Startposition
			iErr	= 1;
			iOld	= 0;				
			
			for(;iIdx>=0;iIdx--,iPos-=iMul)					// Linienschleife
				{
				if(IS_NAN(pValues3[iIdx]))
					{
					iErr = 1;
					iOld = 0;
					continue;
					}

				   dOff = log(pValues3[iIdx]/dMin)*dLog;
				if(dOff>10000.0)
					{
					iNow = 10000;
					}
				else if(dOff<-10000.0)
					{
					iNow = -10000;
					}
				else{
					iNow = (int)dOff;
					}

				if(pData->iFill[2]>=0)						// Kurve gefllt Zeichen
					{
					   iDif = pData->iPosY-iNow-pData->iFill[2];
					if(iDif<pData->iFillMin[2])iDif=pData->iFillMin[2];
					if(iDif>pData->iFillMax[2])iDif=pData->iFillMax[2];

					pImage->Box(pData->iPosX+iPos,pData->iFill[2],iMul,iDif,COLOR_CURVE_Y3,SHADOW_ALPHA);
					}

				if(iErr==0)
					{
					pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
								 pData->iPosX+iPos     ,pData->iPosY-iNow,
								 COLOR_CURVE_Y3,pDraw,!iPos);
					}

				iOld  = iNow;
				iErr  = 0;	
				}
			}
		else{												// Lineare Teilung
			iIdx	= iPos/iMul;
			dAdd	= pData->aScala[3].dFact;
			dVal	= pData->aScala[3].dMax;
			dBit	= pData->aScala[3].dScale*0.25;
			dMin	= pData->aScala[3].dMin-dBit;
			dOff	= (pValues3[iIdx]-dMin)*dAdd;			// Startposition
			iErr	= 1;
			iOld	= 0;				
			
			for(;iIdx>=0;iIdx--,iPos-=iMul)					// Linienschleife
				{
				if(IS_NAN(pValues3[iIdx]))
					{
					iErr = 1;
					iOld = 0;
					continue;
					}

				   dOff = (pValues3[iIdx]-dMin)*dAdd;
				if(dOff>10000.0)
					{
					iNow = 10000;
					}
				else if(dOff<-10000.0)
					{
					iNow = -10000;
					}
				else{
					iNow = (int)dOff;
					}

				if(pData->iFill[2]>=0)						// Kurve gefllt Zeichen
					{
					   iDif = pData->iPosY-iNow-pData->iFill[2];
					if(iDif<pData->iFillMin[2])iDif=pData->iFillMin[2];
					if(iDif>pData->iFillMax[2])iDif=pData->iFillMax[2];

					pImage->Box(pData->iPosX+iPos,pData->iFill[2],iMul,iDif,COLOR_CURVE_Y3,SHADOW_ALPHA);
					}

				if(iErr==0)
					{
					pImage->Line(pData->iPosX+iPos+iMul,pData->iPosY-iOld,
								 pData->iPosX+iPos     ,pData->iPosY-iNow,
								 COLOR_CURVE_Y3,pDraw,!iPos);
					}

				iOld  = iNow;
				iErr  = 0;	
				}
			}
		}	


//******************** Antialisas *********************************************


	if(pData->uOptions&CDR_DOUBLE)							// Doppelt so gro zeichnen
		{
		int		iSizeX;
		int		iSizeY;
		int		iPosX;
		int		iPosY;

		iSizeX = sTemp.Width ()/2;
		iSizeY = sTemp.Height()/2;

		for(iPosY=0;iPosY<iSizeY;iPosY++)
			{
			pRow0 = sTemp.GetRow4(iPosY*2+0);
			pRow1 = sTemp.GetRow4(iPosY*2+1);

			for(iPosX=0;iPosX<iSizeX;iPosX++)
				{
				uCol  = ((pRow0[iPosX*8+0]+pRow0[iPosX*8+4]+pRow1[iPosX*8+0]+pRow1[iPosX*8+4])/4);
				uCol |= ((pRow0[iPosX*8+1]+pRow0[iPosX*8+5]+pRow1[iPosX*8+1]+pRow1[iPosX*8+5])/4)<<8;
				uCol |= ((pRow0[iPosX*8+2]+pRow0[iPosX*8+6]+pRow1[iPosX*8+2]+pRow1[iPosX*8+6])/4)<<16;

				pMain->SetPixel(iPosX,iPosY,uCol);
				}
			}
		}



}

//*****************************************************************************
//*
//*		CurveFrame
//*
//*****************************************************************************
//	Holt die Position des Blattes 
void CurveFrame(CurveData *pData,int &iPosX1,int &iPosY1,int &iPosX2,int &iPosY2)
{

	iPosX1	= pData->iPosX;
	iPosX2	= pData->iPosX+pData->iSizeX;
	iPosY2	= pData->iPosY;
	iPosY1	= pData->iPosY-pData->iSizeY;

	iPosX1 /= pData->aScala[0].iMul;
	iPosX2 /= pData->aScala[0].iMul;
	iPosY2 /= pData->aScala[0].iMul;
	iPosY1 /= pData->aScala[0].iMul;
}