//*****************************************************************************
//*
//*
//*		CurveDrawer.h
//*
//*
//*****************************************************************************
#ifndef 	__CURVEDRAWER_H__
#define 	__CURVEDRAWER_H__

#include	"Image/BitmapImage.h"

#define 	CDR_HAS_Y1				0x00000001		// Es gibt eine zweite Kurve
#define 	CDR_HAS_Y2				0x00000002		// Es gibt eine zweite Kurve
#define 	CDR_HAS_Y3				0x00000004		// Es gibt eine dritte Kurve
#define 	CDR_FILL_1				0x00000010		// Kurve 1 bis zur Nulllinie fllen
#define 	CDR_FILL_2				0x00000020		// Kurve 2 bis zur Nulllinie fllen
#define 	CDR_FILL_3				0x00000040		// Kurve 3 bis zur Nulllinie fllen
#define 	CDR_GRID_X				0x00000100		// Gitter am Hindergrund fr X
#define 	CDR_GRID_Y				0x00000200		// Gitter am Hindergrund fr Y
#define 	CDR_SCALA_X				0x00010000		// X-Skala der Kurven  zeichnen
#define 	CDR_SCALA_1				0x00020000		// Y-Skala der Kurve 1 zeichnen
#define 	CDR_SCALA_2				0x00040000		// Y-Skala der Kurve 2 zeichnen
#define 	CDR_SCALA_3				0x00080000		// Y-Skala der Kurve 3 zeichnen
#define 	CDR_IS_LOG_X			0x01000000		// X-Skala der Kurven  ist logarithmisch
#define 	CDR_IS_LOG_1			0x02000000		// Y-Skala der Kurve 1 ist logarithmisch
#define 	CDR_IS_LOG_2			0x04000000		// Y-Skala der Kurve 2 ist logarithmisch
#define 	CDR_IS_LOG_3			0x08000000		// Y-Skala der Kurve 3 ist logarithmisch
#define 	CDR_DOUBLE				0x10000000		// Doppelt so gro Zeichnen fr Glttung
#define 	CDR_3LINES				0x20000000		// Drei Linien anzeigen
#define 	CDR_SHADOW				0x40000000		// Schatten unter den Kurven zeichnen

#define 	CDR_INDEX_X				0				// Index der X-Scala
#define 	CDR_INDEX_Y1			1				// Index der Y1-Scala
#define 	CDR_INDEX_Y2			2				// Index der Y2-Scala
#define 	CDR_INDEX_Y3			3				// Index der Y3-Scala


typedef struct
	{
	char		cVarName[16];						// Variabelname
	char		cUnitName[8];						// Einheitenname
	int			iNameSize;							// Breite des Namens in Zeichen
	int			iMode;								// 0=X 1=Y-links 2=Y-links-Muli 3=Y2-rechts 4=Y3-rechts -1=nicht sichtbar -2=gleich wie
	int			iMul;								// Vervielfacher 
	int			iIsLog;								// Logaritmische Skala
	int			iPosGrid;							// Position fr das Gitter
	int			iHasGrid;							// Linienlnge fr das Gitter (0=keine)
	int			iDoubleMark;						// Zweiseitige Marke
	int			iPlaces;							// Stellen fr Beschriftung
	int			iComma;								// Kommastellen fr Beschriftung
	int			iBase;								// Anzeigeeinheit (8=- 9=k 10=M ... , -1=auto)
	int			iPosX;								// X-Koordinate der Skala (<0=keine)
	int			iPosY;								// Y-Koordinate der Skala (<0=keine)
	int			iPosF;								// X/Y-Koordinate der Schrift (<0=keine)
	int			iPosNull;							// X/Y-Koordinate der NULL-Skala (<0=keine)
	int			iSize;								// Breite/Hhe der Skala
	int			iStepCnt;							// Schritte in aSteps
	int			iStepAt;							// Bei welchem Schritt wird begonnen
	double		aSteps[8];							// Faktoren fr die Schritte	
	double		dStart;								// Wert der ersten unter Skalierung
	double		dScale;								// Skalierung fr die Scala (einzelner Punkt)
	double		dFact;								// Skalierung fr die Scala (einzelner Punkt)
	double		dMin;								// Anfangswert der Scala
	double		dMax;								// Endwert der Scala
	}CurveScala;


typedef struct
	{
	CurveScala	aScala[4];							// Infos ber die Skalen
	unsigned	uOptions;							// Diverse Flags (siehe CDR_???)
	unsigned	uDrawSub;							// Bereichsbegrenzung frs Zeichnen
	unsigned	uCompine;							// Y1 und Y2 Skala sind zusammen
	int			iFillMax[3];						// Y-Max-Wert der Fllung (-1 keine)
	int			iFillMin[3];						// Y-Min-Wert der Fllung (-1 keine)
	int			iFill[3];							// Y-Position der Fllung (-1 keine)
	int			iSizeX;								// X-Breite der Kurven in Pixel
	int			iSizeY;								// Y-Breite der Kurven in Pixel
	int			iPosX;								// X-Koordinate der Kurven
	int			iPosY;								// Y-Koordinate der Kurven
	int			iEndX;								// X-Koordinate der Kurven (Ende)
	int			iEndY;								// Y-Koordinate der Kurven (Ende)
	int			iOffX;								// X-Offset fr die Linien
	int			iOffY;								// Y-Offset fr die Linien
	int			iBoxX;								// X-Breite des Bildes
	int			iBoxY;								// Y-Breite des Bildes
	}CurveData;


int		CurveInit    (CurveData *pData,BitmapImage *pImage,unsigned uOptions,double *pMinMax,const char *pNames=0);
void	CurveDraw    (CurveData *pData,BitmapImage *pImage,double  *pValues1,double *pValues2=0,double *pValues3=0,int iMarkerCnt=0,double *pMarkerVal=0,char *pMarkerMode=0);
double	CurvePosToVal(CurveData *pData,int    iPos,int iNum,int *pOk=0);
double	CurveValToPos(CurveData *pData,double dVal,int iNum,int *pOk=0);
void	CurveFrame	 (CurveData *pData,int &iPosX1,int &iPosY1,int &iPosX2,int &iPosY2);
int 	CurveColor	 (int iMarker);


#endif 	

