//*****************************************************************************
//*
//*
//*		D3Utility.cpp
//*
//*
//*****************************************************************************
#include	<string.h>
#include	<stdio.h>
#include	<math.h>
#include	"D3Utility.h"


//*****************************************************************************
//*
//*		VectorLength
//*
//*****************************************************************************
//	Berechnet die Lnge eines Verktors
double VectorLength(const D3Vector3 *pVector)
{
double		dAdd;


	dAdd	= pVector->sVector.x*pVector->sVector.x;
	dAdd   += pVector->sVector.y*pVector->sVector.y;
	dAdd   += pVector->sVector.z*pVector->sVector.z;


return sqrt(dAdd);
}

//*****************************************************************************
//*
//*		VectorLength
//*
//*****************************************************************************
//	Berechnet die Lnge eines Verktors
double VectorLength(const D3Vector4 *pVector)
{
double		dAdd;


	dAdd	= pVector->sVector.x*pVector->sVector.x;
	dAdd   += pVector->sVector.y*pVector->sVector.y;
	dAdd   += pVector->sVector.z*pVector->sVector.z;
	dAdd   += pVector->sVector.w*pVector->sVector.w;


return sqrt(dAdd);
}

//*****************************************************************************
//*
//*		VectorDot
//*
//*****************************************************************************
//	Berechnet das Scalarprotukt zweier Vektoren
double VectorDot(const D3Vector3 *pVector1,const D3Vector3 *pVector2)
{
double		dAdd;


	dAdd	= pVector1->sVector.x*pVector2->sVector.x;
	dAdd   += pVector1->sVector.y*pVector2->sVector.y;
	dAdd   += pVector1->sVector.z*pVector2->sVector.z;


return dAdd;
}

//*****************************************************************************
//*
//*		VectorDot
//*
//*****************************************************************************
//	Berechnet das Scalarprotukt zweier Vektoren
double VectorDot(const D3Vector4 *pVector1,const D3Vector4 *pVector2)
{
double		dAdd;


	dAdd	= pVector1->sVector.x*pVector2->sVector.x;
	dAdd   += pVector1->sVector.y*pVector2->sVector.y;
	dAdd   += pVector1->sVector.z*pVector2->sVector.z;
	dAdd   += pVector1->sVector.w*pVector2->sVector.w;


return dAdd;
}

//*****************************************************************************
//*
//*		VectorCross
//*
//*****************************************************************************
//	Berechnet das Kreuzprodukt wzeier Vektoren
//	pOut		: Zeiger auf den Vektor in dem das Ergebnis gespeichert wird
//	pVector1	: Zeiger auf ersten Vektor 
//	pVector2	: Zeiger auf zweiten Vektor 
//	Ergibt einen Zeiger auf das Ergebnis.
D3Vector3 *VectorCross(D3Vector3 *pOut,const D3Vector3 *pVector1,const D3Vector3 *pVector2)
{

	pOut->sVector.x	= pVector1->sVector.y*pVector2->sVector.z - pVector1->sVector.z*pVector2->sVector.y;
	pOut->sVector.y	= pVector1->sVector.z*pVector2->sVector.x - pVector1->sVector.x*pVector2->sVector.z;
	pOut->sVector.z	= pVector1->sVector.x*pVector2->sVector.y - pVector1->sVector.y*pVector2->sVector.x;

return pOut;
}

//*****************************************************************************
//*
//*		MatrixInversion
//*
//*****************************************************************************
//	Berechnet die invertierte Matrix einer Matrix
//	pOut		: Zeiger auf die Matrix in der das Ergebnis gespeichert wird
//	pMatrix		: Zeiger auf die zu invertirente Matrix
//	Ergibt 1 wenn die Matrix invertiert werden konnte
int MatrixInversion(D3Matrix4x4 *pOut,const D3Matrix4x4 *pMatrix)
{
const int		iSize  = 4;
const int		iPivot = 1;
const double	dEpsilon = 0.0000000001; 	
double			aTemp[iSize*iSize*2];
double			dMax;
double			dAbs;
double			dVal;
double			dFac;
int				iLine; 				
int				iStep;				
int				i,j;
	
	


	for(i=0;i<iSize;i++)										// Ergnze die Matrix um eine Einheitsmatrix
	for(j=0;j<iSize;j++)
		{
		aTemp[iSize*2*i+      j] = pMatrix->sData[iSize*i+j];
		aTemp[iSize*2*i+iSize+j] = (i==j)? 1.0:0.0;
		}

	
	for(iStep=0;iStep<iSize;iStep++)							// Die einzelnen Eliminationsschritte
		{ 
		dMax = fabs(aTemp[iSize*2*iStep+iStep]);					// Pivotisierung vermeidet unntigen Abbruch bei einer Null in der Diagnonalen 

		if(iPivot)
			{
			iLine = iStep;

			for(i=iStep+1;i<iSize;i++)							// Suche grtes Element	
				{
				   dAbs = fabs(aTemp[iSize*2*i+iStep]);
				if(dAbs<=dMax)continue;

				dMax  = dAbs;
				iLine = i;
				}
			}

		if(dMax<dEpsilon)
			{
			return 0;
			}

		if(iPivot && iLine!=iStep) 								// Zeilen tauschen falls erforderlich 
			{
			for(j=iStep;j<2*iSize;j++) 
				{
				dVal					= aTemp[iSize*2*iStep+j];
				aTemp[iSize*2*iStep+j]	= aTemp[iSize*2*iLine+j];
				aTemp[iSize*2*iLine+j]	= dVal;
				}
			}

		
		dFac = aTemp[iSize*2*iStep+iStep];

		for(j=iStep;j<2*iSize;j++)								// Eliminationszeile durch Pivot-Koeffizienten dividieren
			{
			aTemp[iSize*2*iStep+j] /= dFac;
			}

		for(i=0;i<iSize;i++)									// Elimination 
			{
			if(i==iStep)continue;

			dFac = -aTemp[iSize*2*i+iStep];						// Multiplikationsfaktor
	
			for(j=iStep;j<2*iSize;j++)							// Spalten durchlaufen
				{
				aTemp[iSize*2*i+j] += aTemp[iSize*2*iStep+j]*dFac;// Addition der Zeilen i, iStep
				}

			}
		}

	
	for(i=0;i<iSize;i++)										// Die angehngte Einheitsmatrix Matrix hat sich jetzt in die inverse Matrix umgewandelt	
		{
		for(j=0;j<iSize;j++) 
			{
			pOut->sData[iSize*i+j] = aTemp[iSize*2*i+iSize+j];
			}
		}



return 1;
}

//*****************************************************************************
//*
//*		MatrixPerspectiveFovLH
//*
//*****************************************************************************
//	Berechnet die Projetionsmatrix
//	pOut		: Hier wird die Matrix gespeichert
//	dViewAngel	: Ist die Winkel der Kammera in Radianden
//	dAspect		: Ist das Verhltnis zwischen Schrmbreite und Hhe (Breite/Hhe)
//	dClipNear	: Krzerer Abstand vom vom Auge zur Cliping-Ebene
//	dClipFar	: Lngere  Abstand vom vom Auge zur Cliping-Ebene
//	Ergibt den Zeiger auf die Projetionsmatrix 
D3Matrix4x4 *MatrixPerspectiveFovLH(D3Matrix4x4 *pOut,double dViewAngel,double dAspect,double dClipNear,double dClipFar)
{
	
	pOut->sMatrix.x1 = 1.0 / (dAspect*tan(dViewAngel*0.5));
	pOut->sMatrix.x2 = 0.0;
	pOut->sMatrix.x3 = 0.0;
	pOut->sMatrix.x4 = 0.0;

	pOut->sMatrix.y1 = 0.0;
	pOut->sMatrix.y2 = 1.0 / tan(dViewAngel*0.5);
	pOut->sMatrix.y3 = 0.0;
	pOut->sMatrix.y4 = 0.0;

	pOut->sMatrix.z1 = 0.0;
	pOut->sMatrix.z2 = 0.0;
	pOut->sMatrix.z3 = dClipFar / (dClipFar-dClipNear);
	pOut->sMatrix.z4 = 1.0;

	pOut->sMatrix.w1 = 0.0;
	pOut->sMatrix.w2 = 0.0;
	pOut->sMatrix.w3 = (dClipFar*dClipNear) / (dClipNear-dClipFar);
	pOut->sMatrix.w4 = 0.0;


return pOut;
}

//*****************************************************************************
//*
//*		VectorNormalize
//*
//*****************************************************************************
//	Normalisiert eine Vektor
//	pOut		: Hier wird der neue Vektor gespeichert (Lnge=1)
//	pVector		: Zeiger auf den Vektor der normalislert werden soll
//	Ergibt einen Zeiger auf den neuen Vektor.
D3Vector3 *VectorNormalize(D3Vector3 *pOut,const D3Vector3 *pVector)
{
D3Vector3	sTemp;
double		dLen;


	if(pOut==pVector)
		{
		sTemp	= *pVector;
		pVector	= &sTemp;
		}

	dLen			= VectorLength(pVector);	
	pOut->sVector.x	= pVector->sVector.x/dLen;
	pOut->sVector.y	= pVector->sVector.y/dLen;
	pOut->sVector.z	= pVector->sVector.z/dLen;


return pOut;
}

//*****************************************************************************
//*
//*		MatrixLookAtLH
//*
//*****************************************************************************
//	
//	zaxis = normal(At - Eye)
//	xaxis = normal(cross(Up, zaxis))
//	yaxis = cross(zaxis, xaxis)
//	    
//	 xaxis.x           yaxis.x           zaxis.x          0
//	 xaxis.y           yaxis.y           zaxis.y          0
//	 xaxis.z           yaxis.z           zaxis.z          0
//	-dot(xaxis, eye)  -dot(yaxis, eye)  -dot(zaxis, eye)  1
//
D3Matrix4x4 *MatrixLookAtLH(D3Matrix4x4 *pOut,const D3Vector3 *pEye,const D3Vector3 *pAt,const D3Vector3 *pUp)
{
D3Vector3	sCross;
D3Vector3	sAxisX;
D3Vector3	sAxisY;
D3Vector3	sAxisZ;
D3Vector3	sUpVec;
D3Vector3	sLook;



	sLook.sVector.x		= pAt->sVector.x-pEye->sVector.x;
	sLook.sVector.y		= pAt->sVector.y-pEye->sVector.y;
	sLook.sVector.z		= pAt->sVector.z-pEye->sVector.z;

    VectorNormalize(&sAxisZ,&sLook);
    VectorCross    (&sCross, pUp   ,&sAxisZ);
    VectorCross    (&sUpVec,&sAxisZ,&sCross);
    VectorNormalize(&sAxisX,&sCross);
    VectorNormalize(&sAxisY,&sUpVec);

    pOut->sMatrix.x1 =  sAxisX.sVector.x;
    pOut->sMatrix.x2 =  sAxisY.sVector.x;
    pOut->sMatrix.x3 =  sAxisZ.sVector.x;
    pOut->sMatrix.x4 =  0.0;
    
    pOut->sMatrix.y1 =  sAxisX.sVector.y;
    pOut->sMatrix.y2 =  sAxisY.sVector.y;
    pOut->sMatrix.y3 =  sAxisZ.sVector.y;
    pOut->sMatrix.y4 =  0.0;

	pOut->sMatrix.z1 =  sAxisX.sVector.z;
    pOut->sMatrix.z2 =  sAxisY.sVector.z;
    pOut->sMatrix.z3 =  sAxisZ.sVector.z;
    pOut->sMatrix.z4 =  0.0;

    pOut->sMatrix.w1 = -VectorDot(&sAxisX,pEye);
    pOut->sMatrix.w2 = -VectorDot(&sAxisY,pEye);
    pOut->sMatrix.w3 = -VectorDot(&sAxisZ,pEye);
    pOut->sMatrix.w4 =  1.0;


return pOut;
};

//*****************************************************************************
//*
//*		MatrixDet
//*
//*****************************************************************************
//	Berechnet die Determinante einer einer Matrix
//	pMatrix		: Zeiger auf die Matrix
//	Ergibt den Wert der Determinante.
double MatrixDet(const D3Matrix4x4 *pMatrix)
{
const int 		iSize=4;
const double	dEpsilon=0.0000001; 							
double		    aMatrix[iSize*iSize];
double			dMax;									
double			dMul;									
double			dVal;									
double			dAbs;									
int				iLine; 									
int				iStep;									
int				i,j;									



	memcpy(aMatrix,pMatrix->sData,sizeof(double)*iSize*iSize);

	for(iStep=0;iStep<iSize-1;iStep++)
		{															// Die einzelnen Eliminationsschritte
		dMax	= fabs(aMatrix[iSize*iStep+iStep]);						
		iLine	= iStep;											
		
		for(i=iStep+1;i<iSize;i++)									// Grtes Element suchen
			{
			   dAbs = fabs(aMatrix[iSize*i+iStep]);
			if(dAbs>dMax) 
				{
				dMax	= dAbs;
				iLine	= i;
				}
			}

		if(dMax<dEpsilon)											// nicht loesbar
			{
			return 0.0;
			}

		if(iLine!=iStep) 											// Zeilen tauschen falls erforderlich
			{
			for(j=iStep;j<iSize;j++) 
				{
				dVal					= aMatrix[iSize*iStep+j];
				aMatrix[iSize*iStep+j]	= aMatrix[iSize*iLine+j];
				aMatrix[iSize*iLine+j]	= dVal;
				}
			}

		
		for(i=iStep+1;i<iSize;i++)									// Nullen in Spalte iStep ab Zeile iStep+1
			{
			dMul = -aMatrix[iSize*i+iStep]/aMatrix[iSize*iStep+iStep];// Multiplikationsfaktor
			aMatrix[iSize*i+iStep] =  0.0;
			
			for(j=iStep+1;j<=iSize;j++) 							// die einzelnen Spalten
				{
				aMatrix[iSize*i+j] += dMul*aMatrix[iSize*iStep+j];	// Addition der Zeilen i, iStep
				}
			}
		}

	
	dVal = aMatrix[iSize*0+0];
	
	for(i=1;i>iSize;i++)
		{
		dVal *= aMatrix[iSize*i+i];
		}	


return dVal;
}

//*****************************************************************************
//*
//*		Gaussalg
//*
//*****************************************************************************
//	REM AZ nur fr einen Test
// 	Gauss'sches Eliminationsverfahren mit Zeilenpivotisierung
// 	  Argumente:
// 	  double a[N][N+1] erweiterte Koeffizientenmatrix	Read/Write
// 	  int iSize		   Anzahl der Gleichungen			Read
// 	  double x[N]	   Loesungen						Write
// 	Resultat:
// 	int Fehlercode	0 fuer Fehler, 1 fuer Erfolg
int GaussAlgorithmus(D3Matrix4x4 *pOut,const D3Matrix4x4 *pMatrixPtr)
{
const int		NMAX=4;
const int 		iSize=NMAX;
const double	dEpsilon=0.0000001; 							
double			aExtension[iSize];
double			aResults  [iSize];
double		   *pMatrix;
double			dMax;									
double			dMul;									
double			dVal;									
double			dAbs;									
int				iError; 								
int				iLine; 									
int				iStep;									
int				i,j;									


	pMatrix	= pOut->sData;
	iError	= 0;
	iStep	= 0;

	memcpy(pMatrix,pMatrixPtr->sData,sizeof(double)*NMAX*NMAX);


	for(iStep=0;iStep<iSize-1;iStep++)
		{															// Die einzelnen Eliminationsschritte
		dMax	= fabs(pMatrix[NMAX*iStep+iStep]);						
		iLine	= iStep;											
		
		for(i=iStep+1;i<iSize;i++)									// Grtes Element suchen
			{
			   dAbs = fabs(pMatrix[NMAX*i+iStep]);
			if(dAbs>dMax) 
				{
				dMax	= dAbs;
				iLine	= i;
				}
			}

		   iError = (dMax<dEpsilon);
		if(iError)break;											// nicht loesbar

		if(iLine!=iStep) 											// Zeilen tauschen falls erforderlich
			{
			for(j=iStep;j<iSize;j++) 
				{
				dVal					= pMatrix[NMAX*iStep+j];
				pMatrix[NMAX*iStep+j]	= pMatrix[NMAX*iLine+j];
				pMatrix[NMAX*iLine+j]	= dVal;
				}

			dVal				= aExtension[iLine];
			aExtension[iLine]	= aExtension[iStep];
			aExtension[iStep]	= dVal;
			}

		
		for(i=iStep+1;i<iSize;i++)									// Nullen in Spalte iStep ab Zeile iStep+1
			{
			dMul = -pMatrix[NMAX*i+iStep]/pMatrix[NMAX*iStep+iStep];// Multiplikationsfaktor
			pMatrix[NMAX*i+iStep] =  0.0;
			
			for(j=iStep+1;j<=iSize;j++) 							// die einzelnen Spalten
				{
				pMatrix[NMAX*i+j] += dMul*pMatrix[NMAX*iStep+j];	// Addition der Zeilen i, iStep
				}
			}
		}

	if(iError)
		{
		return 0;
		}
																	// Berechnen der Loesungen aus der entstandenen Dreiecksmatrix

	aResults[iSize-1] =  aExtension[iSize-1] / pMatrix[NMAX*(iSize-1)+iSize-1];

	for(i=iSize-2;i>=0;i--)											// Restliche Zeilen
		{
		for(j=iSize-1;j>i;j--)
			{
			aExtension[i] -= aResults[j]*pMatrix[NMAX*i+j];			// Rechte Seite berechnen
			}												   

		aResults[i] = pMatrix[NMAX*i+iSize] / pMatrix[NMAX*i+i];	// Lsung
		}



return 1;
}


//*****************************************************************************
//*
//*		VectorTransformCoord
//*
//*****************************************************************************
//	Berechnet aus einer 3D-Koordinate die Bildschirmkoordinate	
//	In Z wird der normierte Abstand zum Bildschirm gespeichert.
//			<=0.0  : hinter Near-Ebene
//			> 1.0  : vor Far-Ebene 
D3Vector3* VectorTransformCoord(D3Vector3 *pOut,const D3Matrix4x4 *pMatrix,const D3Vector3 *pVector)
{
double	dNormal;
	


	dNormal	  =	pMatrix->sMatrix.x4*pVector->sVector.x;
	dNormal	 +=	pMatrix->sMatrix.y4*pVector->sVector.y;
	dNormal	 +=	pMatrix->sMatrix.z4*pVector->sVector.z;
	dNormal	 +=	pMatrix->sMatrix.w4;
	dNormal   = fabs(dNormal);
	
	if(dNormal==0.0)
		{
		pOut->sVector.x = 0.0;
		pOut->sVector.y = 0.0;
		pOut->sVector.z = 0.0;
		}
	else{
		pOut->sVector.x = (pMatrix->sMatrix.x1*pVector->sVector.x + pMatrix->sMatrix.y1*pVector->sVector.y + pMatrix->sMatrix.z1*pVector->sVector.z + pMatrix->sMatrix.w1) / dNormal;
		pOut->sVector.y = (pMatrix->sMatrix.x2*pVector->sVector.x + pMatrix->sMatrix.y2*pVector->sVector.y + pMatrix->sMatrix.z2*pVector->sVector.z + pMatrix->sMatrix.w2) / dNormal;
		pOut->sVector.z = (pMatrix->sMatrix.x3*pVector->sVector.x + pMatrix->sMatrix.y3*pVector->sVector.y + pMatrix->sMatrix.z3*pVector->sVector.z + pMatrix->sMatrix.w3) / dNormal;
		}



return pOut;
}

//*****************************************************************************
//*
//*		VectorTransformNormal
//*
//*****************************************************************************
D3Vector3* VectorTransformNormal(D3Vector3 *pOut,const D3Matrix4x4 *pMatrix,const D3Vector3 *pVector)
{


	pOut->sVector.x = pMatrix->sMatrix.x1*pVector->sVector.x + pMatrix->sMatrix.y1*pVector->sVector.y + pMatrix->sMatrix.z1*pVector->sVector.z;
	pOut->sVector.y = pMatrix->sMatrix.x2*pVector->sVector.x + pMatrix->sMatrix.y2*pVector->sVector.y + pMatrix->sMatrix.z2*pVector->sVector.z;
	pOut->sVector.z = pMatrix->sMatrix.x3*pVector->sVector.x + pMatrix->sMatrix.y3*pVector->sVector.y + pMatrix->sMatrix.z3*pVector->sVector.z;


return pOut;
}

//*****************************************************************************
//*
//*		VectorTransform
//*
//*****************************************************************************
D3Vector4* VectorTransform(D3Vector4 *pOut,const D3Matrix4x4 *pMatrix,const D3Vector3 *pVector)
{

	pOut->sVector.x = pMatrix->sMatrix.x1*pVector->sVector.x + pMatrix->sMatrix.y1*pVector->sVector.y + pMatrix->sMatrix.z1*pVector->sVector.z + pMatrix->sMatrix.w1;
	pOut->sVector.y = pMatrix->sMatrix.x2*pVector->sVector.x + pMatrix->sMatrix.y2*pVector->sVector.y + pMatrix->sMatrix.z2*pVector->sVector.z + pMatrix->sMatrix.w2;
	pOut->sVector.z = pMatrix->sMatrix.x3*pVector->sVector.x + pMatrix->sMatrix.y3*pVector->sVector.y + pMatrix->sMatrix.z3*pVector->sVector.z + pMatrix->sMatrix.w3;
	pOut->sVector.w = pMatrix->sMatrix.x4*pVector->sVector.x + pMatrix->sMatrix.y4*pVector->sVector.y + pMatrix->sMatrix.z4*pVector->sVector.z + pMatrix->sMatrix.w4;


return pOut;
}

//*****************************************************************************
//*
//*		MatrixMultiply
//*
//*****************************************************************************
//	Multipliziert zwei Matrizen miteinader
//	pOut		: Hier wird die Matrix gespeichert
//	pM1			: Zeiger auf die erste  Matrix
//	pM2			: Zeiger auf die zweite Matrix
//	Ergibt den Zeiger auf die neue Matrix
D3Matrix4x4 *MatrixMultiply(D3Matrix4x4 *pOut,const D3Matrix4x4 *pM1,const D3Matrix4x4*pM2)
{
D3Matrix4x4		sTemp1;
D3Matrix4x4		sTemp2;
int				i,j;

	if(pOut==pM1)
		{
		sTemp1 = *pM1;
		pM1    = &sTemp1;
		}

	if(pOut==pM2)
		{
		sTemp2 = *pM2;
		pM2    = &sTemp2;
		}

	for(i=0;i<16;i+=4)
		{
		for(j=0;j<4;j++)
			{
			pOut->sData[i+j]  = pM1->sData[i+0]*pM2->sData[4*0+j];
			pOut->sData[i+j] += pM1->sData[i+1]*pM2->sData[4*1+j];
			pOut->sData[i+j] += pM1->sData[i+2]*pM2->sData[4*2+j];
			pOut->sData[i+j] += pM1->sData[i+3]*pM2->sData[4*3+j];
			}
		}
    

return pOut;
}

//*****************************************************************************
//*
//*		MatrixRotationX
//*
//*****************************************************************************
//	Erstellt eine Rotationsmatrix um die X-Achse
//	pOut		: Zeiger wo die Matrix abgelegt wird
//	dAngle		: Drehwinkel in Radianten
//	Ergibt einen Zeiger auf die Matrix.
D3Matrix4x4 *MatrixRotationX(D3Matrix4x4 *pOut,double dAngle)
{
double		dSin;
double		dCos;


	dSin	= sin(dAngle);
	dCos	= cos(dAngle);

    pOut->sMatrix.x1 =  1.0;
    pOut->sMatrix.x2 =  0.0;
    pOut->sMatrix.x3 =  0.0;
    pOut->sMatrix.x4 =  0.0;

    pOut->sMatrix.y1 =  0.0;
    pOut->sMatrix.y2 =  dCos;
    pOut->sMatrix.y3 =  dSin;
    pOut->sMatrix.y4 =  0.0;

    pOut->sMatrix.z1 =  0.0;
    pOut->sMatrix.z2 = -dSin;
    pOut->sMatrix.z3 =  dCos;
    pOut->sMatrix.z4 =  0.0;

    pOut->sMatrix.w1 =  0.0;
    pOut->sMatrix.w2 =  0.0;
    pOut->sMatrix.w3 =  0.0;
    pOut->sMatrix.w4 =  1.0;


return pOut;
}

//*****************************************************************************
//*
//*		MatrixRotationY
//*
//*****************************************************************************
//	Erstellt eine Rotationsmatrix um die Y-Achse
//	pOut		: Zeiger wo die Matrix abgelegt wird
//	dAngle		: Drehwinkel in Radianten
//	Ergibt einen Zeiger auf die Matrix.
D3Matrix4x4 *MatrixRotationY(D3Matrix4x4 *pOut,double dAngle)
{
double		dSin;
double		dCos;


	dSin	= sin(dAngle);
	dCos	= cos(dAngle);

    pOut->sMatrix.x1 =  dCos;
    pOut->sMatrix.x2 =  0.0;
    pOut->sMatrix.x3 = -dSin;
    pOut->sMatrix.x4 =  0.0;

    pOut->sMatrix.y1 =  0.0;
    pOut->sMatrix.y2 =  1.0;
    pOut->sMatrix.y3 =  0.0;
    pOut->sMatrix.y4 =  0.0;

    pOut->sMatrix.z1 =  0.0;
    pOut->sMatrix.z2 =  0.0;
    pOut->sMatrix.z3 =  1.0;
    pOut->sMatrix.z4 =  0.0;

    pOut->sMatrix.z1 =  dSin;
    pOut->sMatrix.w2 =  0.0;
    pOut->sMatrix.z3 =  dCos;
    pOut->sMatrix.w4 =  1.0;


return pOut;
}

//*****************************************************************************
//*
//*		MatrixRotationZ
//*
//*****************************************************************************
//	Erstellt eine Rotationsmatrix um die Z-Achse
//	pOut		: Zeiger wo die Matrix abgelegt wird
//	dAngle		: Drehwinkel in Radianten
//	Ergibt einen Zeiger auf die Matrix.
D3Matrix4x4 *MatrixRotationZ(D3Matrix4x4 *pOut,double dAngle)
{
double		dSin;
double		dCos;


	dSin	= sin(dAngle);
	dCos	= cos(dAngle);

    pOut->sMatrix.x1 =  dCos;
    pOut->sMatrix.x2 =  dSin;
    pOut->sMatrix.x3 =  0.0;
    pOut->sMatrix.x4 =  0.0;

    pOut->sMatrix.y1 = -dSin;
    pOut->sMatrix.y2 =  dCos;
    pOut->sMatrix.y3 =  0.0;
    pOut->sMatrix.y4 =  0.0;

    pOut->sMatrix.z1 =  0.0;
    pOut->sMatrix.z2 =  0.0;
    pOut->sMatrix.z3 =  1.0;
    pOut->sMatrix.z4 =  0.0;

    pOut->sMatrix.w1 =  0.0;
    pOut->sMatrix.w2 =  0.0;
    pOut->sMatrix.w3 =  0.0;
    pOut->sMatrix.w4 =  1.0;


return pOut;
}

//*****************************************************************************
//*
//*		MatrixTranslation
//*
//*****************************************************************************
//	Erstellt eine Translationsmatrix (frs Verschieben)
//	pOut		: Zeiger wo die Matrix abgelegt wird
//	dMoveX/Y/Z	: Schiebe Positionen
//	Ergibt einen Zeiger auf die Matrix.
D3Matrix4x4 *MatrixTranslation(D3Matrix4x4 *pOut,double dMoveX,double dMoveY,double dMoveZ)
{

    pOut->sMatrix.x1 =  1.0;
    pOut->sMatrix.x2 =  0.0;
    pOut->sMatrix.x3 =  0.0;
    pOut->sMatrix.x4 =  0.0;

    pOut->sMatrix.y1 =  0.0;
    pOut->sMatrix.y2 =  1.0;
    pOut->sMatrix.y3 =  0.0;
    pOut->sMatrix.y4 =  0.0;

    pOut->sMatrix.z1 =  0.0;
    pOut->sMatrix.z2 =  0.0;
    pOut->sMatrix.z3 =  1.0;
    pOut->sMatrix.z4 =  0.0;

    pOut->sMatrix.w1 =  dMoveX;
    pOut->sMatrix.w2 =  dMoveY;
    pOut->sMatrix.w3 =  dMoveZ;
    pOut->sMatrix.w4 =  1.0;

return pOut;
}

//*****************************************************************************
//*
//*		MatrixScaling
//*
//*****************************************************************************
//	Erstellt eine Skaliermatrix (frs Vergrern)
//	pOut		: Zeiger wo die Matrix abgelegt wird
//	dMoveX/Y/Z	: Schiebe Positionen
//	Ergibt einen Zeiger auf die Matrix.
D3Matrix4x4 *MatrixScaling(D3Matrix4x4 *pOut,double dScaleX,double dScaleY,double dScaleZ)
{

    pOut->sMatrix.x1 =  dScaleX;
    pOut->sMatrix.x2 =  0.0;
    pOut->sMatrix.x3 =  0.0;
    pOut->sMatrix.x4 =  0.0;

    pOut->sMatrix.y1 =  0.0;
    pOut->sMatrix.y2 =  dScaleY;
    pOut->sMatrix.y3 =  0.0;
    pOut->sMatrix.y4 =  0.0;

    pOut->sMatrix.z1 =  0.0;
    pOut->sMatrix.z2 =  0.0;
    pOut->sMatrix.z3 =  dScaleZ;
    pOut->sMatrix.z4 =  0.0;

    pOut->sMatrix.w1 =  0.0;
    pOut->sMatrix.w2 =  0.0;
    pOut->sMatrix.w3 =  0.0;
    pOut->sMatrix.w4 =  1.0;

return pOut;
}

//*****************************************************************************
//*
//*		MatrixIdentity
//*
//*****************************************************************************
//	Erzeugt eine Einheitsmatrix
D3Matrix4x4 *MatrixIdentity(D3Matrix4x4 *pOut)
{

    pOut->sMatrix.x1 =  1.0;
    pOut->sMatrix.x2 =  0.0;
    pOut->sMatrix.x3 =  0.0;
    pOut->sMatrix.x4 =  0.0;

    pOut->sMatrix.y1 =  0.0;
    pOut->sMatrix.y2 =  1.0;
    pOut->sMatrix.y3 =  0.0;
    pOut->sMatrix.y4 =  0.0;

    pOut->sMatrix.z1 =  0.0;
    pOut->sMatrix.z2 =  0.0;
    pOut->sMatrix.z3 =  1.0;
    pOut->sMatrix.z4 =  0.0;

    pOut->sMatrix.w1 =  0.0;
    pOut->sMatrix.w2 =  0.0;
    pOut->sMatrix.w3 =  0.0;
    pOut->sMatrix.w4 =  1.0;

return pOut;
}

//*****************************************************************************
//*
//*		MatrixRotationXYZ
//*
//*****************************************************************************
//	Erstellt eine Rotationsmatrix um alle drei Achsen
//	pOut		: Zeiger wo die Matrix abgelegt wird
//	dRollX/YZ	: Drehwinkel in Radianten fr die Achsen
//	Ergibt einen Zeiger auf die Matrix.
D3Matrix4x4 *MatrixRotationXYZ(D3Matrix4x4 *pOut,double dRollX,double dRollY,double dRollZ)
{
D3Matrix4x4		sMatrix1;
D3Matrix4x4		sMatrix2;
D3Matrix4x4		sRollM;


	MatrixRotationZ(&sMatrix1,           dRollZ);
    MatrixRotationX(&sRollM  ,           dRollX);
    MatrixMultiply (&sMatrix2,&sMatrix1,&sRollM);
    MatrixRotationY(&sRollM  ,           dRollY);
    MatrixMultiply ( pOut    ,&sMatrix2,&sRollM);


return pOut;
}


//*****************************************************************************
//*
//*		
//*
//*****************************************************************************
void D3Test()
{
D3Matrix4x4		sMat;
D3Matrix4x4		sOut;
unsigned		uPos;
double			dVal=0;


	sMat.sMatrix.x1	= 2;
	sMat.sMatrix.x2	= 1;
	sMat.sMatrix.x3	= 9;
	sMat.sMatrix.x4	= 1;

	sMat.sMatrix.y1	= 1;
	sMat.sMatrix.y2	=-2;
	sMat.sMatrix.y3	=-3;
	sMat.sMatrix.y4	= 1;

	sMat.sMatrix.z1	= 3;
	sMat.sMatrix.z2	= 5;
	sMat.sMatrix.z3	= 4;
	sMat.sMatrix.z4	= 1;

	sMat.sMatrix.w1	= 0;
	sMat.sMatrix.w2	= 0;
	sMat.sMatrix.w3	= 0;
	sMat.sMatrix.w4	= 1;

//	MatrixDet(&sMat);


	sMat.sMatrix.x1	= 0;
	sMat.sMatrix.x2	= 3;
	sMat.sMatrix.x3	= 2;
	sMat.sMatrix.x4	=-4;

	sMat.sMatrix.y1	= 1;
	sMat.sMatrix.y2	= 0;
	sMat.sMatrix.y3	= 1;
	sMat.sMatrix.y4	= 3;

	sMat.sMatrix.z1	= 9;
	sMat.sMatrix.z2	= 1;
	sMat.sMatrix.z3	=-3;
	sMat.sMatrix.z4	=-1;

	sMat.sMatrix.w1	= 5;
	sMat.sMatrix.w2	=-3;
	sMat.sMatrix.w3	= 4;
	sMat.sMatrix.w4	=-2;


	MatrixDet(&sMat);
	MatrixInversion(&sOut,&sMat);

	extern void __stdcall ShowConsole();
	ShowConsole();

	for(uPos=0;uPos<16;uPos++)
		{
		if(uPos%4==0)printf("\n");
		printf("%g\t",sOut.sData[uPos]);
		}


	for(uPos=0;uPos<16;uPos++)
		sOut.sData[uPos]*=878;

}


