//*****************************************************************************
//*
//*
//*		D3Utility.h
//*
//*
//*****************************************************************************
#ifndef 	__D3UTILITY_H__
#define 	__D3UTILITY_H__

#define 	D3_PI			3.1415926535897932384626433832795


typedef	struct
	{
	double		x;
	double		y;
	double		z;
	}Vector3;

typedef	struct
	{
	double		x;
	double		y;
	double		z;
	double		w;
	}Vector4;

typedef	struct
	{
	double		x1;
	double		x2;
	double		x3;
	double		x4;
	double		y1;
	double		y2;
	double		y3;
	double		y4;
	double		z1;
	double		z2;
	double		z3;
	double		z4;
	double		w1;
	double		w2;
	double		w3;
	double		w4;
	}Matrix4x4;


typedef	union
	{
	double		sData[3];
	Vector3		sVector;
	}D3Vector3;

typedef	union
	{
	double		sData[4];
	Vector4		sVector;
	}D3Vector4;

typedef	union
	{
	double		sData[16];
	Matrix4x4	sMatrix;
	}D3Matrix4x4;


double		 VectorLength          (const D3Vector3 *pVector1);
double		 VectorLength          (const D3Vector4 *pVector1);
double		 VectorDot             (const D3Vector3 *pVector1,const D3Vector3 *pVector2);
double		 VectorDot             (const D3Vector4 *pVector1,const D3Vector4 *pVector2);
D3Vector3   *VectorNormalize       (D3Vector3   *pOut,const D3Vector3   *pVector);
D3Vector3   *VectorCross           (D3Vector3   *pOut,const D3Vector3   *pVector1,const D3Vector3 *pVector2);
D3Vector3   *VectorTransformCoord  (D3Vector3   *pOut,const D3Matrix4x4 *pMatrix ,const D3Vector3 *pVector);
D3Vector3   *VectorTransformNormal (D3Vector3   *pOut,const D3Matrix4x4 *pMatrix ,const D3Vector3 *pVector);
D3Vector4   *VectorTransform	   (D3Vector4   *pOut,const D3Matrix4x4 *pMatrix ,const D3Vector3 *pVector);
int			 MatrixInversion       (D3Matrix4x4 *pOut,const D3Matrix4x4 *pMatrix);
D3Matrix4x4 *MatrixPerspectiveFovLH(D3Matrix4x4 *pOut,double dViewAngel,double dAspect,double dClipNear,double dClipFar);
D3Matrix4x4 *MatrixLookAtLH        (D3Matrix4x4 *pOut,const D3Vector3 *pEye,const D3Vector3 *pAt,const D3Vector3 *pUp);
D3Matrix4x4 *MatrixMultiply        (D3Matrix4x4 *pOut,const D3Matrix4x4 *pM1,const D3Matrix4x4*pM2);
D3Matrix4x4 *MatrixRotationXYZ     (D3Matrix4x4 *pOut,double dRollY,double dRollX,double dRollZ);
D3Matrix4x4 *MatrixRotationX       (D3Matrix4x4 *pOut,double dAngle);
D3Matrix4x4 *MatrixRotationY       (D3Matrix4x4 *pOut,double dAngle);
D3Matrix4x4 *MatrixRotationZ       (D3Matrix4x4 *pOut,double dAngle);
D3Matrix4x4 *MatrixTranslation	   (D3Matrix4x4 *pOut,double dMoveX ,double dMoveY ,double dMoveZ);
D3Matrix4x4 *MatrixScaling		   (D3Matrix4x4 *pOut,double dScaleX,double dScaleY,double dScaleZ);
D3Matrix4x4 *MatrixIdentity        (D3Matrix4x4 *pOut);
double		 MatrixDet       (const D3Matrix4x4 *pMatrix);


#endif 	