//*****************************************************************************
//*
//*
//*		PlaneDrawer.h
//*
//*
//*****************************************************************************
#ifndef 	__PLANEDRAWER_H__
#define 	__PLANEDRAWER_H__

#include	"Image/BitmapImage.h"
#include	"D3Utility.h"

#define 	PDR_DOUBLE				0x00000001		// Doppelt so gro Zeichnen fr Glttung
#define 	PDR_QUARD				0x00000002		// Viermal so gro Zeichnen fr Glttung
#define 	PDR_GRID				0x00000004		// Gitter am Hindergrund fr X
#define 	PDR_COLORFIELD			0x00000008		// Farbenrechteck anzeigen
#define 	PDR_COLORBAR			0x00000010		// Farbskala anzeigen
#define 	PDR_LIGHT				0x00000020		// Licht einschalten
#define 	PDR_SHADOW				0x00000040		// Schatten bei Z-Nullachse
#define 	PDR_PLANE				0x00000080		// Hauptebene zeichnen
#define 	PDR_GRIDDOUBLE			0x00000800		// Breites Gitter
#define 	PDR_TEXT_X				0x00000100		// X-Skala der Kurven  zeichnen
#define 	PDR_TEXT_Y				0x00000200		// Y-Skala der Kurve 1 zeichnen
#define 	PDR_TEXT_Z				0x00000400		// Y-Skala der Kurve 2 zeichnen
#define 	PDR_SCALA_X				0x00010000		// X-Skala der Kurven  zeichnen
#define 	PDR_SCALA_Y				0x00020000		// Y-Skala der Kurve 1 zeichnen
#define 	PDR_SCALA_Z				0x00040000		// Y-Skala der Kurve 2 zeichnen
#define 	PDR_IS_LOG_X			0x01000000		// X-Skala der Kurven  ist logarithmisch
#define 	PDR_IS_LOG_Y			0x02000000		// Y-Skala der Kurve 1 ist logarithmisch
#define 	PDR_IS_LOG_Z			0x04000000		// Y-Skala der Kurve 2 ist logarithmisch

#define 	PDR_INDEX_X				0				// Index der X-Scala
#define 	PDR_INDEX_Y				1				// Index der Y-Scala
#define 	PDR_INDEX_Z				2				// Index der Z-Scala


typedef struct
	{
	char		cVarName[16];						// Variabelname
	char		cUnitName[8];						// Einheitenname
	int			iNameSize;							// Breite des Namens
	int			iMode;								// 0=X 1=Y 2=Z -1=keine
	int			iIsLog;								// Logaritmische Skala
	int			iHasText;							// Gibt es eine Beschriftung
	int			iDoubleMark;						// Zweiseitige Marke
	int			iPlaces;							// Stellen fr Beschriftung
	int			iComma;								// Kommastellen fr Beschriftung
	int			iBase;								// Anzeigeeinheit (8=- 9=k 10=M ... , -1=auto)
	int			iStepCnt;							// Schritte in aSteps
	int			iStepAt;							// Bei welchem Schritt wird begonnen
	double		aSteps[8];							// Faktoren fr die Schritte	
	double		dPosNull;							// Position der NULL-Skala (0..1) (<-1=keine)
	double		dOffset;							// Offset fr Punktumrechnung
	double		dStart;								// Wert der ersten unter Skalierung
	double		dCalc;								// Faktor fr Punktumrechnung invers
	double		dFact;								// Faktor fr Punktumrechnung
	double		dMin;								// Anfangswert der Scala
	double		dMax;								// Endwert der Scala
	}PlaneScala;

typedef struct
	{
	double		dLook;								// Von  -1  bis  +1 (Auf Ab) 
	double		dZoom;								// Von  -1  bis  +1 (Zoomen)
	double		dRollX;								// Von -45 bis +45 Grad (Bild drehen)
	double		dRollY;								// Von -45 bis +45 Grad
	double		dRollZ;								// Von -45 bis +45 Grad
	double		dShadow;							// Position der Schattenebene
	int			iShadowPlane;						// Ebene der Schattenebene (0=X 1=Y 2=Z)
	}PlaneParams;

class PlaneData
	{
public:
	PlaneData();
   ~PlaneData();

	double		dFactX;								// Offsets fr Dreiecksberechnung
	double		dFactY;								//		*
	double		dAddZ;								// 		*
	int			iPosX;								//		*
	int			iPosY;								//		*
	int			iMul;								// Faktor fr Koordinaten
	unsigned	uWhite;								// Faktor fr das Weise
	unsigned	uBufSizeZ;							// Gre des Z-Puffers
	double	   *pBufferZ;							// Z-Buffer fr die Zeichnung
	double	   *pGridValues;						// Normierte Gitter-Werte
	D3Vector3  *pGridVects;							// Transformierte Gitter Vektoren
	D3Vector3	sSunVect;							// Normalvector zur Sonne
	PlaneScala	aScala[3];							// Infos ber die Skalen
	double		dZoom;								// Zoomfaktor
	double		dLook;								// Auf/Ab Schauen
	double		dRollX;								// Drehung um die X-Achse in Grad
	double		dRollY;								// Drehung um die Y-Achse in Grad
	double		dRollZ;								// Drehung um die Z-Achse in Grad
	double		dFontScale;							// Skalierung fr Schriftpixel	
	double		dShadow;							// Position des Schattenebene
	double		dStepMul;							// Gitter Unterteilungsfaktor
	unsigned	uOptions;							// Diverse Flags (siehe PDR_???)
	unsigned	uGridSub;							// Jede n'te Unterteilung als Gitter zeichnen
	unsigned	uGridMask;							// Maske fr Gitterlinien
	int			iShadowPlane;						// Position des Schattenebene (0=X 1=Y 2=Z)
	int			iSteps;								// Gitter Unterteilung
	int			iSizeX;								// X-Breite des Bildes
	int			iSizeY;								// Y-Breite des Bildes
	D3Matrix4x4	sMul;								// Transformationsmatrix
	D3Matrix4x4	sDiv;								// Transformationsmatrix invers
	};


int				PlaneInit(PlaneData *pData,BitmapImage *pImage,unsigned uOptions,double *pMinMax,const char *pNames=0,PlaneParams *pParams=0);
void			PlaneDraw(PlaneData *pData,BitmapImage *pImage,double *pPositons);
int				PlaneCoorToVal(PlaneData *pData,int iPosX,int iPosY,double *pValues);
int				PlaneValToCoor(PlaneData *pData,int&iPosX,int&iPosY,double *pValues);



inline  double	PlaneStepToPos(PlaneData *pData,int iStep)
			{
			return (double)iStep*pData->dStepMul-1.0;
			}

inline	double	PlanePosToVal(PlaneData *pData,double dPos,int iNum)
			{
			double	dVal;

			if(pData->aScala[iNum].iIsLog)
					dVal = exp((dPos-pData->aScala[iNum].dOffset)*pData->aScala[iNum].dCalc);	
			else	dVal = dPos*pData->aScala[iNum].dCalc-pData->aScala[iNum].dOffset;

			return dVal;
			}

inline	double	PlaneStepToVal(PlaneData *pData,int iStep,int iNum)
			{
			double	dVal;
			double	dPos;

			dPos	= PlaneStepToPos(pData,iStep);
			dVal	= PlanePosToVal (pData,dPos,iNum);

			return dVal;
			}

inline	double	PlaneValToPos(PlaneData *pData,double dVal,int iNum)
			{
			double	dPos;

			if(pData->aScala[iNum].iIsLog)
					dPos = log(dVal)*pData->aScala[iNum].dFact+pData->aScala[iNum].dOffset;	
			else	dPos = (dVal+pData->aScala[iNum].dOffset) *pData->aScala[iNum].dFact;

			return dPos;
			}


#endif 	

